@@ -1,7 +1,7 @@
 <?php
 /*
  +-------------------------------------------------------------------------+
- | Copyright (C) 2004-2018 The Cacti Group                                 |
+ | Copyright (C) 2004-2020 The Cacti Group                                 |
  |                                                                         |
  | This program is free software; you can redistribute it and/or           |
  | modify it under the terms of the GNU General Public License             |
@@ -23,11 +23,12 @@
 */
 
 include ('./include/auth.php');
-include_once('./lib/api_tree.php');
 include_once('./lib/api_data_source.php');
+include_once('./lib/api_tree.php');
 include_once('./lib/html_tree.php');
-include_once('./lib/utility.php');
+include_once('./lib/poller.php');
 include_once('./lib/template.php');
+include_once('./lib/utility.php');
 
 $ds_actions = array(
 	1 => __('Delete'),
@@ -122,19 +123,38 @@ function form_save() {
 		$save3['local_data_id']         = 0;
 
 		$save3['t_rrd_maximum']         = form_input_validate((isset_request_var('t_rrd_maximum') ? get_nfilter_request_var('t_rrd_maximum') : ''), 't_rrd_maximum', '', true, 3);
+
 		$save3['rrd_maximum']           = form_input_validate(get_nfilter_request_var('rrd_maximum'), 'rrd_maximum', '^(-?([0-9]+(\.[0-9]*)?|[0-9]*\.[0-9]+)([eE][+\-]?[0-9]+)?)|U$', (isset_request_var('t_rrd_maximum') ? true : false), 3);
+
 		$save3['t_rrd_minimum']         = form_input_validate((isset_request_var('t_rrd_minimum') ? get_nfilter_request_var('t_rrd_minimum') : ''), 't_rrd_minimum', '', true, 3);
+
 		$save3['rrd_minimum']           = form_input_validate(get_nfilter_request_var('rrd_minimum'), 'rrd_minimum', '^(-?([0-9]+(\.[0-9]*)?|[0-9]*\.[0-9]+)([eE][+\-]?[0-9]+)?)|U$', (isset_request_var('t_rrd_minimum') ? true : false), 3);
 
 		$save3['rrd_heartbeat']         = $rrd_heartbeat;
 
 		$save3['t_data_source_type_id'] = form_input_validate((isset_request_var('t_data_source_type_id') ? get_nfilter_request_var('t_data_source_type_id') : ''), 't_data_source_type_id', '', true, 3);
+
 		$save3['data_source_type_id']   = form_input_validate(get_request_var('data_source_type_id'), 'data_source_type_id', '^[0-9]+$', true, 3);
+
 		$save3['t_data_source_name']    = form_input_validate((isset_request_var('t_data_source_name') ? get_nfilter_request_var('t_data_source_name') : ''), 't_data_source_name', '', true, 3);
+
 		$save3['data_source_name']      = form_input_validate(get_nfilter_request_var('data_source_name'), 'data_source_name', '^[a-zA-Z0-9_]{1,19}$', (isset_request_var('t_data_source_name') ? true : false), 3);
+
 		$save3['t_data_input_field_id'] = form_input_validate((isset_request_var('t_data_input_field_id') ? get_nfilter_request_var('t_data_input_field_id') : ''), 't_data_input_field_id', '', true, 3);
+
 		$save3['data_input_field_id']   = form_input_validate((isset_request_var('data_input_field_id') ? get_nfilter_request_var('data_input_field_id') : '0'), 'data_input_field_id', '', true, 3);
 
+		if ($save3['rrd_minimum'] != 'U' && $save3['rrd_maximum'] != 'U') {
+			if ($save3['rrd_minimum'] >= $save3['rrd_maximum']) {
+				raise_message(43);
+
+				$_SESSION['sess_error_fields']['rrd_maximum'] = 'rrd_maximum';
+
+				header('Location: data_templates.php?header=false&action=template_edit&id=' . (empty($data_template_id) ? get_request_var('data_template_id') : $data_template_id) . (isempty_request_var('current_rrd') ? '' : '&view_rrd=' . (get_nfilter_request_var('current_rrd') ? get_nfilter_request_var('current_rrd') : get_request_var('data_template_rrd_id'))));
+				exit;
+			}
+		}
+
 		/* ok, first pull out all 'input' values so we know how much to save */
 		$input_fields = db_fetch_assoc_prepared("SELECT id, input_output, regexp_match,
 			allow_nulls, type_code, data_name
@@ -144,7 +164,7 @@ function form_save() {
 			array(get_request_var('data_input_id')));
 
 		/* pass 1 for validation */
-		if (sizeof($input_fields)) {
+		if (cacti_sizeof($input_fields)) {
 			foreach ($input_fields as $input_field) {
 				$form_value = 'value_' . $input_field['data_name'];
 
@@ -214,7 +234,7 @@ function form_save() {
 					WHERE data_template_data_id = ?',
 					array($data_template_data_id));
 
-				if (sizeof($input_fields)) {
+				if (cacti_sizeof($input_fields)) {
 					foreach ($input_fields as $input_field) {
 						$form_value = 'value_' . $input_field['data_name'];
 
@@ -303,7 +323,7 @@ function form_actions() {
 				db_execute('UPDATE data_template_rrd set local_data_template_rrd_id=0,data_template_id=0 WHERE ' . array_to_sql_or($selected_items, 'data_template_id'));
 				db_execute('UPDATE data_local set data_template_id=0 WHERE ' . array_to_sql_or($selected_items, 'data_template_id'));
 			} elseif (get_nfilter_request_var('drp_action') == '2') { // duplicate
-				for ($i=0;($i<count($selected_items));$i++) {
+				for ($i=0;($i<cacti_count($selected_items));$i++) {
 					api_duplicate_data_source(0, $selected_items[$i], get_nfilter_request_var('title_format'));
 				}
 			} elseif (get_nfilter_request_var('drp_action') == '3') { // change data source profile
@@ -313,7 +333,7 @@ function form_actions() {
 					array(get_filter_request_var('data_source_profile_id')));
 
 				if (!empty($step)) {
-					for ($i=0;($i<count($selected_items));$i++) {
+					for ($i=0;($i<cacti_count($selected_items));$i++) {
 						db_execute_prepared('UPDATE data_template_data
 							SET data_source_profile_id = ?,
 							rrd_step = ?
@@ -351,7 +371,7 @@ function form_actions() {
 
 	html_start_box($ds_actions{get_request_var('drp_action')}, '60%', '', '3', 'center', '');
 
-	if (isset($ds_array) && sizeof($ds_array)) {
+	if (isset($ds_array) && cacti_sizeof($ds_array)) {
 		if (get_request_var('drp_action') == '1') { // delete
 			print "<tr>
 				<td class='textArea'>
@@ -360,7 +380,7 @@ function form_actions() {
 				</td>
 			</tr>\n";
 
-			$save_html = "<input type='button' value='" . __esc('Cancel') . "' onClick='cactiReturnTo()'>&nbsp;<input type='submit' value='" . __esc('Continue') . "' title='" . __esc('Delete Data Template(s)') . "'>";
+			$save_html = "<input type='button' class='ui-button ui-corner-all ui-widget' value='" . __esc('Cancel') . "' onClick='cactiReturnTo()'>&nbsp;<input type='submit' class='ui-button ui-corner-all ui-widget' value='" . __esc('Continue') . "' title='" . __esc('Delete Data Template(s)') . "'>";
 		} elseif (get_request_var('drp_action') == '2') { // duplicate
 			print "<tr>
 				<td class='textArea'>
@@ -370,7 +390,7 @@ function form_actions() {
 				</td>
 			</tr>\n";
 
-			$save_html = "<input type='button' value='" . __esc('Cancel') . "' onClick='cactiReturnTo()'>&nbsp;<input type='submit' value='" . __esc('Continue') . "' title='" . __esc('Duplicate Data Template(s)') . "'>";
+			$save_html = "<input type='button' class='ui-button ui-corner-all ui-widget' value='" . __esc('Cancel') . "' onClick='cactiReturnTo()'>&nbsp;<input type='submit' class='ui-button ui-corner-all ui-widget' value='" . __esc('Continue') . "' title='" . __esc('Duplicate Data Template(s)') . "'>";
 		} elseif (get_request_var('drp_action') == '3') { // change profile
 			print "<tr>
 				<td class='textArea'>
@@ -386,11 +406,12 @@ function form_actions() {
 				</td>
 			</tr>\n";
 
-			$save_html = "<input type='button' value='" . __esc('Cancel') . "' onClick='cactiReturnTo()'>&nbsp;<input type='submit' value='" . __esc('Continue') . "' title='" . __esc('Change Data Source Profile') . "'>";
+			$save_html = "<input type='button' class='ui-button ui-corner-all ui-widget' value='" . __esc('Cancel') . "' onClick='cactiReturnTo()'>&nbsp;<input type='submit' class='ui-button ui-corner-all ui-widget' value='" . __esc('Continue') . "' title='" . __esc('Change Data Source Profile') . "'>";
 		}
 	} else {
-		print "<tr><td class='even'><span class='textError'>" . __('You must select at least one data template.') . "</span></td></tr>\n";
-		$save_html = "<input type='button' value='" . __esc('Return') . "' onClick='cactiReturnTo()'>";
+		raise_message(40);
+		header('Location: data_templates.php?header=false');
+		exit;
 	}
 
 	print "<tr>
@@ -425,7 +446,7 @@ function template_rrd_remove() {
 		OR id = ?',
 		array(get_request_var('id'), get_request_var('id')));
 
-	if (sizeof($children)) {
+	if (cacti_sizeof($children)) {
 		foreach ($children as $item) {
 			db_execute_prepared('DELETE FROM data_template_rrd WHERE id = ?', array($item['id']));
 			db_execute_prepared('DELETE FROM snmp_query_graph_rrd WHERE data_template_rrd_id = ?', array($item['id']));
@@ -466,7 +487,7 @@ function template_rrd_add() {
 
 	db_execute_prepared('INSERT IGNORE INTO data_template_rrd
 		(hash, data_template_id, rrd_maximum, rrd_minimum, rrd_heartbeat, data_source_type_id, data_source_name)
-	    VALUES (?, ?, 0, 0, 600, 1, ?)', array($hash, get_request_var('id'), $dsname));
+	    VALUES (?, ?, "U", 0, 600, 1, ?)', array($hash, get_request_var('id'), $dsname));
 
 	$data_template_rrd_id = db_fetch_insert_id();
 
@@ -477,7 +498,7 @@ function template_rrd_add() {
 		AND local_data_id > 0',
 		array(get_request_var('id')));
 
-	if (sizeof($children)) {
+	if (cacti_sizeof($children)) {
 		foreach ($children as $item) {
 			db_execute_prepared('INSERT IGNORE INTO data_template_rrd
 				(local_data_template_rrd_id, local_data_id, data_template_id, rrd_maximum, rrd_minimum, rrd_heartbeat, data_source_type_id, data_source_name)
@@ -497,11 +518,36 @@ function template_edit() {
 	get_filter_request_var('view_rrd');
 	/* ==================================================== */
 
-	if (!isempty_request_var('id')) {
-		$template_data = db_fetch_row_prepared('SELECT * FROM data_template_data WHERE data_template_id = ? AND local_data_id = 0', array(get_request_var('id')));
-		$template = db_fetch_row_prepared('SELECT * FROM data_template WHERE id = ?', array(get_request_var('id')));
+	$isSNMPget = false;
 
-		$header_label = __('Data Templates [edit: %s]', html_escape($template['name']));
+	if (!isempty_request_var('id')) {
+		$template_data = db_fetch_row_prepared('SELECT dtd.*,
+			SUM(CASE WHEN dl.data_template_id = ? THEN 1 ELSE 0 END) AS data_sources
+			FROM data_template_data AS dtd
+			LEFT JOIN data_local AS dl
+			ON dl.id=dtd.local_data_id
+			WHERE dtd.data_template_id = ?
+			HAVING dtd.local_data_id=0',
+			array(get_request_var('id'), get_request_var('id')));
+
+		$template = db_fetch_row_prepared('SELECT *
+			FROM data_template
+			WHERE id = ?',
+			array(get_request_var('id')));
+
+		if (cacti_sizeof($template_data)) {
+			$snmp_data = db_fetch_row_prepared('SELECT *
+				FROM data_input
+				WHERE hash="3eb92bb845b9660a7445cf9740726522"
+				AND id = ?',
+				array($template_data['data_input_id']));
+
+			if (cacti_sizeof($snmp_data)) {
+				$isSNMPget = true;
+			}
+		}
+
+		$header_label = __esc('Data Templates [edit: %s]', $template['name']);
 
 		?>
 		<table style='width:100%'>
@@ -584,7 +630,12 @@ function template_edit() {
 
 	/* fetch ALL rrd's for this data source */
 	if (!isempty_request_var('id')) {
-		$template_data_rrds = db_fetch_assoc_prepared('SELECT id, data_source_name FROM data_template_rrd WHERE data_template_id = ? AND local_data_id = 0 ORDER BY data_source_name', array(get_request_var('id')));
+		$template_data_rrds = db_fetch_assoc_prepared('SELECT id, data_source_name
+			FROM data_template_rrd
+			WHERE data_template_id = ?
+			AND local_data_id = 0
+			ORDER BY data_source_name',
+			array(get_request_var('id')));
 	}
 
 	/* select the first "rrd" of this data source by default */
@@ -594,17 +645,20 @@ function template_edit() {
 
 	/* get more information about the rrd we chose */
 	if (!isempty_request_var('view_rrd')) {
-		$template_rrd = db_fetch_row_prepared('SELECT * FROM data_template_rrd WHERE id = ?', array(get_request_var('view_rrd')));
+		$template_rrd = db_fetch_row_prepared('SELECT *
+			FROM data_template_rrd
+			WHERE id = ?',
+			array(get_request_var('view_rrd')));
 	}
 
 	$i = 0;
 	if (isset($template_data_rrds)) {
-		if (sizeof($template_data_rrds) > 1) {
+		if (cacti_sizeof($template_data_rrds) > 1) {
 			/* draw the data source tabs on the top of the page */
 			print "<div class='tabs' style='float:left;'><nav><ul role='tablist'>\n";
 
 			foreach ($template_data_rrds as $template_data_rrd) {
-				print "<li class='subTab'><a " . (($template_data_rrd['id'] == get_request_var('view_rrd')) ? "class='pic selected'" : "class='pic'") . " href='" . html_escape('data_templates.php?action=template_edit&id=' . get_request_var('id') . '&view_rrd=' . $template_data_rrd['id']) . "'>" . ($i+1) . ": " . html_escape($template_data_rrd['data_source_name']) . "</a><a class='deleteMarker fa fa-remove' title='" . __esc('Delete') . "' href='" . html_escape('data_templates.php?action=rrd_remove&id=' . $template_data_rrd['id'] . '&data_template_id=' . get_request_var('id')) . "'></a></li>\n";
+				print "<li class='subTab'><a " . (($template_data_rrd['id'] == get_request_var('view_rrd')) ? "class='pic selected'" : "class='pic'") . " href='" . html_escape('data_templates.php?action=template_edit&id=' . get_request_var('id') . '&view_rrd=' . $template_data_rrd['id']) . "'>" . ($i+1) . ": " . html_escape($template_data_rrd['data_source_name']) . "</a>" . ($template_data['data_sources'] == 0 ? "<a class='pic deleteMarker fa fa-times' title='" . __esc('Delete') . "' href='" . html_escape('data_templates.php?action=rrd_remove&id=' . $template_data_rrd['id'] . '&data_template_id=' . get_request_var('id')) . "'></a>":"<a class='deleteMarkerDisabled fa fa-times' href='#' title='" . __esc('Data Templates in use can not be modified') . "'></a>") . "</li>\n";
 
 				$i++;
 			}
@@ -612,20 +666,40 @@ function template_edit() {
 			print "
 			</ul></nav>\n
 			</div>\n";
-		} elseif (sizeof($template_data_rrds) == 1) {
+		} elseif (cacti_sizeof($template_data_rrds) == 1) {
 			set_request_var('view_rrd', $template_data_rrds[0]['id']);
 		}
 	}
 
-	html_start_box(__('Data Source Item [%s]', (isset($template_rrd) ? html_escape($template_rrd['data_source_name']) : '')), '100%', true, '0', 'center', (!isempty_request_var('id') ? 'data_templates.php?action=rrd_add&id=' . get_request_var('id'):''), __('New'));
+	if (get_request_var('id') > 0) {
+		$readOnly = db_fetch_cell_prepared('SELECT *
+			FROM data_local
+			WHERE data_template_id = ?',
+			array(get_request_var('id')));
+	} else {
+		$readOnly = false;
+	}
+
+	if (!$isSNMPget && !$readOnly) {
+		html_start_box(__('Data Source Item [%s]', (isset($template_rrd) ? html_escape($template_rrd['data_source_name']) : '')), '100%', true, '0', 'center', (!isempty_request_var('id') ? 'data_templates.php?action=rrd_add&id=' . get_request_var('id'):''), __('New'));
+	} else {
+		html_start_box(__('Data Source Item [%s]', (isset($template_rrd) ? html_escape($template_rrd['data_source_name']) : '')), '100%', true, '0', 'center', '', '');
+	}
 
 	/* data input fields list */
-	if ((empty($template_data['data_input_id'])) ||
-		((db_fetch_cell_prepared('SELECT type_id FROM data_input WHERE id = ?', array($template_data['data_input_id'])) != '1') &&
-		(db_fetch_cell_prepared('SELECT type_id FROM data_input WHERE id = ?', array($template_data['data_input_id'])) != '5'))) {
+	if (empty($template_data['data_input_id'])) {
 		unset($struct_data_source_item['data_input_field_id']);
 	} else {
-		$struct_data_source_item['data_input_field_id']['sql'] = "SELECT id,CONCAT(data_name,' - ',name) AS name FROM data_input_fields WHERE data_input_id=" . $template_data['data_input_id'] . " AND input_output='out' AND update_rra='on' ORDER BY data_name,name";
+		$input_type = db_fetch_cell_prepared('SELECT type_id
+			FROM data_input
+			WHERE id = ?',
+			array($template_data['data_input_id']));
+
+		if ($input_type != 1 && $input_type != 5) {
+			unset($struct_data_source_item['data_input_field_id']);
+		} else {
+			$struct_data_source_item['data_input_field_id']['sql'] = "SELECT id, CONCAT(data_name, ' - ', name) AS name FROM data_input_fields WHERE data_input_id=" . $template_data['data_input_id'] . " AND input_output='out' AND update_rra='on' ORDER BY data_name, name";
+		}
 	}
 
 	$form_array = array();
@@ -664,19 +738,25 @@ function template_edit() {
 			AND input_output="in" ORDER BY name',
 			array($template_data['data_input_id']));
 
-		html_start_box(__('Custom Data [data input: %s]', html_escape(db_fetch_cell_prepared('SELECT name FROM data_input WHERE id = ?', array($template_data['data_input_id'])))), '100%', true, '3', 'center', '');
+		$name = db_fetch_cell_prepared('SELECT name
+			FROM data_input
+			WHERE id = ?',
+			array($template_data['data_input_id']));
+
+		html_start_box(__('Custom Data [data input: %s]', html_escape($name)), '100%', true, '3', 'center', '');
 
 		/* loop through each field found */
-		if (sizeof($fields) > 0) {
+		if (cacti_sizeof($fields) > 0) {
 			$class = 'odd';
 
 			foreach ($fields as $field) {
 				$data_input_data = db_fetch_row_prepared('SELECT t_value, value
 					FROM data_input_data
 					WHERE data_template_data_id = ?
-					AND data_input_field_id = ?', array($template_data['id'], $field['id']));
+					AND data_input_field_id = ?',
+					array($template_data['id'], $field['id']));
 
-				if (sizeof($data_input_data)) {
+				if (cacti_sizeof($data_input_data)) {
 					$old_value  = $data_input_data['value'];
 					$old_tvalue = $data_input_data['t_value'];
 				} else {
@@ -727,6 +807,44 @@ function template_edit() {
 	}
 
 	form_save_button('data_templates.php', 'return');
+
+	?>
+	<script type='text/javascript'>
+
+	var readOnly = <?php print $readOnly ? 'true':'false';?>;
+
+	$(function() {
+		if (readOnly) {
+			// Data Source
+			$('#data_input_id').prop('disabled', true).addClass('ui-state-disabled');
+			$('#t_data_input_id').prop('disabled', true).addClass('ui-state-disabled');
+
+			if ($('#data_input_id').selectmenu('instance')) {
+				$('#data_input_id').selectmenu('disable');
+			}
+
+			// Data source attributes
+			$('#data_source_name').prop('disabled', true).addClass('ui-state-disabled');
+			$('#t_data_source_name').prop('disabled', true).addClass('ui-state-disabled');
+			$('#data_source_type_id').prop('disabled', true).addClass('ui-state-disabled');
+			$('#t_data_source_type_id').prop('disabled', true).addClass('ui-state-disabled');
+
+			// Custom Data
+			$('#value_index_type').prop('disabled', true).addClass('ui-state-disabled');
+			$('#t_value_index_type').prop('disabled', true).addClass('ui-state-disabled');
+			$('#value_index_value').prop('disabled', true).addClass('ui-state-disabled');
+			$('#t_value_index_value').prop('disabled', true).addClass('ui-state-disabled');
+			$('#value_output_type').prop('disabled', true).addClass('ui-state-disabled');
+			$('#t_value_output_type').prop('disabled', true).addClass('ui-state-disabled');
+
+			if ($('#data_source_type_id').selectmenu('instance')) {
+				$('#data_source_type_id').selectmenu('disable');
+			}
+		}
+	});
+
+	</script>
+	<?php
 }
 
 function template() {
@@ -744,10 +862,9 @@ function template() {
 			'default' => '1'
 			),
 		'filter' => array(
-			'filter' => FILTER_CALLBACK,
+			'filter' => FILTER_DEFAULT,
 			'pageset' => true,
-			'default' => '',
-			'options' => array('options' => 'sanitize_search_string')
+			'default' => ''
 			),
 		'sort_column' => array(
 			'filter' => FILTER_CALLBACK,
@@ -793,7 +910,7 @@ function template() {
 						<?php print __('Search');?>
 					</td>
 					<td>
-						<input id='filter' type='text' name='filter' size='25' value='<?php print html_escape_request_var('filter');?>'>
+						<input type='text' class='ui-state-default ui-corner-all' id='filter' name='filter' size='25' value='<?php print html_escape_request_var('filter');?>'>
 					</td>
 					<td>
 						<?php print __('Profile');?>
@@ -803,7 +920,7 @@ function template() {
 							<option value='-1'<?php print (get_request_var('profile') == '-1' ? ' selected>':'>') . __('All');?></option>
 							<?php
 							$profiles = array_rekey(db_fetch_assoc('SELECT id, name FROM data_source_profiles ORDER BY name'), 'id', 'name');
-							if (sizeof($profiles)) {
+							if (cacti_sizeof($profiles)) {
 								foreach ($profiles as $key => $value) {
 									print "<option value='" . $key . "'"; if (get_request_var('profile') == $key) { print ' selected'; } print '>' . html_escape($value) . "</option>\n";
 								}
@@ -818,7 +935,7 @@ function template() {
 						<select id='rows' name='rows' onChange='applyFilter()'>
 							<option value='-1'<?php print (get_request_var('rows') == '-1' ? ' selected>':'>') . __('Default');?></option>
 							<?php
-							if (sizeof($item_rows)) {
+							if (cacti_sizeof($item_rows)) {
 								foreach ($item_rows as $key => $value) {
 									print "<option value='" . $key . "'"; if (get_request_var('rows') == $key) { print ' selected'; } print '>' . html_escape($value) . "</option>\n";
 								}
@@ -834,8 +951,8 @@ function template() {
 					</td>
 					<td>
 						<span>
-							<input type='button' id='refresh' value='<?php print __esc('Go');?>' title='<?php print __esc('Set/Refresh Filters');?>'>
-							<input type='button' id='clear' value='<?php print __esc('Clear');?>' title='<?php print __esc('Clear Filters');?>'>
+							<input type='button' class='ui-button ui-corner-all ui-widget' id='refresh' value='<?php print __esc('Go');?>' title='<?php print __esc('Set/Refresh Filters');?>'>
+							<input type='button' class='ui-button ui-corner-all ui-widget' id='clear' value='<?php print __esc('Clear');?>' title='<?php print __esc('Clear Filters');?>'>
 						</span>
 					</td>
 				</tr>
@@ -884,7 +1001,7 @@ function template() {
 	/* form the 'where' clause for our main sql query */
 	$rows_where = '';
 	if (get_request_var('filter') != '') {
-		$sql_where = " WHERE (dt.name like '%" . get_request_var('filter') . "%')";
+		$sql_where = ' WHERE (dt.name LIKE ' . db_qstr('%' . get_request_var('filter') . '%') . ')';
 	} else {
 		$sql_where = '';
 	}
@@ -918,7 +1035,7 @@ function template() {
 	$sql_order = get_order_string();
 	$sql_limit = ' LIMIT ' . ($rows*(get_request_var('page')-1)) . ',' . $rows;
 
-	$template_list = db_fetch_assoc("SELECT dt.id, dt.name,
+	$template_list_sql = "SELECT dt.id, dt.name,
 		di.name AS data_input_method, dtd.active AS active, dsp.name AS profile_name,
 		SUM(CASE WHEN dtd.local_data_id>0 THEN 1 ELSE 0 END) AS data_sources
 		FROM data_template AS dt
@@ -932,9 +1049,54 @@ function template() {
 		GROUP BY dt.id
 		$sql_having
 		$sql_order
-		$sql_limit");
+		$sql_limit";
+	$template_list = db_fetch_assoc($template_list_sql);
 
-	$nav = html_nav_bar('data_templates.php?filter=' . get_request_var('filter'), MAX_DISPLAY_PAGES, get_request_var('page'), $rows, $total_rows, 7, __('Data Templates'), 'page', 'main');
+	$display_text = array(
+		'name' => array(
+			'display' => __('Data Template Name'),
+			'align' => 'left',
+			'sort' => 'ASC',
+			'tip' => __('The name of this Data Template.')
+		),
+		'id' => array(
+			'display' => __('ID'),
+			'align' => 'right',
+			'sort' => 'ASC',
+			'tip' => __('The internal database ID for this Data Template.  Useful when performing automation or debugging.')
+		),
+		'nosort' => array(
+			'display' => __('Deletable'),
+			'align' => 'right',
+			'tip' => __('Data Templates that are in use cannot be Deleted.  In use is defined as being referenced by a Data Source.')
+		),
+		'data_sources' => array(
+			'display' => __('Data Sources Using'),
+			'align' => 'right',
+			'sort' => 'DESC',
+			'tip' => __('The number of Data Sources using this Data Template.')
+		),
+		'data_input_method' => array(
+			'display' => __('Input Method'),
+			'align' => 'left',
+			'sort' => 'ASC',
+			'tip' => __('The method that is used to place Data into the Data Source RRDfile.')
+		),
+		'profile_name' => array(
+			'display' => __('Profile Name'),
+			'align' => 'left',
+			'sort' => 'ASC',
+			'tip' => __('The default Data Source Profile for this Data Template.')
+		),
+		'active' => array(
+			'display' => __('Status'),
+			'align' => 'left',
+			'sort' => 'ASC',
+			'tip' => __('Data Sources based on Inactive Data Templates will not be updated when the poller runs.')
+		)
+	);
+
+	$nav = html_nav_bar('data_templates.php?filter=' . get_request_var('filter'), MAX_DISPLAY_PAGES, get_request_var('page'), $rows, $total_rows, sizeof($display_text) + 1, __('Data Templates'), 'page', 'main');
 
 	form_start('data_templates.php', 'chk');
 
@@ -942,30 +1104,25 @@ function template() {
 
 	html_start_box('', '100%', '', '3', 'center', '');
 
-	$display_text = array(
-		'name'              => array('display' => __('Data Template Name'), 'align' => 'left', 'sort' => 'ASC', 'tip' => __('The name of this Data Template.')),
-		'id'                => array('display' => __('ID'), 'align' => 'right', 'sort' => 'ASC', 'tip' => __('The internal database ID for this Data Template.  Useful when performing automation or debugging.')),
-		'nosort'            => array('display' => __('Deletable'), 'align' => 'right', 'tip' => __('Data Templates that are in use cannot be Deleted.  In use is defined as being referenced by a Data Source.')),
-		'data_sources'      => array('display' => __('Data Sources Using'), 'align' => 'right', 'sort' => 'DESC', 'tip' => __('The number of Data Sources using this Data Template.')),
-		'data_input_method' => array('display' => __('Input Method'), 'align' => 'left', 'sort' => 'ASC', 'tip' => __('The method that is used to place Data into the Data Source RRDfile.')),
-		'profile_name' => array('display' => __('Profile Name'), 'align' => 'left', 'sort' => 'ASC', 'tip' => __('The default Data Source Profile for this Data Template.')),
-		'active'            => array('display' => __('Status'), 'align' => 'left', 'sort' => 'ASC', 'tip' => __('Data Sources based on Inactive Data Templates will not be updated when the poller runs.'))
-	);
-
 	html_header_sort_checkbox($display_text, get_request_var('sort_column'), get_request_var('sort_direction'), false);
 
-	if (sizeof($template_list)) {
+	if (cacti_sizeof($template_list)) {
 		foreach ($template_list as $template) {
 			if ($template['data_sources'] > 0) {
 				$disabled = true;
 			} else {
 				$disabled = false;
 			}
+
+			$ds_url = 'data_sources.php?reset=true&template_id=' . $template['id'];
+			if (get_request_var('profile') != '-1') {
+				$ds_url .= '&profile=' . get_request_var('profile');
+			}
 			form_alternate_row('line' . $template['id'], true, $disabled);
 			form_selectable_cell(filter_value($template['name'], get_request_var('filter'), 'data_templates.php?action=template_edit&id=' . $template['id']), $template['id']);
-			form_selectable_cell($template['id'], $template['id'], '', 'text-align:right');
-			form_selectable_cell($disabled ? __('No'):__('Yes'), $template['id'], '', 'text-align:right');
-			form_selectable_cell('<a class="linkEditMain" href="' . html_escape('data_sources.php?reset=true&template_id=' . $template['id']) . '">' . number_format_i18n($template['data_sources']) . '</a>', $template['id'], '', 'text-align:right');
+			form_selectable_cell($template['id'], $template['id'], '', 'right');
+			form_selectable_cell($disabled ? __('No'):__('Yes'), $template['id'], '', 'right');
+			form_selectable_cell('<a class="linkEditMain" href="' . html_escape($ds_url) . '">' . number_format_i18n($template['data_sources']) . '</a>', $template['id'], '', 'right');
 			form_selectable_cell((empty($template['data_input_method']) ? '<em>' . __('None') .'</em>': html_escape($template['data_input_method'])), $template['id']);
 			form_selectable_cell((empty($template['profile_name']) ? __('External'):html_escape($template['profile_name'])), $template['id']);
 			form_selectable_cell((($template['active'] == 'on') ? __('Active'):__('Disabled')), $template['id']);
@@ -973,11 +1130,12 @@ function template() {
 			form_end_row();
 		}
 	} else {
-		print "<tr class='tableRow'><td colspan='6'><em>" . __('No Data Templates Found') . "</em></td></tr>\n";
+		print "<tr class='tableRow'><td colspan='" . (cacti_sizeof($display_text)+1) . "'><em>" . __('No Data Templates Found') . "</em></td></tr>\n";
 	}
+
 	html_end_box(false);
 
-	if (sizeof($template_list)) {
+	if (cacti_sizeof($template_list)) {
 		print $nav;
 	}
 
