@@ -80,12 +80,12 @@ $graph_data_array['image_format'] = $gtype;
 session_write_close();
 
 /* override: graph start time (unix time) */
-if (!isempty_request_var('graph_start') && get_request_var('graph_start') < 1600000000) {
+if (!isempty_request_var('graph_start') && get_request_var('graph_start') < FILTER_VALIDATE_MAX_DATE_AS_INT) {
 	$graph_data_array['graph_start'] = get_request_var('graph_start');
 }
 
 /* override: graph end time (unix time) */
-if (!isempty_request_var('graph_end') && get_request_var('graph_end') < 1600000000) {
+if (!isempty_request_var('graph_end') && get_request_var('graph_end') < FILTER_VALIDATE_MAX_DATE_AS_INT) {
 	$graph_data_array['graph_end'] = get_request_var('graph_end');
 }
 
