@@ -1,7 +1,7 @@
 <?php
 /*
  +-------------------------------------------------------------------------+
- | Copyright (C) 2004-2018 The Cacti Group                                 |
+ | Copyright (C) 2004-2020 The Cacti Group                                 |
  |                                                                         |
  | This program is free software; you can redistribute it and/or           |
  | modify it under the terms of the GNU General Public License             |
@@ -173,7 +173,7 @@ function form_save() {
 			$save['graph_type_id']     = form_input_validate((isset($item['graph_type_id']) ? $item['graph_type_id'] : get_filter_request_var('graph_type_id')), 'graph_type_id', '^[0-9]+$', true, 3);
 
 			if (isset_request_var('line_width') || isset($item['line_width'])) {
-				$save['line_width']    = form_input_validate((isset($item['line_width']) ? $item['line_width'] : get_nfilter_request_var('line_width')), 'line_width', '^[0-9]+[.0-9]+$', true, 3);
+				$save['line_width']    = form_input_validate((isset($item['line_width']) ? $item['line_width'] : get_nfilter_request_var('line_width')), 'line_width', '(^[0-9]+[\.,0-9]+$|^[0-9]+$)', true, 3);
 			}else { # make sure to transfer old LINEx style into line_width on save
 				switch ($save['graph_type_id']) {
 				case GRAPH_ITEM_TYPE_LINE1:
@@ -260,7 +260,7 @@ function form_save() {
 								AND task_item_id = ?',
 								array($save['graph_template_id'], get_nfilter_request_var('task_item_id')));
 
-							if (sizeof($graph_items)) {
+							if (cacti_sizeof($graph_items)) {
 								foreach ($graph_items as $graph_item) {
 									db_execute_prepared('REPLACE INTO graph_template_input_defs
 										(graph_template_input_id, graph_template_item_id)
@@ -310,9 +310,9 @@ function item_movedown() {
 	$arr        = get_graph_group(get_request_var('id'));
 	$next_id    = get_graph_parent(get_request_var('id'), 'next');
 
-	$graph_type = db_fetch_cell_prepared('SELECT graph_type_id 
-		FROM graph_templates_item 
-		WHERE id = ?', 
+	$graph_type = db_fetch_cell_prepared('SELECT graph_type_id
+		FROM graph_templates_item
+		WHERE id = ?',
 		array(get_request_var('id')));
 
 	$text_type  = $graph_item_types[$graph_type];
@@ -342,9 +342,9 @@ function item_moveup() {
 	$arr = get_graph_group(get_request_var('id'));
 	$next_id = get_graph_parent(get_request_var('id'), 'previous');
 
-	$graph_type = db_fetch_cell_prepared('SELECT graph_type_id 
-		FROM graph_templates_item 
-		WHERE id = ?', 
+	$graph_type = db_fetch_cell_prepared('SELECT graph_type_id
+		FROM graph_templates_item
+		WHERE id = ?',
 		array(get_request_var('id')));
 
 	$text_type  = $graph_item_types[$graph_type];
@@ -381,9 +381,9 @@ function item_remove() {
 		AND graph_template_input_defs.graph_template_item_id = ?
 		GROUP BY graph_template_input.id', array(get_request_var('graph_template_id'), get_request_var('id')));
 
-	if (sizeof($graph_item_inputs) > 0) {
+	if (cacti_sizeof($graph_item_inputs) > 0) {
 		foreach ($graph_item_inputs as $graph_item_input) {
-			if (sizeof(db_fetch_assoc_prepared('SELECT graph_template_input_id FROM graph_template_input_defs WHERE graph_template_input_id = ?', array($graph_item_input['id']))) == 1) {
+			if (cacti_sizeof(db_fetch_assoc_prepared('SELECT graph_template_input_id FROM graph_template_input_defs WHERE graph_template_input_id = ?', array($graph_item_input['id']))) == 1) {
 				db_execute_prepared('DELETE FROM graph_template_input WHERE id = ?', array($graph_item_input['id']));
 			}
 		}
@@ -402,7 +402,7 @@ function item_edit() {
 
 	form_start('graph_templates_items.php', 'graph_items');
 
-	$header_label = __('Graph Template Items [edit graph: %s]', htmlspecialchars(db_fetch_cell_prepared('SELECT name FROM graph_templates WHERE id = ?', array(get_request_var('graph_template_id')))));
+	$header_label = __esc('Graph Template Items [edit graph: %s]', db_fetch_cell_prepared('SELECT name FROM graph_templates WHERE id = ?', array(get_request_var('graph_template_id'))));
 
 	html_start_box($header_label, '100%', true, '3', 'center', '');
 
@@ -422,7 +422,7 @@ function item_edit() {
 			ORDER BY sequence DESC',
 			array(get_request_var('graph_template_id')));
 
-		if (sizeof($default) > 0) {
+		if (cacti_sizeof($default) > 0) {
 			$struct_graph_item['task_item_id']['default'] = $default['task_item_id'];
 		} else {
 			$struct_graph_item['task_item_id']['default'] = 0;
@@ -465,7 +465,7 @@ function item_edit() {
 	form_hidden_box('_task_item_id', (isset($template_item) ? $template_item['task_item_id'] : '0'), '');
 	form_hidden_box('save_component_item', '1', '');
 	form_hidden_box('invisible_alpha', $form_array['alpha']['value'], 'FF');
-	form_hidden_box('rrdtool_version', read_config_option('rrdtool_version'), '');
+	form_hidden_box('rrdtool_version', get_rrdtool_version(), '');
 
 	form_save_button('graph_templates.php?action=template_edit&id=' . get_request_var('graph_template_id'));
 
@@ -511,7 +511,7 @@ function item_edit() {
 	function setRowVisibility() {
 		switch($('#graph_type_id').val()) {
 		case '1': // COMMENT
-			$('#row_task_item_id').hide();
+			$('#row_task_item_id').show();
 			$('#row_color_id').hide();
 			$('#row_line_width').hide();
 			$('#row_dashes').hide();
@@ -528,6 +528,22 @@ function item_edit() {
 			$('#row_hard_return').show();
 			break;
 		case '2': // HRULE
+			$('#row_task_item_id').show();
+			$('#row_color_id').show();
+			$('#row_line_width').hide();
+			$('#row_dashes').show();
+			$('#row_dash_offset').show();
+			$('#row_textalign').hide();
+			$('#row_shift').hide();
+			$('#row_alpha').hide();
+			$('#row_consolidation_function_id').hide();
+			$('#row_cdef_id').hide();
+			$('#row_vdef_id').hide();
+			$('#row_value').show();
+			$('#row_gprint_id').hide();
+			$('#row_text_format').show();
+			$('#row_hard_return').show();
+			break;
 		case '3': // VRULE
 			$('#row_task_item_id').hide();
 			$('#row_color_id').show();
