@@ -1,6 +1,6 @@
 /*
  +-------------------------------------------------------------------------+
- | Copyright (C) 2004-2018 The Cacti Group                                 |
+ | Copyright (C) 2004-2020 The Cacti Group                                 |
  |                                                                         |
  | This program is free software; you can redistribute it and/or           |
  | modify it under the terms of the GNU General Public License             |
@@ -38,14 +38,15 @@
 		var clientTimeOffset = clientTime.getTimezoneOffset()*60*(-1);			//requires -1, because PHP return the opposite
 		var timeOffset = 0;
 		var activeElement = '';
+		var mouseDown = false;
 
 		// default values of the different options being offered
 		var defaults = {
-			inputfieldStartTime	: '',                                           // ID of the input field that contains the start date
-			inputfieldEndTime	: '',                                           // ID of the input field that contains the end date
-			submitButton		: 'button_refresh_x',                           // ID of the submit button
-			cookieName			: 'cacti_zoom',                                 // default name required for session cookie
-			serverTimeOffset	: 0												// JS calculates in relation to the localization of the browser :/ - only required for 0.8.8
+			inputfieldStartTime	: '',                  // ID of the input field that contains the start date
+			inputfieldEndTime	: '',                  // ID of the input field that contains the end date
+			submitButton		: 'button_refresh_x',  // ID of the submit button
+			cookieName			: 'cacti_zoom',        // default name required for session cookie
+			serverTimeOffset	: 0					   // JS calculates in relation to the localization of the browser :/ - only required for 0.8.8
 		};
 
 		// define global variables / objects here
@@ -69,8 +70,9 @@
 		};
 
 		// support jQuery's concatenation
-		return this.each(function() { zoom_init( $(this) ); });
-
+		return this.each(function() {
+			zoom_init( $(this) );
+		});
 
 		/* ++++++++++++++++++++ Universal Functions +++++++++++++++++++++++++ */
 
@@ -85,7 +87,7 @@
 			urlParameters = urlBaseAndParameters[1].split('&');
 			parameters['urlBase'] = urlBase;
 
-			for(var i=0; i<urlParameters.length; i++) {
+			for (var i=0; i<urlParameters.length; i++) {
 				parameter = urlParameters[i].split('=');
 				parameters[parameter[0].replace(/^graph_/, '')] = $.isNumeric(parameter[1]) ? +parameter[1] : parameter[1];
 			}
@@ -97,7 +99,7 @@
 		 **/
 		function serialize(object){
 			var str = '';
-			for(var key in object) { str += (key + '=' + object[key] + ','); }
+			for (var key in object) { str += (key + '=' + object[key] + ','); }
 			return str.slice(0, -1);
 		}
 
@@ -110,7 +112,7 @@
 
 			if (string != null) {
 				pairs = string.split(',');
-				for(var i=0; i<pairs.length; i++) {
+				for (var i=0; i<pairs.length; i++) {
 					pair = pairs[i].split('=');
 					if (pair[1] == 'true') {
 						pair[1] = true;
@@ -129,7 +131,6 @@
 		 * converts a Unix time stamp to a formatted date string
 		 **/
 		function unixTime2Date(unixTime){
-
 			var date	= new Date(unixTime*1000+timeOffset);
 			var year	= date.getFullYear();
 			var month	= ((date.getMonth()+1) < 9 ) ? '0' + (date.getMonth()+1) : date.getMonth()+1;
@@ -149,7 +150,7 @@
 		function zoom_init(image) {
 			var $this = image;
 
-			if($('#zoom-container').hasClass('zoom_active_' + zoomGetImageId($this))) {
+			if ($('#zoom-container').hasClass('zoom_active_' + zoomGetImageId($this))) {
 				zoomElements_remove();
 				zoomFunction_init($this);
 			}
@@ -157,8 +158,10 @@
 			$this.parent().disableSelection();
 			$this.off().mouseover(
 				function(){
-					zoomElements_remove();
-					zoomFunction_init($this);
+					if (!mouseDown) {
+						zoomElements_remove();
+						zoomFunction_init($this);
+					}
 				}
 			);
 		}
@@ -253,7 +256,7 @@
 				// Please note: IE does not fire hover or click behaviors on completely transparent elements.
 				// Use a background color and set opacity to 1% as a workaround.(see CSS file)
 				$('<div id="zoom-container"></div>').appendTo('body').delay(1000);
-				$('#zoom-container').css({ position: 'absolute', width:(zoom.image.width-1)+'px', height:(zoom.image.height-1)+'px' }).removeClass().addClass('zoom_active_' + zoomGetId(zoom));
+				$('#zoom-container').css({ position: 'absolute' }).removeClass().addClass('zoom_active_' + zoomGetId(zoom));
 			}
 
 			// add a hidden anchor to use for downloads
@@ -295,78 +298,96 @@
 			if ($('#zoom-menu').length == 0) {
 				$('<div id="zoom-menu" class="zoom-menu">'
 					+ '<div class="first_li">'
-					+ 		'<div class="ui-icon ui-icon-zoomin zoomContextMenuAction__zoom_in"></div>'
-					+       '<span class="zoomContextMenuAction__zoom_in">Zoom In</span>'
+					+ 	'<div class="ui-icon ui-icon-zoomin zoomContextMenuAction__zoom_in"></div>'
+					+ 	'<span class="zoomContextMenuAction__zoom_in">' + zoom_i18n_zoom_in + '</span>'
 					+ '</div>'
 					+ '<div class="first_li">'
-					+ 		'<div class="ui-icon ui-icon-zoomout zoomContextMenuAction__zoom_out"></div>'
-					+ 		'<span class="zoomContextMenuAction__zoom_out">Zoom Out (2x)</span>'
-					+ 		'<div class="inner_li">'
-					+ 			'<span class="zoomContextMenuAction__zoom_out__2">2x</span>'
-					+ 			'<span class="zoomContextMenuAction__zoom_out__4">4x</span>'
-					+ 			'<span class="zoomContextMenuAction__zoom_out__8">8x</span>'
-					+ 			'<span class="zoomContextMenuAction__zoom_out__16">16x</span>'
-					+ 			'<span class="zoomContextMenuAction__zoom_out__32">32x</span>'
-					+ 		'</div>'
+					+ 	'<div class="ui-icon ui-icon-zoomout zoomContextMenuAction__zoom_out"></div>'
+					+	'<div class="ui-icon ui-icon-play ui-icon-right"></div>'
+					+ 	'<span class="zoomContextMenuAction__zoom_out">' + zoom_i18n_zoom_out + '</span>'
+					+ 	'<div class="inner_li">'
+					+ 		'<span class="zoomContextMenuAction__zoom_out__2">' + zoom_i18n_zoom_2 + '</span>'
+					+ 		'<span class="zoomContextMenuAction__zoom_out__4">' + zoom_i18n_zoom_4 + '</span>'
+					+ 		'<span class="zoomContextMenuAction__zoom_out__8">' + zoom_i18n_zoom_8 + '</span>'
+					+ 		'<span class="zoomContextMenuAction__zoom_out__16">' + zoom_i18n_zoom_16 + '</span>'
+					+ 		'<span class="zoomContextMenuAction__zoom_out__32">' + zoom_i18n_zoom_32 + '</span>'
+					+ 	'</div>'
 					+ '</div>'
 					+ '<div class="sep_li"></div>'
 					+ '<div class="first_li">'
-					+ 		'<div class="ui-icon ui-icon-empty"></div><span>Zoom Mode</span>'
-					+ 		'<div class="inner_li">'
-					+ 			'<span class="zoomContextMenuAction__set_zoomMode__quick">Quick</span>'
-					+ 			'<span class="zoomContextMenuAction__set_zoomMode__advanced">Advanced</span>'
-					+ 		'</div>'
+					+ 	'<div class="ui-icon ui-icon-empty"></div>'
+					+	'<div class="ui-icon ui-icon-play ui-icon-right"></div>'
+					+	'<span>' + zoom_i18n_mode + '</span>'
+					+ 	'<div class="inner_li">'
+					+ 		'<span class="zoomContextMenuAction__set_zoomMode__quick">' + zoom_i18n_quick + '</span>'
+					+ 		'<span class="zoomContextMenuAction__set_zoomMode__advanced">' + zoom_i18n_advanced + '</span>'
+					+ 	'</div>'
 					+ '</div>'
 					+ '<div class="sep_li"></div>'
 					+ '<div class="first_li">'
-					+ 		'<div class="ui-icon ui-icon-empty"></div><span>Graph</span>'
-					+ 		'<div class="inner_li">'
-					+ 			'<span class="zoomContextMenuAction__newTab">Open in new tab</a></span>'
-					+			'<span class="zoomContextMenuAction__save">Save graph</span>'
-					+ 			'<span class="zoomContextMenuAction__link">Copy graph link</span>'
-					+ 		'</div>'
+					+ 	'<div class="ui-icon ui-icon-empty"></div>'
+					+	'<div class="ui-icon ui-icon-play ui-icon-right"></div>'
+					+	'<span>' + zoom_i18n_graph + '</span>'
+					+	'<div class="inner_li">'
+					+ 		'<span class="zoomContextMenuAction__newTab">' + zoom_i18n_newTab + '</span>'
+					+		'<span class="zoomContextMenuAction__save">' + zoom_i18n_save_graph + '</span>'
+					+		'<span class="zoomContextMenuAction__copy">' + zoom_i18n_copy_graph + '</span>'
+					+		'<span class="zoomContextMenuAction__link">' + zoom_i18n_copy_graph_link + '</span>'
+					+	'</div>'
 					+ '</div>'
 					+ '<div class="first_li">'
-					+ 		'<div class="ui-icon ui-icon-wrench"></div><span>Settings</span>'
+					+	'<div class="ui-icon ui-icon-wrench"></div>'
+					+	'<div class="ui-icon ui-icon-play ui-icon-right"></div>'
+					+	'<span>' + zoom_i18n_settings + '</span>'
+					+	'<div class="inner_li">'
+					+		'<div class="sec_li">'
+					+			'<div class="ui-icon ui-icon-play ui-icon-right"></div>'
+					+			'<span>' + zoom_i18n_timestamps + '</span>'
+					+			'<div class="inner_li">'
+					+ 				'<span class="zoomContextMenuAction__set_zoomTimestamps__on">' + zoom_i18n_on + '</span>'
+					+ 				'<span class="zoomContextMenuAction__set_zoomTimestamps__auto">' + zoom_i18n_auto + '</span>'
+					+ 				'<span class="zoomContextMenuAction__set_zoomTimestamps__off">' + zoom_i18n_off + '</span>'
+					+ 			'</div>'
+					+ 		'</div>'
+					+ 		'<div class="sep_li"></div>'
+					+ 		'<div class="sec_li">'
+					+			'<div class="ui-icon ui-icon-play ui-icon-right"></div>'
+					+			'<span>' + zoom_i18n_zoom_out_factor + '</span>'
 					+ 			'<div class="inner_li">'
-					+ 				'<div class="sec_li advanced_mode"><span>Timestamps</span></span>'
-					+ 					'<div class="inner_li advanced_mode">'
-					+ 						'<span class="zoomContextMenuAction__set_zoomTimestamps__on">Always On</span>'
-					+ 						'<span class="zoomContextMenuAction__set_zoomTimestamps__auto">Auto</span>'
-					+ 						'<span class="zoomContextMenuAction__set_zoomTimestamps__off">Always Off</span>'
-					+ 					'</div>'
+					+ 				'<span class="zoomContextMenuAction__set_zoomOutFactor__2">' + zoom_i18n_zoom_2 + '</span>'
+					+ 				'<span class="zoomContextMenuAction__set_zoomOutFactor__4">' + zoom_i18n_zoom_4 + '</span>'
+					+ 				'<span class="zoomContextMenuAction__set_zoomOutFactor__8">' + zoom_i18n_zoom_8 + '</span>'
+					+ 				'<span class="zoomContextMenuAction__set_zoomOutFactor__16">' + zoom_i18n_zoom_16 + '</span>'
+					+ 				'<span class="zoomContextMenuAction__set_zoomOutFactor__32">' + zoom_i18n_zoom_32 + '</span>'
+					+ 			'</div>'
+					+ 		'</div>'
+					+ 		'<div class="sec_li">'
+					+			'<div class="ui-icon ui-icon-play ui-icon-right"></div>'
+					+			'<span>' + zoom_i18n_zoom_out_positioning + '</span>'
+					+ 				'<div class="inner_li">'
+					+ 					'<span class="zoomContextMenuAction__set_zoomOutPositioning__begin">' + zoom_i18n_begin + '</span>'
+					+ 					'<span class="zoomContextMenuAction__set_zoomOutPositioning__center">' + zoom_i18n_center + '</span>'
+					+ 					'<span class="zoomContextMenuAction__set_zoomOutPositioning__end">' + zoom_i18n_end + '</span>'
 					+ 				'</div>'
-					+ 				'<div class="sep_li"></div>'
-					+ 				'<div class="sec_li"><span>Zoom Out Factor</span>'
-					+ 					'<div class="inner_li">'
-					+ 						'<span class="zoomContextMenuAction__set_zoomOutFactor__2">2x</span>'
-					+ 						'<span class="zoomContextMenuAction__set_zoomOutFactor__4">4x</span>'
-					+ 						'<span class="zoomContextMenuAction__set_zoomOutFactor__8">8x</span>'
-					+ 						'<span class="zoomContextMenuAction__set_zoomOutFactor__16">16x</span>'
-					+ 						'<span class="zoomContextMenuAction__set_zoomOutFactor__32">32x</span>'
-					+ 					'</div>'
-					+ 				'</div>'
-					+ 				'<div class="sec_li"><span>Zoom Out Positioning</span>'
-					+ 					'<div class="inner_li">'
-					+ 						'<span class="zoomContextMenuAction__set_zoomOutPositioning__begin">Begin with</span>'
-					+ 						'<span class="zoomContextMenuAction__set_zoomOutPositioning__center">Center</span>'
-					+ 						'<span class="zoomContextMenuAction__set_zoomOutPositioning__end">End with</span>'
-					+ 					'</div>'
-					+ 				'</div>'
-					+ 				'<div class="sec_li"><span>3rd Mouse Button</span>'
-					+ 					'<div class="inner_li">'
-					+ 						'<span class="zoomContextMenuAction__set_zoom3rdMouseButton__zoom_in">Zoom in</span>'
-					+ 						'<span class="zoomContextMenuAction__set_zoom3rdMouseButton__zoom_out">Zoom out</span>'
-					+ 						'<span class="zoomContextMenuAction__set_zoom3rdMouseButton__off">Disabled</span>'
-					+ 					'</div>'
+					+ 			'</div>'
+					+ 			'<div class="sec_li">'
+					+				'<div class="ui-icon ui-icon-play ui-icon-right"></div>'
+					+				'<span>' + zoom_i18n_3rd_button + '</span>'
+					+ 				'<div class="inner_li">'
+					+ 					'<span class="zoomContextMenuAction__set_zoom3rdMouseButton__zoom_in">' + zoom_i18n_zoom_in + '</span>'
+					+ 					'<span class="zoomContextMenuAction__set_zoom3rdMouseButton__zoom_out">' + zoom_i18n_zoom_out + '</span>'
+					+ 					'<span class="zoomContextMenuAction__set_zoom3rdMouseButton__off">' + zoom_i18n_disabled + '</span>'
 					+ 				'</div>'
 					+ 			'</div>'
 					+ 		'</div>'
+					+ 	'</div>'
 					+ '<div class="sep_li"></div>'
 					+ '<div class="first_li">'
-					+ 		'<div class="ui-icon ui-icon-close zoomContextMenuAction__close"></div><span class="zoomContextMenuAction__close">Close</span>'
-				+ '</div>').appendTo('body');
+					+ 	'<div class="ui-icon ui-icon-close zoomContextMenuAction__close"></div>'
+					+	'<span class="zoomContextMenuAction__close">' + zoom_i18n_close + '</span>'
+					+ '</div>').appendTo('body');
 			}
+
 			zoomElements_reposition();
 			zoomElements_reset();
 			zoomContextMenu_init();
@@ -399,13 +420,13 @@
 				$(this).removeAttr('style');
 			});
 			$('#zoom-container').off();
-			$('#zoom-container').bind('contextmenu', function(e) { zoomContextMenu_show(e); return false;} );
+			$('#zoom-container').bind('contextmenu', function(e) { zoomContextMenu_toggle(e); return false;} );
 			$('#zoom-box').off();
 			$('#zoom-box').css({ cursor:'crosshair', width:zoom.box.width + 'px', height:zoom.box.height + 'px', top:zoom.box.top+'px', left:zoom.box.left+'px' });
-			$('#zoom-box').bind('contextmenu', function(e) { zoomContextMenu_show(e); return false;} );
+			$('#zoom-box').bind('contextmenu', function(e) { zoomContextMenu_toggle(e); return false;} );
 			$('#zoom-area').off().css({ top:zoom.box.top+'px', height:zoom.box.height+'px' });
 			$('.zoom-area-excluded').off();
-			$('.zoom-area-excluded').bind('contextmenu', function(e) { zoomContextMenu_show(e); return false;} );
+			$('.zoom-area-excluded').bind('contextmenu', function(e) { zoomContextMenu_toggle(e); return false;} );
 			$('.zoom-area-excluded').bind('click', function(e) { zoomContextMenu_hide(); return false;} );
 			$('.zoom-marker-arrow-up').css({ top:(zoom.box.height-6) + 'px' });
 			$('.zoom-marker-tooltip-value').disableSelection();
@@ -415,7 +436,6 @@
 		* registers all the different mouse click event handler
 		*/
 		function zoomAction_init(image) {
-
 			if (zoom.custom.zoomMode == 'quick') {
 				$('#zoom-area').resizable({ containment: '#zoom-box', handles: 'e, w' });
 				$('#zoom-box').off('mousedown').on('mousedown', function(e) {
@@ -459,6 +479,18 @@
 					zoomAction_draw(e);
 				} );
 
+				/* capture mouse up/down events for zoom */
+				$(document).off('mousedown').on('mousedown', function() {
+					mouseDown = true;
+				}).off('mouseup').on('mouseup', function() {
+					if (mouseDown) {
+						if (zoom.attr.start != 'none') {
+							zoomAction_zoom_in();
+						}
+					}
+					mouseDown = false;
+				});
+
 				/* moving the mouse pointer quickly will avoid that the mousemove event has enough time to actualize the zoom area */
 				$('#zoom-box').mouseout( function(e) {
 					zoomAction_draw(e);
@@ -564,7 +596,6 @@
 									},
 								drag:
 									function(event, ui) {
-
 										if (ui.position['left'] < zoom.box.left) {
 											zoom.marker[marker].left = zoom.box.left;
 										} else if (ui.position['left'] > zoom.box.right) {
@@ -640,7 +671,6 @@
 			}
 		}
 
-
 		/*
 		* executes a dynamic zoom in
 		*/
@@ -688,7 +718,9 @@
 				$('#' + zoom.options.inputfieldEndTime).val(unixTime2Date(newGraphEndTime));
 
 				if (graph_start !== null && graph_end !== null) {
-					if (myGraphLocation != 'graph') {
+					zoom.attr.start = 'none';
+
+					if (pageAction != 'graph') {
 						graph_start = newGraphStartTime;
 						graph_end = newGraphEndTime;
 
@@ -711,6 +743,7 @@
 				open(zoom.attr.location[0] + '?action=' + zoom.graph.action + '&local_graph_id=' + zoom.graph.local_graph_id + '&rra_id=' + zoom.graph.rra_id + '&view_type=' + zoom.graph.view_type + '&graph_start=' + newGraphStartTime + '&graph_end=' + newGraphEndTime + '&graph_height=' + zoom.graph.height + '&graph_width=' + zoom.graph.width + '&title_font_size=' + zoom.graph.title_font_size, '_self');
 			}
 
+			zoom.attr.start = 'none';
 		}
 
 		/*
@@ -725,6 +758,26 @@
 			}
 		}
 
+		function getZoomOutFactorText(zoomOutFactor) {
+			switch(zoomOutFactor) {
+				case 2:
+					return zoom_i18n_zoom_out + ' (' + zoom_i18n_zoom_2 + ')';
+					break;
+				case 4:
+					return zoom_i18n_zoom_out + ' (' + zoom_i18n_zoom_4 + ')';
+					break;
+				case 8:
+					return zoom_i18n_zoom_out + ' (' + zoom_i18n_zoom_8 + ')';
+					break;
+				case 16:
+					return zoom_i18n_zoom_out + ' (' + zoom_i18n_zoom_16 + ')';
+					break;
+				case 32:
+					return zoom_i18n_zoom_out + ' (' + zoom_i18n_zoom_32 + ')';
+					break;
+			}
+		}
+
 		/*
 		* executes a static zoom out (as right click event)
 		*/
@@ -791,7 +844,9 @@
 				$('#' + zoom.options.inputfieldEndTime).val(unixTime2Date(newGraphEndTime));
 
 				if (graph_start !== null && graph_end !== null) {
-					if (myGraphLocation != 'graph') {
+					zoom.attr.start = 'none';
+
+					if (pageAction != 'graph') {
 						graph_start = newGraphStartTime;
 						graph_end = newGraphEndTime;
 
@@ -831,7 +886,6 @@
 		* updates the css parameters of the zoom area to reflect user's interaction
 		*/
 		function zoomAction_draw(event) {
-
 			if (zoom.attr.start == 'none') { return; }
 
 			/* mouse has been moved from right to left */
@@ -860,7 +914,7 @@
 			$('.zoomContextMenuAction__set_zoomOutFactor__' + zoom.custom.zoomOutFactor).addClass('zoom-menu-highlight');
 			$('.zoomContextMenuAction__set_zoomOutPositioning__' + zoom.custom.zoomOutPositioning).addClass('zoom-menu-highlight');
 			$('.zoomContextMenuAction__set_zoom3rdMouseButton__' + ((zoom.custom.zoom3rdMouseButton === false) ? 'off' : zoom.custom.zoom3rdMouseButton) ).addClass('zoom-menu-highlight');
-			$('.zoomContextMenuAction__zoom_out').text('Zoom Out (' + zoom.custom.zoomOutFactor + 'x)');
+			$('.zoomContextMenuAction__zoom_out').text(getZoomOutFactorText(zoom.custom.zoomOutFactor));
 
 			if (zoom.custom.zoomMode == 'quick') {
 				$('.advanced_mode').hide();
@@ -900,7 +954,7 @@
 			$('.first_li , .sec_li, .inner_li span').hover(
 				function () {
 					$(this).addClass('zoom-menu-hover');
-					if ( $(this).children().size() >0 )
+					if ( $(this).children().length >0 )
 						if (zoom.custom.zoomMode == 'quick') {
 							$(this).children('.inner_li:not(.advanced_mode)').show();
 						} else {
@@ -924,6 +978,7 @@
 				case 'mode':
 					if ( zoom.custom.zoomMode != value) {
 						zoom.custom.zoomMode = value;
+
 						$('[class*=zoomContextMenuAction__set_zoomMode__]').toggleClass('zoom-menu-highlight');
 
 						if (value == 'quick') {
@@ -939,6 +994,7 @@
 							zoom.custom.zoomMode			= 'advanced';
 							storage.set(zoom.options.cookieName, serialize(zoom.custom));
 						}
+
 						zoomElements_reset();
 						zoomAction_init(zoom.initiator);
 
@@ -965,7 +1021,7 @@
 						storage.set(zoom.options.cookieName, serialize(zoom.custom));
 						$('[class*=zoomContextMenuAction__set_zoomOutFactor__]').removeClass('zoom-menu-highlight');
 						$('.zoomContextMenuAction__set_zoomOutFactor__' + value).addClass('zoom-menu-highlight');
-						$('.zoomContextMenuAction__zoom_out').text('Zoom Out (' + value + 'x)');
+						$('.zoomContextMenuAction__zoom_out').text(getZoomOutFactorText(value));
 					}
 					break;
 				case 'outpositioning':
@@ -991,15 +1047,29 @@
 			switch(action) {
 				case 'close':
 					zoomContextMenu_hide();
+
 					break;
 				case 'zoom_out':
 					if (value == undefined) {
 						value = zoom.custom.zoomOutFactor;
 					}
 					zoomAction_zoom_out(value);
+
 					break;
 				case 'zoom_in':
 					zoomAction_zoom_in();
+
+					break;
+				case 'copy':
+					$('#zoom-textarea').html('<img src="data:image/png;base64,'+btoa(unescape(encodeURIComponent(zoom.image.data)))+'" width="'+zoom.image.width+'" height="'+zoom.image.height+'">').select();
+
+					try {
+						var successful = document.execCommand('copy');
+					} catch (err) {
+						alert('Unsupported Browser');
+					}
+					return false;
+
 					break;
 				case 'save':
 					var arraybuffer = new ArrayBuffer(zoom.image.data.length);
@@ -1022,6 +1092,7 @@
 						var objectUrl = URL.createObjectURL(blob);
 						$('#zoom-image').removeAttr('target').attr({'download':zoom.image.name, 'href':objectUrl }).get(0).click();
 					}
+
 					break;
 				case 'newTab':
 					var url = zoom.attr.urlPath + 'graph_image.php?local_graph_id=' + zoom.image.id;
@@ -1030,6 +1101,7 @@
 					}
 					url += '&graph_start=' + zoom.graph.start + '&graph_end=' + zoom.graph.end + '&graph_width=' + zoom.graph.width + '&graph_height=' + zoom.graph.height + ( (zoom.image.legend === true) ? '' : '&graph_nolegend=true' );
 					$('#zoom-image').removeAttr('download').attr({ 'href':url, 'target': '_bank' }).get(0).click();
+
 					break;
 				case 'link':
 					var url = zoom.attr.origin + ((zoom.attr.urlPath == '') ? '/' : zoom.attr.urlPath) + 'graph_image.php?local_graph_id=' + zoom.image.id + '&graph_start=' + zoom.graph.start + '&graph_end=' + zoom.graph.end + '&graph_width=' + zoom.graph.width + '&graph_height=' + zoom.graph.height + ( (zoom.image.legend === true) ? '' : '&graph_nolegend=true' );
@@ -1040,12 +1112,12 @@
 						alert('Unsupported Browser');
 					}
 					return false;
+
 					break;
 			}
 		}
 
 		function zoomContextMenu_show(e){
-
 			var menu_y_pos			= e.pageY;
 			var menu_y_offset		= 5;
 			var menu_x_pos			= e.pageX;
@@ -1086,6 +1158,10 @@
 			$('#zoom-menu').hide();
 		}
 
+		function zoomContextMenu_toggle(e){
+			($('#zoom-menu').css('display') == 'none') ? zoomContextMenu_show(e) : zoomContextMenu_hide();
+		}
+
 	};
 
 })(jQuery);
