@@ -1,7 +1,7 @@
 <?php
 /*
  +-------------------------------------------------------------------------+
- | Copyright (C) 2004-2018 The Cacti Group                                 |
+ | Copyright (C) 2004-2020 The Cacti Group                                 |
  |                                                                         |
  | This program is free software; you can redistribute it and/or           |
  | modify it under the terms of the GNU General Public License             |
@@ -22,19 +22,32 @@
  +-------------------------------------------------------------------------+
 */
 
-function upgrade_to_0_8_1() {
-	db_install_add_column ('user_log', array('name' => 'user_id', 'type' => 'mediumint(8)', 'NULL' => false, 'after' => 'username'));
-	db_install_execute("ALTER TABLE user_log change time time datetime not null;");
-	db_install_execute("ALTER TABLE user_log drop primary key;");
-	db_install_execute("ALTER TABLE user_log add primary key (username, user_id, time);");
-	db_install_add_column ('user_auth', array('name' => 'realm', 'type' => 'mediumint(8)', 'NULL' => false, 'after' => 'password'));
-	db_install_execute("UPDATE user_auth set realm = 1 where full_name='ldap user';");
+function upgrade_to_1_1_17() {
+	// Finalize fix to LDAP authentication
+	db_install_execute('UPDATE user_auth SET realm=3 WHERE realm=1');
 
-	$_src = db_fetch_assoc("select id, username from user_auth");
+	if (!db_column_exists('data_source_profiles_rra', 'timespan')) {
+		db_install_execute('ALTER TABLE data_source_profiles_rra
+			ADD COLUMN timespan int(10) unsigned NOT NULL DEFAULT "0"');
 
-	if (sizeof($_src) > 0) {
-		foreach ($_src as $item) {
-			db_install_execute("UPDATE user_log set user_id = " . $item["id"] . " where username = '" . $item["username"] . "';");
+		$rras = db_fetch_assoc("SELECT * FROM data_source_profiles_rra");
+
+		if (cacti_sizeof($rras)) {
+			foreach($rras as $rra) {
+				$interval = db_fetch_cell_prepared('SELECT step
+					FROM data_source_profiles
+					WHERE id = ?',
+					array($rra['data_source_profile_id']));
+
+				$timespan = $rra['steps'] * $interval * $rra['rows'];
+
+				$timespan = get_nearest_timespan($timespan);
+
+				db_install_execute('UPDATE data_source_profiles_rra
+					SET timespan = ?
+					WHERE id = ?',
+					array($timespan, $rra['id']));
+			}
 		}
 	}
 }
