@@ -1,7 +1,7 @@
 <?php
 /*
  +-------------------------------------------------------------------------+
- | Copyright (C) 2004-2018 The Cacti Group                                 |
+ | Copyright (C) 2004-2020 The Cacti Group                                 |
  |                                                                         |
  | This program is free software; you can redistribute it and/or           |
  | modify it under the terms of the GNU General Public License             |
@@ -31,27 +31,24 @@ function boost_file_size_display($file_size, $digits = 2) {
 
 			if ($file_size > 1024) {
 				$file_size = $file_size / 1024;
-				$file_suffix = ' GBytes';
+				return __('%s GBytes', number_format_i18n($file_size, $digits));
 			} else {
-				$file_suffix = ' MBytes';
+				return __('%s MBytes', number_format_i18n($file_size, $digits));
 			}
 		} else {
-			$file_suffix = ' KBytes';
+			return __('%s KBytes', number_format_i18n($file_size, $digits));
 		}
 	} else {
-		$file_suffix = ' Bytes';
+		return __('%s Bytes', number_format_i18n($file_size, $digits));
 	}
-
-	$file_size = number_format_i18n($file_size, $digits) . $file_suffix;
-
-	return $file_size;
 }
 
 function boost_get_total_rows() {
-	return db_fetch_cell("SELECT SUM(TABLE_ROWS) FROM INFORMATION_SCHEMA.TABLES
-		WHERE table_schema=SCHEMA()
-		AND (table_name LIKE 'poller_output_boost_arch_%' OR table_name LIKE 'poller_output_boost')
-		GROUP BY table_schema");
+	return db_fetch_cell("SELECT SUM(TABLE_ROWS)
+		FROM information_schema.tables
+		WHERE table_schema = SCHEMA()
+		AND (table_name LIKE 'poller_output_boost_arch_%'
+		OR table_name LIKE 'poller_output_boost')");
 }
 
 function boost_error_handler($errno, $errmsg, $filename, $linenum, $vars) {
@@ -113,9 +110,17 @@ function boost_check_correct_enabled() {
 }
 
 function boost_poller_on_demand(&$results) {
-	global $config;
+	global $config, $remote_db_cnn_id;
+
+	if ($config['poller_id'] > 1 && $config['connection'] == 'online') {
+		$conn = $remote_db_cnn_id;
+	} else {
+		$conn = false;
+	}
+
+	if (read_config_option('boost_rrd_update_enable') == 'on' || $config['poller_id'] > 1) {
+		set_config_option('boost_rrd_update_enable', 'on');
 
-	if (read_config_option('boost_rrd_update_enable') == 'on') {
 		/* suppress warnings */
 		if (defined('E_DEPRECATED')) {
 			error_reporting(E_ALL ^ E_DEPRECATED);
@@ -126,10 +131,12 @@ function boost_poller_on_demand(&$results) {
 		/* install the boost error handler */
 		set_error_handler('boost_error_handler');
 
-		$outbuf      = '';
+		$out_buffer  = '';
 		$sql_prefix  = 'INSERT INTO poller_output_boost (local_data_id, rrd_name, time, output) VALUES ';
 		$sql_suffix  = ' ON DUPLICATE KEY UPDATE output=VALUES(output)';
-		$overhead    = strlen($sql_prefix) + strlen($sql_suffix);
+
+		// Add 1 here for potential delimiter
+		$overhead    = strlen($sql_prefix) + strlen($sql_suffix) + 1;
 
 		if (boost_check_correct_enabled()) {
 			/* if boost redirect is on, rows are being inserted directly */
@@ -141,39 +148,56 @@ function boost_poller_on_demand(&$results) {
 			$max_allowed_packet = db_fetch_row("SHOW VARIABLES LIKE 'max_allowed_packet'");
 			$max_allowed_packet = $max_allowed_packet['Value'];
 
-			if (sizeof($results)) {
-				$i          = 1;
+			if (cacti_sizeof($results)) {
+				$delim      = '';
+				$delim_len  = 0;
 				$out_length = 0;
 
 				foreach($results as $result) {
-					if ($i == 1) {
-						$delim = '';
-					} else {
-						$delim = ',';
-					}
-
-					$outbuf2 =
-						$delim . "('" .
+					$tmp_buffer =
+						"('" .
 						$result['local_data_id'] . "','" .
 						$result['rrd_name'] . "','" .
 						$result['time'] . "','" .
 						$result['output'] .	"')";
 
-					$out_length += strlen($outbuf2);
+					$tmp_length = strlen($tmp_buffer);
 
-					if (($out_length + $overhead) > $max_allowed_packet) {
-						db_execute($sql_prefix . $outbuf . $sql_suffix);
-						$outbuf     = $outbuf2;
-						$out_length = strlen($outbuf2);
-						$i          = 1;
+					// Calculate length of output buffer, plus overhead, plus the temp buffer
+					// is it greater than what SQL allows?
+					if (($out_length + $overhead + $tmp_length) > $max_allowed_packet) {
+
+						// Overall length was greater, but do we actually have anything
+						// already buffered? Or was it just the temp buffer that overflowed
+						// things?
+						if ($out_length > 0) {
+							db_execute($sql_prefix . $out_buffer . $sql_suffix, true, $conn);
+						}
+
+						// Make the temp buffer the starting point for the output buffer, but
+						// we don't need a delimiter at this point, so don't include it
+						$out_buffer = $tmp_buffer;
+						$out_length = $tmp_length;
 					} else {
-						$outbuf .= $outbuf2;
-						$i++;
+						// We didn't overflow so lets add the temp buffer to the output buffer
+						// and include the delimiter string/length.  This will be a blank
+						// delimiter on the first iteration as the output buffer will always
+						// be blank.
+						$out_buffer .= $delim . $tmp_buffer;
+						$out_length += $delim_len + $tmp_length;
+					}
+
+					// Only on the first iteration do we need to set the delimiter as
+					// after that, we will always need it when we are not overflowing
+					if ($delim_len == 0) {
+						$delim     = ',';
+						$delim_len = strlen($delim);
 					}
 				}
 
-				if ($outbuf != '') {
-					db_execute($sql_prefix . $outbuf . $sql_suffix);
+				// output buffer had something left, lets flush it
+				if ($out_buffer != '') {
+					db_execute($sql_prefix . $out_buffer . $sql_suffix, true, $conn);
 				}
 			}
 
@@ -213,14 +237,14 @@ function boost_poller_id_check() {
 	return true;
 }
 
-function boost_fetch_cache_check($local_data_id) {
-	global $config, $rrdtool_pipe;
+function boost_fetch_cache_check($local_data_id, $rrdtool_pipe = false) {
+	global $config;
 
 	if (read_config_option('boost_rrd_update_enable') == 'on') {
 		/* include poller processing routinges */
 		include_once($config['library_path'] . '/poller.php');
 
-		/* check to see if boost can do it's job */
+		/* check to see if boost can do its job */
 		if (!boost_poller_id_check()) {
 			return false;
 		}
@@ -236,18 +260,25 @@ function boost_fetch_cache_check($local_data_id) {
 		set_error_handler('boost_error_handler');
 
 		/* process input parameters */
-		$rrdtool_pipe = rrd_init();
+		if (!is_resource($rrdtool_pipe)) {
+			$rrdtool_pipe = rrd_init();
+			$close_pipe = true;
+		} else {
+			$close_pipe = false;
+		}
 
 		/* get the information to populate into the rrd files */
 		if (boost_check_correct_enabled()) {
-			boost_process_poller_output($local_data_id);
+			boost_process_poller_output($local_data_id, $rrdtool_pipe);
 		}
 
 		/* restore original error handler */
 		restore_error_handler();
 
 		/* close rrdtool */
-		rrd_close($rrdtool_pipe);
+		if ($close_pipe) {
+			rrd_close($rrdtool_pipe);
+		}
 	}
 }
 
@@ -281,7 +312,7 @@ function boost_graph_cache_check($local_graph_id, $rra_id, $rrdtool_pipe, &$grap
 	/* install the boost error handler */
 	set_error_handler('boost_error_handler');
 
-	/* check to see if boost can do it's job */
+	/* check to see if boost can do its job */
 	if (!boost_poller_id_check()) {
 		return false;
 	}
@@ -315,11 +346,12 @@ function boost_graph_cache_check($local_graph_id, $rra_id, $rrdtool_pipe, &$grap
 			AND data_template_rrd.local_data_id > 0', array($local_graph_id));
 
 		/* first update the RRD files */
-		if (sizeof($local_data_ids)) {
+		if (cacti_sizeof($local_data_ids)) {
 			$updates = 0;
 			foreach($local_data_ids as $local_data_id) {
 				$updates += boost_process_poller_output($local_data_id['local_data_id'], $rrdtool_pipe);
 			}
+
 			if ($updates) {
 				/* restore original error handler */
 				restore_error_handler();
@@ -476,6 +508,7 @@ function boost_graph_set_file(&$output, $local_graph_id, $rra_id) {
 				if (isset($graph_data_array['graph_height'])) {
 					$cache_file .= '_height_' . $graph_data_array['graph_height'];
 				}
+
 				if (isset($graph_data_array['graph_width'])) {
 					$cache_file .= '_width_' . $graph_data_array['graph_width'];
 				}
@@ -553,15 +586,13 @@ function boost_timer_get_overhead() {
 
 /* boost_get_arch_table_name - returns current archive boost table or false if no arch table is present currently */
 function boost_get_arch_table_name() {
-	global $database_default;
-
 	$tables = db_fetch_assoc("SELECT table_name AS name
 		FROM information_schema.tables
-		WHERE table_schema = '$database_default'
+		WHERE table_schema = SCHEMA()
 		AND table_name LIKE 'poller_output_boost_arch_%'");
 
 	foreach($tables as $table) {
-		$rows = db_fetch_cell('SELECT count(*) FROM '.$table['name']);
+		$rows = db_fetch_cell('SELECT COUNT(local_data_id) FROM ' . $table['name']);
 		if (is_numeric($rows) && intval($rows) > 0) {
 			return $table['name'];
 		}
@@ -576,6 +607,7 @@ function boost_get_arch_table_name() {
 function boost_process_poller_output($local_data_id = '', $rrdtool_pipe = '') {
 	global $config, $database_default, $boost_sock, $boost_timeout, $debug, $get_memory, $memory_used;
 
+	static $archive_table = false;
 	static $warning_issued;
 
 	include_once($config['library_path'] . '/rrd.php');
@@ -590,95 +622,55 @@ function boost_process_poller_output($local_data_id = '', $rrdtool_pipe = '') {
 	/* install the boost error handler */
 	set_error_handler('boost_error_handler');
 
-	/* tiny SQL where addendum for boost */
-	if ($local_data_id != '') {
-		/* we can simplify the delete process if only one local_data_id */
-		$single_local_data_id = true;
-		$orig_local_data_id   = $local_data_id;
+	/* we can simplify the delete process if only one local_data_id */
+	$orig_local_data_id   = $local_data_id;
 
-		/* aquire lock in order to prevent race conditions */
-		while (!db_fetch_cell("SELECT GET_LOCK('boost.single_ds.$orig_local_data_id', 1)")) {
-			usleep(50000);
-		}
-	} else {
-		$single_local_data_id = false;
-
-		$archive_table = boost_get_arch_table_name();
-		if ($archive_table === false) {
-			if ($warning_issued != true) {
-				cacti_log('NOTE: Failed to determine archive table', false, 'BOOST', POLLER_VERBOSITY_MEDIUM);
-			}
-
-			$warning_issued = true;
-
-			return 0;
-		}
+	/* aquire lock in order to prevent race conditions */
+	while (!db_fetch_cell("SELECT GET_LOCK('boost.single_ds.$orig_local_data_id', 1)")) {
+		usleep(50000);
 	}
 
 	$data_ids_to_get = read_config_option('boost_rrd_update_max_records_per_select');
 
-	/* get the records */
-	if ($single_local_data_id) {
-		$query_string = '';
-		$arch_tables  = db_fetch_assoc("SELECT table_name AS name
-			FROM information_schema.tables
-			WHERE table_schema = '$database_default'
-			AND table_name LIKE 'poller_output_boost_arch_%'");
+	$query_string = '';
+	$arch_tables  = db_fetch_assoc("SELECT table_name AS name
+		FROM information_schema.tables
+		WHERE table_schema = SCHEMA()
+		AND table_name LIKE 'poller_output_boost_arch_%'");
 
-		if (count($arch_tables)) {
-			foreach($arch_tables as $table) {
-				$rows = db_fetch_cell('SELECT COUNT(*) FROM ' . $table['name']);
-				if (db_table_exists($table['name']) && is_numeric($rows) && intval($rows) > 0) {
-					if ($query_string != '') {
-						$query_string .= ' UNION ';
-					}
+	if (cacti_count($arch_tables)) {
+		foreach($arch_tables as $table) {
+			$rows = db_fetch_cell('SELECT COUNT(local_data_id
+				FROM ' . $table['name'] . '
+				WHERE local_data_id = ' . $local_data_id, '', false);
 
-					$query_string .= ' (SELECT local_data_id, UNIX_TIMESTAMP(time) AS timestamp, rrd_name, output
-						FROM ' . $table['name'] . "
-						WHERE local_data_id = $local_data_id) ";
+			if (db_table_exists($table['name']) && is_numeric($rows) && intval($rows) > 0) {
+				if ($query_string != '') {
+					$query_string .= ' UNION ';
 				}
+
+				$query_string .= ' (SELECT local_data_id, UNIX_TIMESTAMP(time) AS timestamp, rrd_name, output
+					FROM ' . $table['name'] . "
+					WHERE local_data_id = $local_data_id) ";
 			}
 		}
-
-		if ($query_string != '') {
-			$query_string .= ' UNION ';
-		}
-
-		$timestamp = time();
-
-		$query_string .= " (SELECT local_data_id, UNIX_TIMESTAMP(time) AS timestamp, rrd_name, output
-			FROM poller_output_boost
-			WHERE local_data_id = $orig_local_data_id
-			AND time < FROM_UNIXTIME($timestamp))";
-
-		$query_string .= ' ORDER BY timestamp ASC, rrd_name ASC ';
-	} else {
-		$query_string = '';
-		$arch_tables  = db_fetch_assoc("SELECT table_name AS name
-			FROM information_schema.tables
-			WHERE table_schema = '$database_default'
-			AND table_name LIKE 'poller_output_boost_arch_%'");
-
-		if (count($arch_tables)) {
-			foreach($arch_tables as $table) {
-				$rows = db_fetch_cell('SELECT COUNT(*) FROM ' . $table['name']);
-				if (db_table_exists($table['name']) && is_numeric($rows) && intval($rows) > 0) {
-					if ($query_string != '') {
-						$query_string .= ' UNION ';
-					}
-
-					$query_string .= ' (SELECT local_data_id, UNIX_TIMESTAMP(time) AS timestamp, rrd_name, output
-						FROM ' . $table['name'] . ') ';
-				}
-			}
-		}
-
-		$query_string .= "ORDER BY local_data_id ASC, time ASC, rrd_name ASC
-			LIMIT $data_ids_to_get ";
 	}
 
+	if ($query_string != '') {
+		$query_string .= ' UNION ';
+	}
+
+	$timestamp = time();
+
+	$query_string .= " (SELECT local_data_id, UNIX_TIMESTAMP(time) AS timestamp, rrd_name, output
+		FROM poller_output_boost
+		WHERE local_data_id = $orig_local_data_id
+		AND time < FROM_UNIXTIME($timestamp))";
+
+	$query_string .= ' ORDER BY timestamp ASC, rrd_name ASC ';
+
 	boost_timer('get_records', BOOST_TIMER_START);
-	$results = db_fetch_assoc($query_string);
+	$results = db_fetch_assoc($query_string, false);
 	boost_timer('get_records', BOOST_TIMER_END);
 
 	/* log memory */
@@ -690,17 +682,7 @@ function boost_process_poller_output($local_data_id = '', $rrdtool_pipe = '') {
 		}
 	}
 
-	if ($single_local_data_id) {
-		$log_verbosity = POLLER_VERBOSITY_DEBUG;
-
-		if ($debug) {
-			$log_verbosity = POLLER_VERBOSITY_LOW;
-		}
-
-		cacti_log("NOTE: Updating Local Data ID:'$local_data_id', Total of '" . sizeof($results) . "' Updates to Process", false, 'BOOST', $log_verbosity);
-	}
-
-	if (sizeof($results)) {
+	if (cacti_sizeof($results)) {
 		$rrdp_auto_close = false;
 
 		if (!$rrdtool_pipe) {
@@ -722,37 +704,12 @@ function boost_process_poller_output($local_data_id = '', $rrdtool_pipe = '') {
 		/* we are going to blow away all record if ok */
 		$vals_in_buffer = 0;
 
-		/* cut last DS (likely to be incomplete due to LIMIT)*/
-		if (!$single_local_data_id) {
-			reset($results);
-
-			$first_ds = $results[key($results)];
-			$first_ds = intval($first_ds['local_data_id']);
-
-			end($results);
-
-			$last_ds = $results[key($results)];
-			$last_ds = intval($last_ds['local_data_id']);
-
-			reset($results);
-
-			if ($first_ds == $last_ds) {
-				if (sizeof($results) == $data_ids_to_get) {
-					cacti_log("FALURE: Current LIMIT ($data_ids_to_get) is too low to run multiple DS RRD writes, consider raising it", false, 'BOOST');
-				}
-
-				restore_error_handler();
-
-				return boost_process_poller_output($first_ds, $rrdtool_pipe);
-			}
-		}
-
 		$upd_string_len = read_config_option('boost_rrd_update_string_length');
 
 		/* initialize some variables */
 		$rrd_tmpl     = '';
 		$rrd_path     = '';
-		$initial_time = '';
+		$initial_time = 0;
 		$outlen       = 0;
 
 		boost_timer('results_cycle', BOOST_TIMER_START);
@@ -761,27 +718,17 @@ function boost_process_poller_output($local_data_id = '', $rrdtool_pipe = '') {
 		foreach ($results as $item) {
 			$item['timestamp'] = trim($item['timestamp']);
 
-			if ($single_local_data_id && /* duplicate entry, possible when fetching data while rotating poller_output_boost */
-				$last_item['local_data_id'] == $item['local_data_id'] &&
+			if ($last_item['local_data_id'] == $item['local_data_id'] &&
 				$last_item['timestamp'] == $item['timestamp'] &&
 				strcmp($last_item['rrd_name'], $item['rrd_name']) == 0) {
 
 				continue;
 			}
 
-			if (!$single_local_data_id &&
-				$first_ds != $last_ds &&
-				$last_ds == $item['local_data_id']) {
-				/* we faced last and possibly incomplete DS, bail out */
-				break;
-			}
-
 			/* if the local_data_id changes, we need to flush the buffer */
 			if ($local_data_id != $item['local_data_id']) {
 				/* update the rrd for the previous local_data_id */
 				if ($vals_in_buffer) {
-					cacti_log("NOTE: Updating Local Data Id:'$local_data_id', Template:" . $rrd_tmpl . ', Output:' . $outbuf, false, 'BOOST', POLLER_VERBOSITY_MEDIUM);
-
 					boost_timer('rrdupdate', BOOST_TIMER_START);
 					$return_value = boost_rrdtool_function_update($local_data_id, $rrd_path, $rrd_tmpl, $initial_time, $outbuf, $rrdtool_pipe);
 					boost_timer('rrdupdate', BOOST_TIMER_END);
@@ -789,7 +736,7 @@ function boost_process_poller_output($local_data_id = '', $rrdtool_pipe = '') {
 					$vals_in_buffer = 0;
 
 					/* check return status for delete operation */
-					if (strpos(trim($return_value), 'OK') === false) {
+					if (strpos(trim($return_value), 'OK') === false && $return_value != '') {
 						cacti_log("WARNING: RRD Update Warning '" . $return_value . "' for Local Data ID '$local_data_id'", false, 'BOOST');
 					}
 				}
@@ -808,15 +755,21 @@ function boost_process_poller_output($local_data_id = '', $rrdtool_pipe = '') {
 				$local_data_id  = $item['local_data_id'];
 				$time           = $item['timestamp'];
 				$initial_time   = $time;
-				$output         = ' ' . $time;
+
+				if ($time < $last_update && cacti_version_compare(get_rrdtool_version(), '1.5', '<')) {
+					$output = '';
+				} else {
+					$output = ' ' . $time;
+				}
+
 				$outbuf         = $output;
 				$outlen         = strlen($output);
 				$multi_vals_set = false;
 			}
 
 			/* don't generate error messages if the RRD has already been updated */
-			if ($time <= $last_update){
-				cacti_log("WARNING: Stale Poller Data Found! Item Time:'" . $time . "', RRD Time:'" . $last_update . "' Ignoring Value!", false, 'BOOST');
+			if ($time < $last_update && cacti_version_compare(get_rrdtool_version(), '1.5', '<')) {
+				cacti_log("WARNING: Stale Poller Data Found! Item Time:'" . $time . "', RRD Time:'" . $last_update . "' Ignoring Value!", false, 'BOOST', POLLER_VERBOSITY_HIGH);
 				$value = 'DNP';
 			} else {
 				$value = trim($item['output']);
@@ -824,8 +777,6 @@ function boost_process_poller_output($local_data_id = '', $rrdtool_pipe = '') {
 
 			if ($time != $item['timestamp']) {
 				if ($outlen > $upd_string_len) {
-					cacti_log("NOTE: Updating Local Data Id:'$local_data_id', Template:" . $rrd_tmpl . ', Output:' . $outbuf, false, 'BOOST', POLLER_VERBOSITY_MEDIUM);
-
 					boost_timer('rrdupdate', BOOST_TIMER_START);
 					$return_value = boost_rrdtool_function_update($local_data_id, $rrd_path, $rrd_tmpl, $initial_time, $outbuf, $rrdtool_pipe);
 					boost_timer('rrdupdate', BOOST_TIMER_END);
@@ -835,21 +786,24 @@ function boost_process_poller_output($local_data_id = '', $rrdtool_pipe = '') {
 					$vals_in_buffer = 0;
 
 					/* check return status for delete operation */
-					if (strpos(trim($return_value), 'OK') === false) {
+					if (strpos(trim($return_value), 'OK') === false && $return_value != '') {
 						cacti_log("WARNING: RRD Update Warning '" . $return_value . "' for Local Data ID '$local_data_id'", false, 'BOOST');
 					}
 				}
 
-				$output  = ' ' . $item['timestamp'];
-				$outbuf .= $output;
-				$outlen += strlen($output);
+				if (strpos($value, 'DNP') === false) {
+					$output  = ' ' . $item['timestamp'];
+					$outbuf .= $output;
+					$outlen += strlen($output);
+				}
+
 				$time    = $item['timestamp'];
 			}
 
 			/* single one value output */
-			if (strcmp($value, 'DNP') == 0) {
+			if (strpos($value, 'DNP') !== false) {
 				/* continue, bad time */
-			} elseif ((is_numeric($value)) || (strcmp($value, 'U') == 0)) {
+			} elseif ((is_numeric($value)) || ($value == 'U')) {
 				$output  = ':' . $value;
 				$outbuf .= $output;
 				$outlen += strlen($output);
@@ -877,19 +831,17 @@ function boost_process_poller_output($local_data_id = '', $rrdtool_pipe = '') {
 				$first_tmpl = true;
 				$multi_ok   = false;
 
-				for ($i=0; $i<count($values); $i++) {
-					if (preg_match("/^([a-zA-Z0-9_\.-]+):([eE0-9\+\.-]+)$/", $values[$i], $matches)) {
+				for ($i=0; $i<cacti_count($values); $i++) {
+					if (preg_match("/^([a-zA-Z0-9_\.-]+):([eE0-9Uu\+\.-]+)$/", $values[$i], $matches)) {
 						if (isset($rrd_field_names[$matches[1]])) {
 							$multi_ok = true;
 
-							cacti_log("Parsed MULTI output field '" . $matches[0] . "' [map " . $matches[1] . '->' . $rrd_field_names[$matches[1]] . ']' , false, 'BOOST', POLLER_VERBOSITY_DEBUG);
-
 							if (!$multi_vals_set) {
 								if (!$first_tmpl) {
 									$rrd_tmpl .= ':';
 								}
 
-								$rrd_tmpl .= $rrd_field_names[$matches[1]];
+								$rrd_tmpl  .= $rrd_field_names[$matches[1]];
 								$first_tmpl = false;
 							}
 
@@ -925,14 +877,12 @@ function boost_process_poller_output($local_data_id = '', $rrdtool_pipe = '') {
 
 		/* process the last rrdupdate if applicable */
 		if ($vals_in_buffer) {
-			cacti_log("NOTE: Updating Local Data Id:'$local_data_id', Template:" . $rrd_tmpl . ', Output:' . $outbuf, false, 'BOOST', POLLER_VERBOSITY_MEDIUM);
-
 			boost_timer('rrdupdate', BOOST_TIMER_START);
 			$return_value = boost_rrdtool_function_update($local_data_id, $rrd_path, $rrd_tmpl, $initial_time, $outbuf, $rrdtool_pipe);
 			boost_timer('rrdupdate', BOOST_TIMER_END);
 
 			/* check return status for delete operation */
-			if (strpos(trim($return_value), 'OK') === false) {
+			if (strpos(trim($return_value), 'OK') === false && $return_value != '') {
 				cacti_log("WARNING: RRD Update Warning '" . $return_value . "' for Local Data ID '$local_data_id'", false, 'BOOST');
 			}
 		}
@@ -941,29 +891,20 @@ function boost_process_poller_output($local_data_id = '', $rrdtool_pipe = '') {
 
 		/* remove the entries from the table */
 		boost_timer('delete', BOOST_TIMER_START);
-		if ($single_local_data_id) {
-			if (count($arch_tables)) {
-				foreach($arch_tables as $table) {
-					db_execute_prepared('DELETE FROM ' . $table['name'] . '
-						WHERE local_data_id = ?',
-						array($local_data_id));
-				}
-			}
 
-			db_execute_prepared('DELETE FROM poller_output_boost
-				WHERE local_data_id = ?
-				AND time < FROM_UNIXTIME(?)',
-				array($local_data_id, $timestamp));
-		} else {
-			if (count($arch_tables)) {
-				foreach($arch_tables as $table) {
-					db_execute_prepared('DELETE FROM ' . $table['name'] . '
-						WHERE local_data_id BETWEEN ? AND ?',
-						array($first_ds, $last_ds));
-				}
+		if (cacti_count($arch_tables)) {
+			foreach($arch_tables as $table) {
+				db_execute_prepared('DELETE FROM ' . $table['name'] . '
+					WHERE local_data_id = ?',
+					array($local_data_id));
 			}
 		}
 
+		db_execute_prepared('DELETE FROM poller_output_boost
+			WHERE local_data_id = ?
+			AND time < FROM_UNIXTIME(?)',
+			array($local_data_id, $timestamp));
+
 		boost_timer('delete', BOOST_TIMER_END);
 
 		if ($rrdp_auto_close) {
@@ -971,14 +912,12 @@ function boost_process_poller_output($local_data_id = '', $rrdtool_pipe = '') {
 		}
 	}
 
-	if ($single_local_data_id) {
-		db_execute("SELECT RELEASE_LOCK('boost.single_ds.$orig_local_data_id')");
-	}
+	db_execute("SELECT RELEASE_LOCK('boost.single_ds.$orig_local_data_id')");
 
 	/* restore original error handler */
 	restore_error_handler();
 
-	return sizeof($results);
+	return cacti_sizeof($results);
 }
 
 function boost_rrdtool_get_last_update_time($rrd_path, &$rrdtool_pipe) {
@@ -986,12 +925,12 @@ function boost_rrdtool_get_last_update_time($rrd_path, &$rrdtool_pipe) {
 
 	if (read_config_option('storage_location')) {
 		$file_exists = rrdtool_execute("file_exists $rrd_path" , true, RRDTOOL_OUTPUT_BOOLEAN, $rrdtool_pipe, 'BOOST');
-	}else {
+	} else {
 		$file_exists = file_exists($rrd_path);
 	}
 
 	if ($file_exists == true) {
-		$return_value = rrdtool_execute("last $rrd_path", true, RRDTOOL_OUTPUT_STDOUT, $rrdtool_pipe, 'BOOST');
+		$return_value = rrdtool_execute("last $rrd_path", true, RRDTOOL_OUTPUT_STDOUT, false, 'BOOST');
 	}
 
 	return trim($return_value);
@@ -1032,7 +971,7 @@ function boost_determine_caching_state() {
 
 /* boost_get_rrd_filename_and_template - pulls
    1) the rrd_update template from the database in form of
-      update decisions for multi-output RRD's
+      update decisions for multi-output RRDs
    2) rrd filename
    @arg $local_data_id - the data source to obtain information from */
 function boost_get_rrd_filename_and_template($local_data_id) {
@@ -1049,7 +988,7 @@ function boost_get_rrd_filename_and_template($local_data_id) {
 		WHERE dtr.local_data_id = ?
 		ORDER BY data_source_name ASC", array($local_data_id));
 
-	if (sizeof($ds_names)) {
+	if (cacti_sizeof($ds_names)) {
 		foreach($ds_names as $ds_name) {
 			if ($rrd_path == '') {
 				$rrd_path = $ds_name['rrd_path'];
@@ -1123,7 +1062,7 @@ function boost_rrdtool_function_create($local_data_id, $initial_time, $show_sour
 	/* if we find that this DS has no RRA associated; get out.  This would
 	 * indicate that a data sources has been deleted
 	 */
-	if (sizeof($rras) <= 0) {
+	if (cacti_sizeof($rras) <= 0) {
 		return false;
 	}
 
@@ -1148,7 +1087,7 @@ function boost_rrdtool_function_create($local_data_id, $initial_time, $show_sour
 	- There is multiple data sources and this item is not the main one.
 	- There is only one data source (then use it) */
 
-	if (sizeof($data_sources)) {
+	if (cacti_sizeof($data_sources)) {
 		foreach ($data_sources as $data_source) {
 			/* use the cacti ds name by default or the user defined one, if entered */
 			$data_source_name = get_data_source_item_name($data_source['id']);
@@ -1158,19 +1097,11 @@ function boost_rrdtool_function_create($local_data_id, $initial_time, $show_sour
 				$data_source['rrd_maximum'] = 'U';
 			} elseif (strpos($data_source['rrd_maximum'], '|query_') !== false) {
 				$data_local = db_fetch_row_prepared('SELECT * FROM data_local WHERE id = ?', array($local_data_id));
-				if ($data_source['rrd_maximum'] == '|query_ifSpeed|' || $data_source['rrd_maximum'] == '|query_ifHighSpeed|') {
-					$highSpeed = db_fetch_cell_prepared("SELECT field_value
-						FROM host_snmp_cache
-						WHERE host_id = ?
-						AND snmp_query_id = ?
-						AND snmp_index = ?
-						AND field_name = 'ifHighSpeed'", array($data_local['host_id'], $data_local['snmp_query_id'], $data_local['snmp_index']));
 
-					if (!empty($highSpeed)) {
-						$data_source['rrd_maximum'] = $highSpeed * 1000000;
-					} else {
-						$data_source['rrd_maximum'] = substitute_snmp_query_data('|query_ifSpeed|',$data_local['host_id'], $data_local['snmp_query_id'], $data_local['snmp_index']);
-					}
+				$speed = rrdtool_function_interface_speed($data_local);
+
+				if ($data_source['rrd_maximum'] == '|query_ifSpeed|' || $data_source['rrd_maximum'] == '|query_ifHighSpeed|') {
+					$data_source['rrd_maximum'] = $speed;
 				} else {
 					$data_source['rrd_maximum'] = substitute_snmp_query_data($data_source['rrd_maximum'],$data_local['host_id'], $data_local['snmp_query_id'], $data_local['snmp_index']);
 				}
@@ -1256,26 +1187,33 @@ function boost_rrdtool_function_update($local_data_id, $rrd_path, $rrd_update_te
 	}
 
 	if ($file_exists == false) {
-		$valid_entry = boost_rrdtool_function_create($local_data_id, $initial_time, false, $rrdtool_pipe);
+		$ds_exists = db_fetch_cell_prepared('SELECT id FROM data_local WHERE id = ?', array($local_data_id));
+
+		// Check for a Data Source that has been removed
+		if ($ds_exists) {
+			$valid_entry = boost_rrdtool_function_create($local_data_id, $initial_time, false, $rrdtool_pipe);
+		} else {
+			return 'OK';
+		}
+	}
+
+	if (cacti_version_compare(get_rrdtool_version(), '1.5', '>=')) {
+		$update_options='--skip-past-updates';
+	} else {
+		$update_options='';
 	}
 
 	if ($valid_entry) {
-		rrdtool_execute("update $rrd_path --template $rrd_update_template $rrd_update_values", false, RRDTOOL_OUTPUT_STDOUT, $rrdtool_pipe, 'BOOST');
+		rrdtool_execute("update $rrd_path $update_options --template $rrd_update_template $rrd_update_values", false, RRDTOOL_OUTPUT_STDOUT, $rrdtool_pipe, 'BOOST');
 		return 'OK';
 	}
 }
 
-function boost_poller_command_args ($args) {
-	boost_memory_limit();
-
-	return $args;
-}
-
 function boost_memory_limit() {
 	ini_set('memory_limit', read_config_option('boost_poller_mem_limit') . 'M');
 }
 
-function boost_poller_bottom () {
+function boost_poller_bottom() {
 	global $config;
 
 	if (read_config_option('boost_rrd_update_enable') == 'on') {
@@ -1283,20 +1221,24 @@ function boost_poller_bottom () {
 
 		chdir($config['base_path']);
 
+		$redirect_args = '';
+
 		boost_update_snmp_statistics();
 
 		$command_string = read_config_option('path_php_binary');
 		if (read_config_option('path_boost_log') != '') {
 			if ($config['cacti_server_os'] == 'unix') {
-				$extra_args = '-q ' . $config['base_path'] . '/poller_boost.php >> ' . read_config_option('path_boost_log') . ' 2>&1';
+				$extra_args    = '-q ' . $config['base_path'] . '/poller_boost.php --debug';
+				$redirect_args =  '>> ' . read_config_option('path_boost_log') . ' 2>&1';
 			} else {
-				$extra_args = '-q ' . $config['base_path'] . '/poller_boost.php >> ' . read_config_option('path_boost_log');
+				$extra_args    = '-q ' . $config['base_path'] . '/poller_boost.php --debug';
+				$redirect_args = '>> ' . read_config_option('path_boost_log');
 			}
 		} else {
 			$extra_args = '-q ' . $config['base_path'] . '/poller_boost.php';
 		}
 
-		exec_background($command_string, $extra_args);
+		exec_background($command_string, $extra_args, $redirect_args);
 	}
 }
 
@@ -1306,9 +1248,10 @@ function boost_update_snmp_statistics () {
 
 	/* get the boost table status */
 	$boost_table_status = db_fetch_assoc("SELECT *
-		FROM INFORMATION_SCHEMA.TABLES
-		WHERE table_schema=SCHEMA()
-		AND (table_name LIKE 'poller_output_boost_arch_%' OR table_name LIKE 'poller_output_boost')");
+		FROM information_schema.tables
+		WHERE table_schema = SCHEMA()
+		AND (table_name LIKE 'poller_output_boost_arch_%'
+		OR table_name LIKE 'poller_output_boost')");
 
 	$total_data_sources = db_fetch_cell('SELECT COUNT(*) FROM poller_item');
 
