@@ -1,7 +1,7 @@
 <?php
 /*
  +-------------------------------------------------------------------------+
- | Copyright (C) 2004-2018 The Cacti Group                                 |
+ | Copyright (C) 2004-2020 The Cacti Group                                 |
  |                                                                         |
  | This program is free software; you can redistribute it and/or           |
  | modify it under the terms of the GNU General Public License             |
@@ -51,39 +51,66 @@ function clog_get_graphs_from_datasource($local_data_id) {
 		array($local_data_id)), 'id', 'name');
 }
 
-function clog_purge_logfile() {
+function clog_validate_filename(&$file, &$filepath, &$filename, $filecheck = false) {
 	global $config;
 
 	$logfile = read_config_option('path_cactilog');
 	$logbase = basename($logfile);
+
 	if ($logfile == '') {
 		$logfile = $config['base_path'] . '/log/cacti.log';
 	}
 
-	if (get_nfilter_request_var('filename') != '') {
-		if (strpos(get_nfilter_request_var('filename'), $logbase) === false) {
-			raise_message('clog_invalid');
-			header('Location: ' . get_current_page() . '?filename=' . $logbase);
-			exit(0);
-		}
+	$errfile = read_config_option('path_stderrlog');
+	$errbase = basename($errfile);
+
+	$file = basename($file);
+	$filepath = '';
+	$filename = '';
+	$filefull = '';
+
+	if (!empty($errfile) && strpos($file, $errbase) === 0) {
+		$filepath = dirname($errfile);
+		$filename = $errbase;
+		$filefull = $filepath . '/' . $file;
+	} elseif (!empty($logfile) && strpos($file, $logbase) === 0) {
+		$filepath = dirname($logfile);
+		$filename = $logbase;
+		$filefull = $filepath . '/' . $file;
 	}
 
-	$purgefile = dirname($logfile) . '/' . get_nfilter_request_var('filename');
-	if (strstr($purgefile, $logfile) === false) {
+	return ($filecheck ? file_exists($filefull) : !empty($filefull));
+}
+
+function clog_purge_logfile() {
+	global $config;
+
+	$filename = get_nfilter_request_var('filename');
+
+	if (!clog_validate_filename($filename, $logpath, $logname)) {
 		raise_message('clog_invalid');
-		header('Location: ' . get_current_page() . '?header=false');
+		header('Location: ' . get_current_page());
 		exit(0);
 	}
 
+	$purgefile = $logpath . '/' . $filename;
+	$logfile = $logpath . '/'. $logname;
+
 	if (file_exists($purgefile)) {
 		if (is_writable($purgefile)) {
 			if ($logfile != $purgefile) {
 				unlink($purgefile);
 				raise_message('clog_remove');
 			} else {
-				$timestamp = date('Y-m-d H:i:s');
+				/* fill in the current date for printing in the log */
+				if (defined('CACTI_DATE_TIME_FORMAT')) {
+					$date = date(CACTI_DATE_TIME_FORMAT);
+				} else {
+					$date = date('Y-m-d H:i:s');
+				}
+
 				$log_fh = fopen($logfile, 'w');
-				fwrite($log_fh, "$timestamp - WEBUI: Cacti Log Cleared from Web Management Interface\n");
+				fwrite($log_fh, __('%s - WEBUI NOTE: Cacti Log Cleared from Web Management Interface.', $date) . PHP_EOL);
 				fclose($log_fh);
 				raise_message('clog_purged');
 			}
@@ -103,29 +130,6 @@ function clog_view_logfile() {
 	$exclude_reported = false;
 
 	$clogAdmin = clog_admin();
-	$logfile   = read_config_option('path_cactilog');
-	$logbase   = basename($logfile);
-
-	if (is_base64_encoded(get_nfilter_request_var('rfilter'))) {
-		set_request_var('rfilter', base64_decode(get_nfilter_request_var('rfilter')));
-	}
-
-	if (isset_request_var('filename')) {
-		$requestedFile = dirname($logfile) . '/' . basename(get_nfilter_request_var('filename'));
-		if (file_exists($requestedFile)) {
-			$logfile = $requestedFile;
-		}
-	} elseif ($logfile == '') {
-		$logfile = $config['base_path'] . '/log/cacti.log';
-	}
-
-	if (get_nfilter_request_var('filename') != '') {
-		if (strpos(get_nfilter_request_var('filename'), $logbase) === false) {
-			raise_message('clog_invalid');
-			header('Location: ' . get_current_page() . '?filename=' . $logbase);
-			exit(0);
-		}
-	}
 
 	/* ================= input validation and session storage ================= */
 	$filters = array(
@@ -143,6 +147,12 @@ function clog_view_logfile() {
 			'default' => '-1',
 			'pageset' => true
 		),
+		'filename' => array(
+			'filter'  => FILTER_CALLBACK,
+			'default' => read_config_option('path_cactilog'),
+			'pageset' => true,
+			'options' => array('options' => 'sanitize_search_string')
+		),
 		'refresh' => array(
 			'filter'  => FILTER_VALIDATE_INT,
 			'default' => read_config_option('log_refresh_interval')
@@ -167,9 +177,18 @@ function clog_view_logfile() {
 	set_request_var('page_referrer', 'view_logfile');
 	load_current_session_value('page_referrer', 'page_referrer', 'view_logfile');
 
+	$logfile = basename(get_nfilter_request_var('filename'));
+	$logname = '';
+
+	if (!clog_validate_filename($logfile, $logpath, $logname, true)) {
+		$logfile = read_config_option('path_cactilog');
+	} else {
+		$logfile = $logpath . '/' . $logfile;
+	}
+
 	if ($clogAdmin && isset_request_var('purge_continue')) {
 		clog_purge_logfile();
-		$logfile   = read_config_option('path_cactilog');
+		$logfile = read_config_option('path_cactilog');
 	}
 
 	$page_nr = get_request_var('page');
@@ -198,9 +217,9 @@ function clog_view_logfile() {
 			</td>
 		</tr>
 		<tr class='saveRow'>
-			<td colspan='2' align='right'>
-				<input id='cancel' type='button' value='" . __esc('Cancel') . "'>&nbsp
-				<input id='pc' type='button' name='purge_continue' value='" . __esc('Continue') . "' title='" . __esc('Purge Log') . "'>
+			<td colspan='2' class='right'>
+				<input type='button' class='ui-button ui-corner-all ui-widget' id='cancel' value='" . __esc('Cancel') . "'>&nbsp
+				<input type='button' class='ui-button ui-corner-all ui-widget' id='pc' name='purge_continue' value='" . __esc('Continue') . "' title='" . __esc('Purge Log') . "'>
 				<script type='text/javascript'>
 				$('#pc').click(function() {
 					strURL = location.pathname+'?purge_continue=1&header=false&filename=" . basename($logfile) . "';
@@ -225,7 +244,7 @@ function clog_view_logfile() {
 	}
 
 	html_start_box(__('Log Filters'), '100%', '', '3', 'center', '');
-	filter($clogAdmin);
+	filter($clogAdmin, basename($logfile));
 	html_end_box();
 
 	/* read logfile into an array and display */
@@ -241,19 +260,19 @@ function clog_view_logfile() {
 	if (!$clogAdmin) {
 		$exclude_regex = read_config_option('clog_exclude', true);
 		if ($exclude_regex != '') {
-			$ad_filter = __(' - Admin Filter in Affect');
+			$ad_filter = __(' - Admin Filter active');
 		} else {
-			$ad_filter = __(' - No Admin Filter in Affect');
+			$ad_filter = __(' - Admin Unfiltered');
 		}
 	} else {
-		$ad_filter = __(' - Admin View');
+		$ad_filter = __(' - Admin view');
 		$exclude_regex = '';
 	}
 
-	if (get_request_var('message_type') > 0) {
-		$start_string = __('Log [Total Lines: %d %s - Additional Filter in Affect]', $total_rows, $ad_filter);
+	if (get_request_var('message_type') > 0 || get_request_var('rfilter') != '') {
+		$start_string = __('Log [Total Lines: %d %s - Filter active]', $total_rows, $ad_filter);
 	} else {
-		$start_string = __('Log [Total Lines: %d %s - No Other Filter in Affect]', $total_rows, $ad_filter);
+		$start_string = __('Log [Total Lines: %d %s - Unfiltered]', $total_rows, $ad_filter);
 	}
 
 	$rfilter      = get_request_var('rfilter');
@@ -261,9 +280,9 @@ function clog_view_logfile() {
 	$refreshTime  = get_request_var('refresh');
 	$message_type = get_request_var('message_type');
 	$tail_lines   = get_request_var('tail_lines');
-	$base_url     = 'clog.php?rfilter='.$rfilter.'&reverse='.$reverse.'&refresh='.$refreshTime.'&message_type='.$message_type.'&tail_lines='.$tail_lines.'&filename='.basename($logfile);
+	$base_url     = $config['url_path'] . 'clog.php';
 
-	$nav = html_nav_bar($base_url, MAX_DISPLAY_PAGES, $page_nr, $number_of_lines, $total_rows, 13, __('Entries'), 'page', 'main');
+	$nav = html_nav_bar($base_url, MAX_DISPLAY_PAGES, $page_nr, $number_of_lines, $total_rows, 1, __('Entries'), 'page', 'main');
 
 	print $nav;
 
@@ -271,7 +290,11 @@ function clog_view_logfile() {
 
 	$linecolor = false;
 
-	$hosts = db_fetch_assoc_prepared('SELECT id, description FROM host');
+	$hosts = db_fetch_assoc('SELECT id, description
+		FROM host
+		WHERE disabled = ""
+		AND deleted = ""');
+
 	$hostDescriptions = array();
 	foreach ($hosts as $host) {
 		$hostDescriptions[$host['id']] = html_escape($host['description']);
@@ -333,7 +356,117 @@ function clog_view_logfile() {
 	bottom_footer();
 }
 
-function filter($clogAdmin) {
+function filter_sort($a, $b) {
+	$a_parts = explode('-', $a);
+	$b_parts = explode('-', $b);
+
+	$a_date = '99999999';
+	if (count($a_parts) > 1) {
+		$a_date = $a_parts[1];
+	}
+
+	$b_date = '99999999';
+	if (count($b_parts) > 1) {
+		$b_date = $b_parts[1];
+	}
+
+	// Invert the order, replace _'s with +'s to make them sort after .'s, prefix the date
+	// This makes cacti_stderr.log appear after cacti.log in date descending order with
+	// no date files first
+	return strcmp($b_date . '-' . str_replace('_','+',$b_parts[0]), $a_date . '-' . str_replace('_','+',$a_parts[0]));
+}
+
+function clog_get_logfiles() {
+	global $config;
+
+	$stdFileArray = $stdLogFileArray = $stdErrFileArray = array();
+	$configLogPath = read_config_option('path_cactilog');
+	$configLogBase = basename($configLogPath);
+	$stderrLogPath = read_config_option('path_stderrlog');
+	$stderrLogBase = basename($stderrLogPath);
+
+	if ($configLogPath == '') {
+		$logPath = $config['base_path'] . '/log/';
+	} else {
+		$logPath = dirname($configLogPath);
+	}
+
+	if (is_readable($logPath)) {
+		$files = @scandir($logPath);
+	} else {
+		$files = array('cacti.log');
+	}
+
+	// Defaults go first and second
+	$stdFileArray[] = basename($configLogPath);
+
+	// After Defaults, do Cacti log first (of archived)
+	if (cacti_sizeof($files)) {
+		$stdLogFileArray = array();
+		foreach ($files as $logFile) {
+			if (in_array($logFile, array('.', '..', '.htaccess', $configLogBase, $stderrLogBase))) {
+				continue;
+			}
+
+			$explode = explode('.', $logFile);
+			if (substr($explode[max(array_keys($explode))], 0, 3) != 'log') {
+				continue;
+			}
+
+			if (!clog_validate_filename($logFile, $logPath, $logName)) {
+				continue;
+			}
+
+			if (!empty($stderrlogbase) && strpos($logFile, $stderrLogBase) === 0){
+				$stdErrFileArray[] = $logFile;
+			} else {
+				$stdLogFileArray[] = $logFile;
+			}
+		}
+
+		$stdErrFileArray = array_unique($stdErrFileArray);
+		$stdLogFileArray = array_unique($stdLogFileArray);
+	}
+
+	// Defaults go first and second
+	if (!empty($stderrLogPath)) {
+		$stdFileArray[] = basename($stderrLogPath);
+
+		// After Defaults, do Cacti StdErr log second (of archived)
+		if (dirname($stderrLogPath) != $logPath) {
+			$errFiles = @scandir(dirname($stderrLogPath));
+			$files = $errFiles;
+			if (cacti_sizeof($files)) {
+				$stdErrFileArray = array();
+				foreach ($files as $logFile) {
+					if (in_array($logFile, array('.', '..', '.htaccess', $configLogBase, $stderrLogBase))) {
+						continue;
+					}
+
+					$explode = explode('.', $logFile);
+					if (substr($explode[max(array_keys($explode))], 0, 3) != 'log') {
+						continue;
+					}
+
+					if (!clog_validate_filename($logFile, $logPath, $logName)) {
+						continue;
+					}
+
+					$stdErrFileArray[] = $logFile;
+				}
+
+				$stdErrFileArray = array_unique($stdErrFileArray);
+			}
+		}
+	}
+
+	arsort($stdLogFileArray, SORT_NATURAL);
+	arsort($stdErrFileArray, SORT_NATURAL);
+
+	return array_unique(array_merge($stdFileArray, $stdLogFileArray, $stdErrFileArray));
+}
+
+function filter($clogAdmin, $selectedFile) {
 	global $page_refresh_interval, $log_tail_lines, $config;
 	?>
 	<tr class='even'>
@@ -347,47 +480,9 @@ function filter($clogAdmin) {
 					<td>
 						<select id='filename'>
 						<?php
-						$configLogPath = read_config_option('path_cactilog');
-						$configLogBase = basename($configLogPath);
-						$selectedFile  = basename(get_nfilter_request_var('filename'));
-
-						if ($configLogPath == '') {
-							$logPath = $config['base_path'] . '/log/';
-						} else {
-							$logPath = dirname($configLogPath);
-						}
-
-						if (is_readable($logPath)) {
-							$files = scandir($logPath);
-						} else {
-							$files = array('cacti.log');
-						}
-
-						if (sizeof($files)) {
-							$logFileArray = array();
-							foreach ($files as $logFile) {
-								if (in_array($logFile, array('.', '..', '.htaccess'))) {
-									continue;
-								}
-
-								$explode = explode('.', $logFile);
-								if (substr($explode[max(array_keys($explode))], 0, 3) != 'log') {
-									continue;
-								}
-
-								if (strpos($logFile, $configLogBase) === false) {
-									continue;
-								}
-
-								if (strcmp($logFile, 'cacti.log') == 0) {
-									continue;
-								}
-								$logFileArray[] = $logFile;
-							}
-
-							arsort($logFileArray);
-							array_unshift($logFileArray,'cacti.log');
+						$logFileArray = clog_get_logfiles();
 
+						if (cacti_sizeof($logFileArray)) {
 							foreach ($logFileArray as $logFile) {
 								print "<option value='" . $logFile . "'";
 
@@ -395,7 +490,12 @@ function filter($clogAdmin) {
 									print ' selected';
 								}
 
-								print '>' . $logFile . "</option>\n";
+								$logParts = explode('-', $logFile);
+
+								$logDate = count($logParts) < 2 ? '' : $logParts[1] . (isset($logParts[2]) ? '-' . $logParts[2]:'');
+								$logName = $logParts[0];
+
+								print '>' . $logName . ($logDate != '' ? ' [' . substr($logDate,4) . ']':'') . "</option>\n";
 							}
 						}
 						?>
@@ -419,9 +519,9 @@ function filter($clogAdmin) {
 					</td>
 					<td>
 						<span>
-							<input type='button' id='go' name='go' value='<?php print __esc('Go');?>'>
-							<input type='button' id='clear' name='clear' value='<?php print __esc('Clear');?>'>
-						<?php if ($clogAdmin) {?><input type='button' id='purge' name='purge' value='<?php print __esc('Purge');?>'><?php }?>
+							<input type='button' class='ui-button ui-corner-all ui-widget' id='go' name='go' value='<?php print __esc('Go');?>'>
+							<input type='button' class='ui-button ui-corner-all ui-widget' id='clear' name='clear' value='<?php print __esc('Clear');?>'>
+						<?php if ($clogAdmin) {?><input type='button' class='ui-button ui-corner-all ui-widget' id='purge' name='purge' value='<?php print __esc('Purge');?>'><?php }?>
 						</span>
 					</td>
 				</tr>
@@ -474,7 +574,7 @@ function filter($clogAdmin) {
 						<?php print __('Search');?>
 					</td>
 					<td>
-						<input id='rfilter' type='text' size='75' value='<?php print html_escape_request_var('rfilter');?>'>
+						<input type='text' class='ui-state-default ui-corner-all' id='rfilter' size='75' value='<?php print html_escape_request_var('rfilter');?>'>
 					</td>
 				</tr>
 			</table>
@@ -524,7 +624,7 @@ function filter($clogAdmin) {
 		});
 
 		function clearFilter() {
-			strURL = basename(location.pathname) + '?clear=1&header=false';
+			strURL = basename(location.pathname) + '?clear=1&header=false&nostate=true';
 			loadPageNoHeader(strURL);
 		}
 
@@ -551,13 +651,19 @@ function filter($clogAdmin) {
 function clog_get_regex_array() {
 	static $regex_array = array();
 
-	if (!sizeof($regex_array)) {
+	if (!cacti_sizeof($regex_array)) {
 		$regex_array = array(
-			1 => array('name' => 'DS',     'regex' => '( DS\[)([, \d]+)(\])',      'func' => 'clog_regex_datasource'),
-			2 => array('name' => 'DQ',     'regex' => '( DQ\[)([, \d]+)(\])',      'func' => 'clog_regex_dataquery'),
-			3 => array('name' => 'Device', 'regex' => '( Device\[)([, \d]+)(\])',  'func' => 'clog_regex_device'),
-			4 => array('name' => 'Poller', 'regex' => '( Poller\[)([, \d]+)(\])',  'func' => 'clog_regex_poller'),
-			5 => array('name' => 'RRA',    'regex' => "([_\/])(\d+)(\.rrd&#039;)", 'func' => 'clog_regex_rra')
+			1  => array('name' => 'DS',     'regex' => '( DS\[)([, \d]+)(\])',       'func' => 'clog_regex_datasource'),
+			2  => array('name' => 'DQ',     'regex' => '( DQ\[)([, \d]+)(\])',       'func' => 'clog_regex_dataquery'),
+			3  => array('name' => 'Device', 'regex' => '( Device\[)([, \d]+)(\])',   'func' => 'clog_regex_device'),
+			4  => array('name' => 'Poller', 'regex' => '( Poller\[)([, \d]+)(\])',   'func' => 'clog_regex_poller'),
+			5  => array('name' => 'RRA',    'regex' => "([_\/])(\d+)(\.rrd&#039;)",  'func' => 'clog_regex_rra'),
+			6  => array('name' => 'GT',     'regex' => '( GT\[)([, \d]+)(\])',       'func' => 'clog_regex_graphtemplates'),
+			7  => array('name' => 'Graph',  'regex' => '( Graph\[)([, \d]+)(\])',    'func' => 'clog_regex_graphs'),
+			8  => array('name' => 'Graphs', 'regex' => '( Graphs\[)([, \d]+)(\])',   'func' => 'clog_regex_graphs'),
+			9  => array('name' => 'User',   'regex' => '( User\[)([, \d]+)(\])',     'func' => 'clog_regex_users'),
+			10 => array('name' => 'User',   'regex' => '( Users\[)([, \d]+)(\])',    'func' => 'clog_regex_users'),
+			11 => array('name' => 'Rule',   'regex' => '( Rule\[)([, \d]+)(\])',   	 'func' => 'clog_regex_rule'),
 		);
 
 		$regex_array = api_plugin_hook_function('clog_regex_array',$regex_array);
@@ -577,16 +683,18 @@ function clog_regex_parser($matches) {
 	$match = $matches[0];
 
 	$key_match = -1;
-	for ($index = 1; $index < sizeof($matches); $index++) {
+	for ($index = 1; $index < cacti_sizeof($matches); $index++) {
 		if ($match == $matches[$index]) {
 			$key_match = $index;
+			break;
 		}
 	}
 
 	if ($key_match != -1) {
 		$key_setting = ($key_match - 1) / 4 + 1;
 		$regex_array = clog_get_regex_array();
-		if (sizeof($regex_array)) {
+
+		if (cacti_sizeof($regex_array)) {
 			if (array_key_exists($key_setting, $regex_array)) {
 				$regex_setting = $regex_array[$key_setting];
 
@@ -595,7 +703,11 @@ function clog_regex_parser($matches) {
 					$rekey_array[$j] = $matches[$key_match + $j];
 				}
 
-				$result=call_user_func_array($regex_setting['func'],array($rekey_array));
+				if (function_exists($regex_setting['func'])) {
+					$result=call_user_func_array($regex_setting['func'],array($rekey_array));
+				} else {
+					$result=$match;
+				}
 			}
 		}
 	}
@@ -609,15 +721,17 @@ function clog_regex_device($matches) {
 	$result = $matches[0];
 
 	$dev_ids = explode(',',str_replace(" ","",$matches[2]));
-	if (sizeof($dev_ids)) {
-		$hosts = db_fetch_assoc_prepared('SELECT id, description
+	if (cacti_sizeof($dev_ids)) {
+		$result = '';
+		$hosts = db_fetch_assoc('SELECT id, description
 			FROM host
-			WHERE id in (?)',
-			array(implode(',',$dev_ids)));
+			WHERE id in (' . implode(',',$dev_ids) . ')');
 
 		$hostDescriptions = array();
-		foreach ($hosts as $host) {
-			$hostDescriptions[$host['id']] = html_escape($host['description']);
+		if (cacti_sizeof($hosts)) {
+			foreach ($hosts as $host) {
+				$hostDescriptions[$host['id']] = html_escape($host['description']);
+			}
 		}
 
 		foreach ($dev_ids as $host_id) {
@@ -634,45 +748,47 @@ function clog_regex_datasource($matches) {
 	$result = $matches[0];
 
 	$ds_ids = explode(',',str_replace(" ","",$matches[2]));
-	if (sizeof($ds_ids)) {
+	if (cacti_sizeof($ds_ids)) {
 		$result = '';
-		$graph_add = $config['url_path'] . 'graph_view.php?page=1&style=selective&action=preview&graph_add=';
 
-		$title = '';
-		$i     = 0;
+		$graph_rows = array_rekey(db_fetch_assoc('SELECT DISTINCT
+			gtg.local_graph_id AS id
+			FROM graph_templates_graph AS gtg
+			INNER JOIN graph_templates_item AS gti
+			ON gtg.local_graph_id=gti.local_graph_id
+			INNER JOIN data_template_rrd AS dtr
+			ON gti.task_item_id=dtr.id
+			WHERE gtg.local_graph_id>0
+			AND dtr.local_data_id IN (' . $matches[2] . ')'),'id','id');
 
-		foreach($ds_ids as $ds_id) {
-			$graph_ids = clog_get_graphs_from_datasource($ds_id);
+		$graph_results = '';
+		if (cacti_sizeof($graph_rows)) {
+			$graph_ids = implode(',',$graph_rows);
+			$graph_array = array( 0 => '', 1 => ' Graphs[', 2 => $graph_ids, 3 => ']');
 
-			if (sizeof($graph_ids)) {
-				$result .= " Graphs[<a href='";
-
-				foreach($graph_ids as $key => $title) {
-					$graph_add .= ($i > 0 ? '%2C' : '') . $key;
-					$title     .= ($title != '' ? ', ':'') . html_escape($title);
-
-					$i++;
-				}
-
-				$result .= html_escape($graph_add) . "' title='" . __esc('View Graphs') . "'>" . $title . '</a>]';
-			}
+			$graph_results = clog_regex_graphs($graph_array);
 		}
 
 		$result .= $matches[1];
 		$i       = 0;
 
+		$ds_ids = array_unique($ds_ids);
 		$ds_titles = clog_get_datasource_titles($ds_ids);
+		if (!isset($ds_titles)) {
+			$ds_titles = array();
+		}
+
 		foreach($ds_ids as $ds_id) {
 			$ds_title = $ds_id;
 			if (array_key_exists($ds_id, $ds_titles)) {
 				$ds_title = $ds_titles[$ds_id];
 			}
-			$result .= ($i == 0 ? '':', ') . "<a href='" . html_escape($config['url_path'] . 'data_sources.php?action=ds_edit&id=' . $ds_id) . "'>" . $ds_title . '</a>';
+			$result .= ($i == 0 ? '':', ') . "<a href='" . html_escape($config['url_path'] . 'data_sources.php?action=ds_edit&id=' . $ds_id) . "'>" . html_escape($ds_title) . '</a>';
 
 			$i++;
 		}
 
-		$result .= $matches[3];
+		$result .= $matches[3] . $graph_results;
 	}
 
 	return $result;
@@ -684,16 +800,17 @@ function clog_regex_poller($matches) {
 	$result = $matches[0];
 
 	$poller_ids = explode(',',str_replace(" ","",$matches[2]));
-	if (sizeof($poller_ids)) {
+	if (cacti_sizeof($poller_ids)) {
 		$result = '';
 		$pollers = db_fetch_assoc_prepared('SELECT id, name
 			FROM poller
-			WHERE id in (?)',
-			array(implode(',',$poller_ids)));
+			WHERE id in (' . implode(',',$poller_ids) . ')');
 
 		$pollerDescriptions = array();
-		foreach ($pollers as $poller) {
-			$pollerDescriptions[$poller['id']] = html_escape($poller['name']);
+		if (cacti_sizeof($pollers)) {
+			foreach ($pollers as $poller) {
+				$pollerDescriptions[$poller['id']] = html_escape($poller['name']);
+			}
 		}
 
 		foreach ($poller_ids as $poller_id) {
@@ -710,20 +827,21 @@ function clog_regex_dataquery($matches) {
 	$result = $matches[0];
 
 	$query_ids = explode(',',str_replace(" ","",$matches[2]));
-	if (sizeof($query_ids)) {
+	if (cacti_sizeof($query_ids)) {
 		$result = '';
-		$querys = db_fetch_assoc_prepared('SELECT id, name
+		$querys = db_fetch_assoc('SELECT id, name
 			FROM snmp_query
-			WHERE id in (?)',
-			array(implode(',',$query_ids)));
+			WHERE id in (' . implode(',',$query_ids) . ')');
 
 		$queryDescriptions = array();
-		foreach ($querys as $query) {
-			$queryDescriptions[$query['id']] = html_escape($query['name']);
+		if (cacti_sizeof($querys)) {
+			foreach ($querys as $query) {
+				$queryDescriptions[$query['id']] = html_escape($query['name']);
+			}
 		}
 
 		foreach ($query_ids as $query_id) {
-			$result .= $matches[1].'<a href=\'' . html_escape($config['url_path'] . 'data_query.php?action=edit&id=' . $query_id) . '\'>' . (isset($queryDescriptions[$query_id]) ? $queryDescriptions[$query_id]:$query_id) . '</a>' . $matches[3];
+			$result .= $matches[1].'<a href=\'' . html_escape($config['url_path'] . 'data_queries.php?action=edit&id=' . $query_id) . '\'>' . (isset($queryDescriptions[$query_id]) ? $queryDescriptions[$query_id]:$query_id) . '</a>' . $matches[3];
 		}
 	}
 
@@ -737,7 +855,7 @@ function clog_regex_rra($matches) {
 
 	$local_data_ids = $matches[2];
 	if (strlen($local_data_ids)) {
-		$datasource_array = array( 0 => '', 1 => 'DS[', 2 => $local_data_ids, 3 => ']');
+		$datasource_array = array( 0 => '', 1 => ' DS[', 2 => $local_data_ids, 3 => ']');
 		$datasource_result = clog_regex_datasource($datasource_array);
 		if (strlen($datasource_result)) {
 			$result .= ' '. $datasource_result;
@@ -747,3 +865,134 @@ function clog_regex_rra($matches) {
 	return $result;
 }
 
+function clog_regex_graphs($matches) {
+	global $config;
+
+	$result = $matches[0];
+
+	$query_ids = explode(',',str_replace(" ","",$matches[2]));
+	if (cacti_sizeof($query_ids)) {
+		$result = '';
+		$graph_add = $config['url_path'] . 'graph_view.php?page=1&style=selective&action=preview&graph_add=';
+
+		$title = '';
+		$i     = 0;
+
+		$querys = db_fetch_assoc('SELECT DISTINCT
+			gtg.local_graph_id AS id,
+			gtg.title_cache AS title
+			FROM graph_templates_graph AS gtg
+			INNER JOIN graph_templates_item AS gti
+			ON gtg.local_graph_id=gti.local_graph_id
+			INNER JOIN data_template_rrd AS dtr
+			ON gti.task_item_id=dtr.id
+			WHERE gtg.local_graph_id in (' . implode(',',$query_ids) . ')');
+
+		$result .= $matches[1] . "<a href='";
+
+		$queryDescriptions = array();
+		if (cacti_sizeof($querys)) {
+			foreach ($querys as $query) {
+				$queryDescriptions[$query['id']] = html_escape($query['title']);
+			}
+		}
+
+		$i=0;
+		foreach ($query_ids as $query_id) {
+			$graph_add .= ($i > 0 ? '%2C' : '') . $query_id;
+			$title     .= ($title != '' ? ', ':'') . html_escape((isset($queryDescriptions[$query_id]) ? $queryDescriptions[$query_id]:$query_id));
+			$i++;
+		}
+
+		$result .= html_escape($graph_add) . '\'>' . $title . '</a>' . $matches[3];
+	}
+	return $result;
+}
+
+function clog_regex_graphtemplates($matches) {
+	global $config;
+
+	$result = $matches[0];
+
+	$query_ids = explode(',',str_replace(" ","",$matches[2]));
+	if (cacti_sizeof($query_ids)) {
+		$result = '';
+		$querys = db_fetch_assoc('SELECT id, name
+			FROM graph_templates
+			WHERE id in ('  . implode(',',$query_ids) . ')');
+
+		$queryDescriptions = array();
+		if (cacti_sizeof($querys)) {
+			foreach ($querys as $query) {
+				$queryDescriptions[$query['id']] = html_escape($query['name']);
+			}
+		}
+
+		foreach ($query_ids as $query_id) {
+			$result .= $matches[1].'<a href=\'' . html_escape($config['url_path'] . 'graph_templates.php?action=template_edit&id=' . $query_id) . '\'>' . (isset($queryDescriptions[$query_id]) ? $queryDescriptions[$query_id]:$query_id) . '</a>' . $matches[3];
+		}
+	}
+
+	return $result;
+}
+
+function clog_regex_users($matches) {
+	global $config;
+
+	$result = $matches[0];
+
+	$query_ids = explode(',',str_replace(" ","",$matches[2]));
+	if (cacti_sizeof($query_ids)) {
+		$result = '';
+
+		$querys = db_fetch_assoc('SELECT DISTINCT
+			id, username
+			FROM user_auth
+			WHERE id in (' . implode(',',$query_ids) . ')');
+
+		$queryDescriptions = array();
+		if (cacti_sizeof($querys)) {
+			foreach ($querys as $query) {
+				$queryDescriptions[$query['id']] = html_escape($query['username']);
+			}
+		}
+
+		foreach ($query_ids as $query_id) {
+			$result .= $matches[1];
+			if (isset($queryDescriptions[$query_id])) {
+				$result .= '<a href=\'' . html_escape($config['url_path'] . 'user_admin.php?action=user_edit&tab=general&id=' . $query_id) . '\'>' . $queryDescriptions[$query_id] . '</a>';
+			} else {
+				$result .= $query_id;
+			}
+			$result .= $matches[3];
+		}
+	}
+	return $result;
+}
+
+function clog_regex_rule($matches) {
+	global $config;
+
+	$result = $matches[0];
+
+	$dev_ids = explode(',',str_replace(" ","",$matches[2]));
+	if (cacti_sizeof($dev_ids)) {
+		$result = '';
+		$rules = db_fetch_assoc('SELECT id, name
+			FROM automation_graph_rules
+			WHERE id in (' . implode(',',$dev_ids) . ')');
+
+		$ruleNames = array();
+		if (cacti_sizeof($rules)) {
+			foreach ($rules as $rule) {
+				$ruleNames[$rule['id']] = html_escape($rule['name']);
+			}
+		}
+
+		foreach ($dev_ids as $rule_id) {
+			$result .= $matches[1].'<a href=\'' . html_escape($config['url_path'] . 'automation_graph_rules.php?action=edit&id=' . $rule_id) . '\'>' . (isset($ruleNames[$rule_id]) ? $ruleNames[$rule_id]:$rule_id) . '</a>' . $matches[3];
+		}
+	}
+
+	return $result;
+}
