@@ -1,7 +1,7 @@
 <?php
 /*
  +-------------------------------------------------------------------------+
- | Copyright (C) 2004-2018 The Cacti Group                                 |
+ | Copyright (C) 2004-2020 The Cacti Group                                 |
  |                                                                         |
  | This program is free software; you can redistribute it and/or           |
  | modify it under the terms of the GNU General Public License             |
@@ -23,7 +23,7 @@
 */
 
 function import_xml_data(&$xml_data, $import_as_new, $profile_id, $remove_orphans = false) {
-	global $config, $hash_type_codes, $cacti_version_codes, $preview_only, $remove_orphans, $import_debug_info;
+	global $config, $hash_type_codes, $cacti_version_codes, $preview_only, $remove_orphans, $import_debug_info, $legacy_template;
 
 	include_once($config['library_path'] . '/xml.php');
 
@@ -32,7 +32,7 @@ function import_xml_data(&$xml_data, $import_as_new, $profile_id, $remove_orphan
 
 	$xml_array = xml2array($xml_data);
 
-	if (sizeof($xml_array) == 0) {
+	if (cacti_sizeof($xml_array) == 0) {
 		raise_message(7); /* xml parse error */
 		return $info_array;
 	}
@@ -42,7 +42,9 @@ function import_xml_data(&$xml_data, $import_as_new, $profile_id, $remove_orphan
 		$parsed_hash = parse_xml_hash($hash);
 
 		/* invalid/wrong hash */
-		if ($parsed_hash == false) { return $info_array; }
+		if ($parsed_hash == false) {
+			return $info_array;
+		}
 
 		if (isset($dep_hash_cache[$parsed_hash['type']])) {
 			array_push($dep_hash_cache[$parsed_hash['type']], $parsed_hash);
@@ -53,8 +55,101 @@ function import_xml_data(&$xml_data, $import_as_new, $profile_id, $remove_orphan
 
 	//print '<pre>';print_r($dep_hash_cache);print '</pre>';exit;
 
+	// Populate the hash cache with preexisting objects.
 	$hash_cache = array();
-	$repair     = 0;
+	$hash_types_to_db_info = array(
+		'graph_template' => array(
+			'table' => 'graph_templates',
+		),
+		'graph_template_item' => array(
+			'table' => 'graph_templates_item',
+		),
+		'graph_template_input' => array(
+			'table' => 'graph_template_input',
+		),
+		'data_template' => array(
+			'table' => 'data_template',
+		),
+		'data_template_item' => array(
+			'table' => 'data_template_rrd',
+		),
+		'host_template' => array(
+			'table' => 'host_template',
+		),
+		'data_input_method' => array(
+			'table' => 'data_input',
+		),
+		'data_input_field' => array(
+			'table' => 'data_input_fields',
+		),
+		'data_query' => array(
+			'table' => 'snmp_query',
+		),
+		'data_query_graph' => array(
+			'table' => 'snmp_query_graph',
+		),
+		'data_query_sv_graph' => array(
+			'table' => 'snmp_query_graph_sv',
+		),
+		'data_query_sv_data_source' => array(
+			'table' => 'snmp_query_graph_rrd_sv',
+		),
+		'gprint_preset' => array(
+			'table' => 'graph_templates_gprint',
+		),
+		'cdef' => array(
+			'table' => 'cdef',
+		),
+		'cdef_item' => array(
+			'table' => 'cdef_items',
+		),
+		'vdef' => array(
+			'table' => 'vdef',
+		),
+		'vdef_item' => array(
+			'table' => 'vdef_items',
+		),
+		'data_source_profiles' => array(
+			'table' => 'data_source_profiles',
+		),
+		'data_source_profile_rra' => array(
+			'table' => 'data_source_profiles_rra',
+			'hash_field' => 'name',
+		),
+	);
+	$hash_cache_sql_union_selects = array();
+
+	foreach ($hash_types_to_db_info as $hash_type => $db_info) {
+		$db_id_field = (
+			isset($db_info['id_field'])
+			? $db_info['id_field']
+			: 'id'
+		);
+		$db_hash_field = (
+			isset($db_info['hash_field'])
+			? $db_info['hash_field']
+			: 'hash'
+		);
+		$db_table = $db_info['table'];
+		$hash_cache_sql_union_selects[] = "
+			SELECT
+				'$hash_type' AS type,
+				`$db_id_field` AS id,
+				`$db_hash_field` AS hash
+			FROM
+				`$db_table`
+		";
+	}
+
+	$hash_cache_sql = implode(' UNION ALL ', $hash_cache_sql_union_selects);
+	$hash_cache_results = db_fetch_assoc($hash_cache_sql);
+	if (cacti_sizeof($hash_cache_results)) {
+		foreach ($hash_cache_results as $hash_cache_row) {
+			$hash_cache[$hash_cache_row['type']][$hash_cache_row['hash']] = $hash_cache_row['id'];
+		}
+	}
+
+	$repair = 0;
 
 	/* the order of the $hash_type_codes array is ordered such that the items
 	with the most dependencies are last and the items with no dependencies are first.
@@ -63,15 +158,26 @@ function import_xml_data(&$xml_data, $import_as_new, $profile_id, $remove_orphan
 		/* do we have any matches for this type? */
 		if (isset($dep_hash_cache[$type])) {
 			/* yes we do. loop through each match for this type */
-			for ($i=0; $i<count($dep_hash_cache[$type]); $i++) {
+			for ($i=0; $i<cacti_count($dep_hash_cache[$type]); $i++) {
 				$import_debug_info = false;
 
+				if (!isset($cacti_version_codes[$dep_hash_cache[$type][$i]['version']])) {
+					return false;
+				}
+
 				cacti_log('$dep_hash_cache[$type][$i][\'type\']: ' . $dep_hash_cache[$type][$i]['type'], false, 'IMPORT', POLLER_VERBOSITY_HIGH);
 				cacti_log('$dep_hash_cache[$type][$i][\'version\']: ' . $dep_hash_cache[$type][$i]['version'], false, 'IMPORT', POLLER_VERBOSITY_HIGH);
 				cacti_log('$cacti_version_codes[$dep_hash_cache[$type][$i][\'version\']]: ' . $cacti_version_codes[$dep_hash_cache[$type][$i]['version']], false, 'IMPORT', POLLER_VERBOSITY_HIGH);
 				cacti_log('$dep_hash_cache[$type][$i][\'hash\']: ' . $dep_hash_cache[$type][$i]['hash'], false, 'IMPORT', POLLER_VERBOSITY_HIGH);
 
-				$hash_array = $xml_array['hash_' . $hash_type_codes[$dep_hash_cache[$type][$i]['type']] . $cacti_version_codes[$dep_hash_cache[$type][$i]['version']] . $dep_hash_cache[$type][$i]['hash']];
+				if (isset($xml_array['hash_' . $hash_type_codes[$dep_hash_cache[$type][$i]['type']] . $cacti_version_codes[$dep_hash_cache[$type][$i]['version']] . $dep_hash_cache[$type][$i]['hash']])) {
+					$hash_array = $xml_array['hash_' . $hash_type_codes[$dep_hash_cache[$type][$i]['type']] . $cacti_version_codes[$dep_hash_cache[$type][$i]['version']] . $dep_hash_cache[$type][$i]['hash']];
+				} elseif (isset($xml_array['hash_' . $hash_type_codes[$dep_hash_cache[$type][$i]['type']] . $dep_hash_cache[$type][$i]['hash']])) {
+					$hash_array = $xml_array['hash_' . $hash_type_codes[$dep_hash_cache[$type][$i]['type']] . $dep_hash_cache[$type][$i]['hash']];
+				} else {
+					raise_message(7); /* xml parse error */
+					return false;
+				}
 
 				switch($type) {
 				case 'graph_template':
@@ -109,10 +215,20 @@ function import_xml_data(&$xml_data, $import_as_new, $profile_id, $remove_orphan
 				case 'round_robin_archive':
 					// Deprecated
 					break;
+				default:
+					$param = array();
+					$param['hash']       = $dep_hash_cache[$type][$i]['hash'];
+					$param['xml_array']  = $hash_array;
+					$param['hash_cache'] = $hash_cache;
+					$param['type']       = $type;
+					$param['version']    = $dep_hash_cache[$type][$i]['version'];
+					$param = api_plugin_hook_function('import_action', $param);
+					$hash_cache += $param['hash_cache'];
+					break;
 				}
 
 				if (!empty($import_debug_info)) {
-					$info_array[$type]{isset($info_array[$type]) ? count($info_array[$type]) : 0} = $import_debug_info;
+					$info_array[$type][isset($info_array[$type]) ? cacti_count($info_array[$type]) : 0] = $import_debug_info;
 				}
 			}
 		}
@@ -136,14 +252,18 @@ EOD;
     return $public_key;
 }
 
-function import_package($xmlfile, $profile_id = 1, $remove_orphans = false, $preview = false) {
+function import_package($xmlfile, $profile_id = 1, $remove_orphans = false, $preview = false, $info_only = false, $limitex = true) {
 	global $config, $preview_only;
 
 	$preview_only = $preview;
 
+	@ini_set('zlib.output_compression', '0');
+
 	/* set new timeout and memory settings */
-	ini_set('max_execution_time', '50');
-	ini_set('memory_limit', '128M');
+	if ($limitex) {
+		ini_set('max_execution_time', '50');
+		ini_set('memory_limit', '-1');
+	}
 
 	$public_key = get_public_key();
 
@@ -161,7 +281,7 @@ function import_package($xmlfile, $profile_id = 1, $remove_orphans = false, $pre
 				$binary_signature =  base64_decode(trim(str_replace(array('<signature>', '</signature>'), array('', ''), $x)));
 				$x = "   <signature></signature>\n";
 
-				cacti_log('NOTE: Got Package Signature', false, 'IMPORT', POLLER_VERBOSITY_HIGH);
+				cacti_log('NOTE: Got Package Signature', false, 'IMPORT', POLLER_VERBOSITY_LOW);
 			}
 			$xml .= "$x";
 		}
@@ -183,7 +303,7 @@ function import_package($xmlfile, $profile_id = 1, $remove_orphans = false, $pre
 		return false;
 	}
 
-	cacti_log('Loading Plugin Information from package', false, 'IMPORT', POLLER_VERBOSITY_HIGH);
+	cacti_log('Loading Plugin Information from package', false, 'IMPORT', POLLER_VERBOSITY_LOW);
 
 	$xmlget     = simplexml_load_string($xml);
 	$data       = xml_to_array($xmlget);
@@ -191,7 +311,12 @@ function import_package($xmlfile, $profile_id = 1, $remove_orphans = false, $pre
 
 	$plugin = $data['info']['name'];
 
-	cacti_log('Verifying each files signature', false, 'IMPORT', POLLER_VERBOSITY_HIGH);
+	if ($info_only) {
+		return $data['info'];
+		exit;
+	}
+
+	cacti_log('Verifying each files signature', false, 'IMPORT', POLLER_VERBOSITY_LOW);
 
 	if (isset($data['files']['file']['data'])) {
 		$data['files']['file'] = array($data['files']['file']);
@@ -202,14 +327,14 @@ function import_package($xmlfile, $profile_id = 1, $remove_orphans = false, $pre
 		$fdata = base64_decode($f['data']);
 		$ok = openssl_verify($fdata, $binary_signature, $public_key, OPENSSL_ALGO_SHA1);
 		if ($ok == 1) {
-			cacti_log('NOTE: File OK: ' . $f['name'], false, 'IMPORT', POLLER_VERBOSITY_HIGH);
+			cacti_log('NOTE: File OK: ' . $f['name'], false, 'IMPORT', POLLER_VERBOSITY_LOW);
 		} else {
 			cacti_log('FATAL: Could not Verify Signature for file: ' . $f['name'], true, 'IMPORT', POLLER_VERBOSITY_LOW);
 			return false;
 		}
 	}
 
-	cacti_log('Writing Files', false, 'IMPORT', POLLER_VERBOSITY_HIGH);
+	cacti_log('Writing Files', false, 'IMPORT', POLLER_VERBOSITY_LOW);
 
 	foreach ($data['files']['file'] as $f) {
 		$fdata = base64_decode($f['data']);
@@ -217,13 +342,14 @@ function import_package($xmlfile, $profile_id = 1, $remove_orphans = false, $pre
 
 		if (strpos($name, 'scripts/') !== false || strpos($name, 'resource/') !== false) {
 			$filename = $config['base_path'] . "/$name";
-			cacti_log('Writing file: ' . $filename, false, 'IMPORT', POLLER_VERBOSITY_HIGH);
+			cacti_log('Writing file: ' . $filename, false, 'IMPORT', POLLER_VERBOSITY_LOW);
+
 			if (!$preview) {
-				if (is_writable($filename)) {
-					$file = fopen($filename,'wb');
+				if ((is_writeable(dirname($filename)) && !file_exists($filename)) || is_writable($filename)) {
+					$file = fopen($filename, 'wb');
 
 					if (is_resource($file)) {
-						fwrite($file ,$fdata, strlen($fdata));
+						fwrite($file , $fdata, strlen($fdata));
 						fclose($file);
 						clearstatcache();
 						$filestatus[$filename] = __('written');
@@ -239,6 +365,8 @@ function import_package($xmlfile, $profile_id = 1, $remove_orphans = false, $pre
 				} else {
 					$filestatus[$filename] = __('not writable');
 				}
+
+				cacti_log('Write Status file: ' . $filename . ', with Status ' . $filestatus[$filename], false, 'IMPORT', POLLER_VERBOSITY_LOW);
 			} else {
 				if (!is_writable($filename)) {
 					$filestatus[$filename] = __('not writable');
@@ -246,14 +374,19 @@ function import_package($xmlfile, $profile_id = 1, $remove_orphans = false, $pre
 					$filestatus[$filename] = __('writable');
 				}
 			}
+
 		} else {
 			cacti_log('Importing XML Data for ' . $name, false, 'IMPORT', POLLER_VERBOSITY_LOW);
 
 			$debug_data = import_xml_data($fdata, false, $profile_id, $remove_orphans, $preview_only);
+
+			if ($debug_data === false) {
+				return false;
+			}
 		}
 	}
 
-	cacti_log('File creation complete', false, 'IMPORT', POLLER_VERBOSITY_HIGH);
+	cacti_log('File creation complete', false, 'IMPORT', POLLER_VERBOSITY_LOW);
 
 	return array($debug_data, $filestatus);
 }
@@ -273,9 +406,11 @@ function xml_to_graph_template($hash, &$xml_array, &$hash_cache, $hash_version,
 	if (!empty($_graph_template_id)) {
 		$previous_data = db_fetch_row_prepared('SELECT *
 			FROM graph_templates
-			WHERE id = ?', array($_graph_template_id));
+			WHERE id = ?',
+			array($_graph_template_id));
 	} else {
 		$previous_data = array();
+		clear_cached_allowed_types();
 	}
 
 	$save['id']   = (empty($_graph_template_id) ? '0' : $_graph_template_id);
@@ -302,8 +437,10 @@ function xml_to_graph_template($hash, &$xml_array, &$hash_cache, $hash_version,
 	$save['id'] = (empty($_graph_template_id) ? '0' : db_fetch_cell_prepared('SELECT gtg.id
 		FROM graph_templates AS gt
 		INNER JOIN graph_templates_graph AS gtg
-		ON gt.id=gtg.graph_template_id
-		WHERE gt.id = ? AND gtg.local_graph_id=0', array($graph_template_id)));
+		ON gt.id = gtg.graph_template_id
+		WHERE gt.id = ?
+		AND gtg.local_graph_id = 0',
+		array($graph_template_id)));
 
 	if (!empty($_graph_template_id)) {
 		$previous_data = db_fetch_row_prepared('SELECT *
@@ -354,7 +491,7 @@ function xml_to_graph_template($hash, &$xml_array, &$hash_cache, $hash_version,
 				AND local_graph_id = 0',
 				array($graph_template_id));
 
-			if (sizeof($items)) {
+			if (cacti_sizeof($items)) {
 				foreach($items as $item) {
 					$orphaned_items[$item['hash']] = $item;
 				}
@@ -380,7 +517,7 @@ function xml_to_graph_template($hash, &$xml_array, &$hash_cache, $hash_version,
 				FROM graph_templates_item
 				WHERE hash = ?
 				AND graph_template_id = ?
-				AND local_graph_id=0',
+				AND local_graph_id = 0',
 				array($parsed_hash['hash'], $graph_template_id));
 
 			if (!empty($_graph_template_item_id)) {
@@ -424,7 +561,7 @@ function xml_to_graph_template($hash, &$xml_array, &$hash_cache, $hash_version,
 			}
 
 			if (!empty($_graph_template_id)) {
-				if (!sizeof($previous_data)) {
+				if (!cacti_sizeof($previous_data)) {
 					$new_items[$parsed_hash['hash']] = $save;
 				}
 			}
@@ -490,7 +627,7 @@ function xml_to_graph_template($hash, &$xml_array, &$hash_cache, $hash_version,
 				$hash_items = explode('|', $item_array['items']);
 
 				if (!empty($hash_items[0])) {
-					for ($i=0; $i<count($hash_items); $i++) {
+					for ($i=0; $i<cacti_count($hash_items); $i++) {
 						/* parse information from the hash */
 						$parsed_hash = parse_xml_hash($hash_items[$i]);
 
@@ -499,7 +636,7 @@ function xml_to_graph_template($hash, &$xml_array, &$hash_cache, $hash_version,
 
 						if (isset($hash_cache['graph_template_item'][$parsed_hash['hash']])) {
 							db_execute_prepared('REPLACE INTO graph_template_input_defs
-								(graph_template_input_id,graph_template_item_id)
+								(graph_template_input_id, graph_template_item_id)
 								VALUES (?, ?)',
 								array($graph_template_input_id, $hash_cache['graph_template_item'][$parsed_hash['hash']]));
 						}
@@ -514,7 +651,7 @@ function xml_to_graph_template($hash, &$xml_array, &$hash_cache, $hash_version,
 	$import_debug_info['title']  = $xml_array['name'];
 	$import_debug_info['result'] = ($preview_only ? 'preview':(empty($graph_template_id) ? 'fail' : 'success'));
 
-	if (isset($new_items) && sizeof($new_items)) {
+	if (isset($new_items) && cacti_sizeof($new_items)) {
 		$new_text = array();
 		foreach($new_items as $item) {
 			$new_text[] = 'New Graph Items, Type: ' . $graph_item_types[$item['graph_type_id']] . ', Text Format: ' . $item['text_format'] . ', Value: ' . $item['value'];
@@ -522,7 +659,7 @@ function xml_to_graph_template($hash, &$xml_array, &$hash_cache, $hash_version,
 		$import_debug_info['new_items'] = $new_text;
 	}
 
-	if (isset($orphaned_items) && sizeof($orphaned_items)) {
+	if (isset($orphaned_items) && cacti_sizeof($orphaned_items)) {
 		$orphan_text = array();
 		foreach($orphaned_items as $item) {
 			if ($remove_orphans) {
@@ -550,7 +687,7 @@ function xml_to_graph_template($hash, &$xml_array, &$hash_cache, $hash_version,
 }
 
 function xml_to_data_template($hash, &$xml_array, &$hash_cache, $import_as_new, $profile_id) {
-	global $struct_data_source, $struct_data_source_item, $import_template_id, $preview_only, $import_debug_info;
+	global $struct_data_source, $struct_data_source_item, $import_template_id, $preview_only, $import_debug_info, $legacy_template;
 
 	/* track changes */
 	$status = 0;
@@ -594,7 +731,8 @@ function xml_to_data_template($hash, &$xml_array, &$hash_cache, $import_as_new,
 		INNER JOIN data_template_data AS dtd
 		ON dt.id=dtd.data_template_id
 		WHERE dt.id = ?
-		AND dtd.local_data_id=0', array($data_template_id)));
+		AND dtd.local_data_id = 0',
+		array($data_template_id)));
 
 	if (!empty($save['id'])) {
 		$previous_data = db_fetch_row_prepared('SELECT *
@@ -668,7 +806,7 @@ function xml_to_data_template($hash, &$xml_array, &$hash_cache, $import_as_new,
 				FROM data_template_rrd
 				WHERE hash = ?
 				AND data_template_id = ?
-				AND local_data_id=0',
+				AND local_data_id = 0',
 				array($parsed_hash['hash'], $data_template_id));
 
 			if (!empty($_data_template_rrd_id)) {
@@ -714,12 +852,42 @@ function xml_to_data_template($hash, &$xml_array, &$hash_cache, $import_as_new,
 				$save['rrd_heartbeat'] = read_config_option('poller_interval') * 2;
 			}
 
+			if ($legacy_template) {
+				if ($save['data_source_type_id'] == 1 || $save['data_source_type_id'] == 4) {
+					if ($save['rrd_maximum'] == '0' && $save['rrd_minimum'] == '0') {
+						$save['rrd_maximum'] = 'U';
+					}
+				} elseif ($save['data_source_type_id'] == 3 || $save['data_source_type_id'] == 7) {
+					if ($save['rrd_maximum'] == '0' && $save['rrd_minimum'] == '0') {
+						$save['rrd_maximum'] = 'U';
+						$save['rrd_minimum'] = 'U';
+					}
+				}
+			}
+
 			/* check for status changes */
 			$status += compare_data($save, $previous_data, 'data_template_rrd');
 
 			if (!$preview_only) {
 				$data_template_rrd_id = sql_save($save, 'data_template_rrd');
 
+				if ($legacy_template) {
+					// Correct max values in templates and data sources: GAUGE/ABSOLUTE (1,4)
+					db_execute("UPDATE data_template_rrd
+						SET rrd_maximum='U'
+						WHERE rrd_maximum = '0'
+						AND rrd_minimum = '0'
+						AND data_template_id = $data_template_id
+						AND data_source_type_id IN(1,4)");
+
+					// Correct min/max values in templates and data sources: DERIVE/DDERIVE (3,7)
+					db_execute("UPDATE data_template_rrd
+						SET rrd_maximum='U', rrd_minimum='U'
+						WHERE (rrd_maximum = '0' OR rrd_minimum = '0')
+						AND data_template_id = $data_template_id
+						AND data_source_type_id IN(3,7)");
+				}
+
 				$hash_cache['data_template_item'][$parsed_hash['hash']] = $data_template_rrd_id;
 			} else {
 				$hash_cache['data_template_item'][$parsed_hash['hash']] = $_data_template_rrd_id;
@@ -1077,9 +1245,9 @@ function xml_to_data_source_profile($hash, &$xml_array, &$hash_cache, $import_as
 			$hash_items = explode('|', $xml_array['cf_items']);
 
 			if (!empty($hash_items[0])) {
-				for ($i=0; $i<count($hash_items); $i++) {
+				for ($i=0; $i<cacti_count($hash_items); $i++) {
 					db_execute_prepared('REPLACE INTO data_source_profiles_cf
-						(data_source_profile_id,consolidation_function_id)
+						(data_source_profile_id, consolidation_function_id)
 						VALUES (?, ?)',
 						array($dsp_id, $hash_items[$i]));
 				}
@@ -1162,7 +1330,7 @@ function xml_to_host_template($hash, &$xml_array, &$hash_cache) {
 		$hash_items = explode('|', $xml_array['graph_templates']);
 
 		if (!empty($hash_items[0])) {
-			for ($i=0; $i<count($hash_items); $i++) {
+			for ($i=0; $i<cacti_count($hash_items); $i++) {
 				/* parse information from the hash */
 				$parsed_hash = parse_xml_hash($hash_items[$i]);
 
@@ -1171,7 +1339,7 @@ function xml_to_host_template($hash, &$xml_array, &$hash_cache) {
 
 				if (isset($hash_cache['graph_template'][$parsed_hash['hash']])) {
 					db_execute_prepared('REPLACE INTO host_template_graph
-						(host_template_id,graph_template_id)
+						(host_template_id, graph_template_id)
 						VALUES (?, ?)',
 						array($host_template_id, $hash_cache['graph_template'][$parsed_hash['hash']]));
 				}
@@ -1182,7 +1350,7 @@ function xml_to_host_template($hash, &$xml_array, &$hash_cache) {
 		$hash_items = explode('|', $xml_array['data_queries']);
 
 		if (!empty($hash_items[0])) {
-			for ($i=0; $i<count($hash_items); $i++) {
+			for ($i=0; $i<cacti_count($hash_items); $i++) {
 				/* parse information from the hash */
 				$parsed_hash = parse_xml_hash($hash_items[$i]);
 
@@ -1191,7 +1359,7 @@ function xml_to_host_template($hash, &$xml_array, &$hash_cache) {
 
 				if (isset($hash_cache['data_query'][$parsed_hash['hash']])) {
 					db_execute_prepared('REPLACE INTO host_template_snmp_query
-						(host_template_id,snmp_query_id)
+						(host_template_id, snmp_query_id)
 						VALUES (?, ?)',
 						array($host_template_id, $hash_cache['data_query'][$parsed_hash['hash']]));
 				}
@@ -1447,6 +1615,18 @@ function xml_to_data_input_method($hash, &$xml_array, &$hash_cache) {
 	/* track changes */
 	$status = 0;
 
+	$system_hashes = array(
+		'3eb92bb845b9660a7445cf9740726522', // Get SNMP Data
+		'bf566c869ac6443b0c75d1c32b5a350e', // Get SNMP Data (Indexed)
+		'80e9e4c4191a5da189ae26d0e237f015', // Get Script Data (Indexed)
+		'332111d8b54ac8ce939af87a7eac0c06', // Get Script Server Data (Indexed)
+	);
+
+	// Leave system data input methods alone
+	if (array_search($hash, $system_hashes) !== false) {
+		return $hash_cache;
+	}
+
 	/* aggregate field arrays */
 	$fields_data_input_field_edit += $fields_data_input_field_edit_1;
 
@@ -1593,7 +1773,7 @@ function compare_data($save, $previous_data, $table) {
 		$ignores = array();
 	}
 
-	if (!sizeof($previous_data)) {
+	if (!cacti_sizeof($previous_data)) {
 		return 0;
 	} else {
 		$different = 0;
@@ -1610,13 +1790,17 @@ function compare_data($save, $previous_data, $table) {
 
 					if (empty($previous_data[$column]) && empty($value)) {
 						continue;
+					} elseif ($table == 'data_template_rrd' && $column == 'rrd_heartbeat') {
+						continue;
+					} elseif ($table == 'data_template_data' && $column == 'rrd_step') {
+						continue;
 					}
 				} elseif (empty($previous_data[$column]) && empty($value)) {
 					continue;
 				}
 
 				$different++;
-				$import_debug_info['differences'][] = 'Table: ' . $table . ', Column: ' . $column . ', New Value: \'' . $value . '\', Old Value: \'' . $previous_data[$column] . '\'';
+				$import_debug_info['differences'][] = 'Table: ' . $table . ', Column: ' . $column . ', New Value: ' . $value . ', Old Value: ' . $previous_data[$column];
 			}
 		}
 
@@ -1699,6 +1883,15 @@ function hash_to_friendly_name($hash, $display_type_name) {
 			array($parsed_hash['hash']));
 		case 'round_robin_archive':
 			return $prepend;
+		default:
+			$param = array();
+			$param['hash']    = $parsed_hash['hash'];
+			$param['type']    = $parsed_hash['type'];
+			$param['prepend'] = $prepend;
+
+			api_plugin_hook_function('get_friendly_name', $param);
+
+			return $param['prepend'];
 	}
 }
 
@@ -1722,6 +1915,8 @@ function resolve_hash_to_id($hash, &$hash_cache_array) {
 			return 0;
 		} elseif ($parsed_hash['hash'] == 'cbbe5c1ddfb264a6e5d509ce1c78c95f') {
 			return 0;
+		} elseif ($parsed_hash['hash'] == '51bde3d899e12bde28ad979166985584') {
+			return 0;
 		}
 
 		cacti_log("Import Error: Import found an invalid dependency hash of :" . $parsed_hash['hash'] . ".  Please open bug on GitHub.");
@@ -1733,6 +1928,8 @@ function resolve_hash_to_id($hash, &$hash_cache_array) {
 }
 
 function parse_xml_hash($hash) {
+	global $legacy_template;
+
 	if (preg_match('/hash_([a-f0-9]{2})([a-f0-9]{4})([a-f0-9]{32})/', $hash, $matches)) {
 		$parsed_hash['type']    = check_hash_type($matches[1]);
 		$parsed_hash['version'] = strval(check_hash_version($matches[2]));
@@ -1740,6 +1937,20 @@ function parse_xml_hash($hash) {
 
 		/* an error has occurred */
 		if (($parsed_hash['type'] === false) || ($parsed_hash['version'] === false)) {
+			raise_message(7); /* xml parse error */
+
+			cacti_log(__FUNCTION__ . ' ERROR type or version not found for hash: ' . $hash, false, 'IMPORT', POLLER_VERBOSITY_LOW);
+			return false;
+		}
+	} elseif (preg_match('/hash_([a-f0-9]{2})([a-f0-9]{32})/', $hash, $matches)) {
+		$parsed_hash['type']    = check_hash_type($matches[1]);
+		$parsed_hash['version'] = strval(check_hash_version('0101'));
+		$parsed_hash['hash']    = $matches[2];
+
+		/* an error has occurred */
+		if (($parsed_hash['type'] === false) || ($parsed_hash['version'] === false)) {
+			raise_message(7); /* xml parse error */
+
 			cacti_log(__FUNCTION__ . ' ERROR type or version not found for hash: ' . $hash, false, 'IMPORT', POLLER_VERBOSITY_LOW);
 			return false;
 		}
@@ -1751,10 +1962,17 @@ function parse_xml_hash($hash) {
 			return false;
 		}
 
+		raise_message(7); /* xml parse error */
+
 		cacti_log(__FUNCTION__ . ' ERROR wrong hash format for hash: ' . $hash, false, 'IMPORT', POLLER_VERBOSITY_LOW);
+
 		return false;
 	}
 
+	if (cacti_version_compare($parsed_hash['version'], '1.2.3', '<')) {
+		$legacy_template = true;
+	}
+
 	return $parsed_hash;
 }
 
@@ -1781,31 +1999,28 @@ function check_hash_type($hash_type) {
 function check_hash_version($hash_version) {
 	global $cacti_version_codes, $config;
 
-	$i = 0;
-
 	foreach ($cacti_version_codes as $version => $code) {
 		if ($version == CACTI_VERSION) {
-			$current_version_index = $i;
+			$current_version_code = $code;
 		}
 
 		if ($code == $hash_version) {
-			$hash_version_index = $i;
+			$hash_version_code = $code;
 			$current_version = $version;
 		}
-
-		$i++;
 	}
 
-	if (!isset($current_version_index)) {
-		cacti_log("ERROR: $hash_version Current Cacti Version does not exist!", false, 'IMPORT', POLLER_VERBOSITY_HIGH);
+	if (!isset($current_version_code)) {
+		cacti_log("ERROR: $hash_version Current Cacti Version does not exist!", false, 'IMPORT');
 		raise_message(15); /* error: current cacti version does not exist! */
 		return false;
-	} elseif (!isset($hash_version_index)) {
-		cacti_log("ERROR: $hash_version hash version does not exist!", false, 'IMPORT', POLLER_VERBOSITY_HIGH);
+	} elseif (!isset($hash_version_code)) {
+		cacti_log("ERROR: $hash_version hash version does not exist!", false, 'IMPORT');
 		raise_message(16); /* error: hash version does not exist! */
 		return false;
-	} elseif ($hash_version_index > $current_version_index) {
-		cacti_log("ERROR: $hash_version hash version if for a newer Cacti!", false, 'IMPORT', POLLER_VERBOSITY_HIGH);
+	} elseif ($hash_version_code > $current_version_code) {
+		cacti_log("ERROR: $hash_version_code > $current_version_code", false, 'IMPORT');
+		cacti_log("ERROR: $hash_version hash version is for a newer Cacti!", false, 'IMPORT');
 		raise_message(17); /* error: hash made with a newer version of cacti */
 		return false;
 	}
@@ -1846,10 +2061,10 @@ function import_display_results($import_debug_info, $filestatus, $web = false, $
 	// Capture to a buffer
 	ob_start();
 
-	if (sizeof($import_debug_info)) {
+	if (cacti_sizeof($import_debug_info)) {
 		html_start_box(($preview ? __('Import Preview Results'):__('Import Results')), '100%', '', '3', 'center', '');
 
-		if (sizeof($filestatus)) {
+		if (cacti_sizeof($filestatus)) {
 			if ($preview) {
 				print "<tr class='odd'><td><p class='textArea'>" . __('Cacti would make the following changes if the Package was imported:') . "</p>\n";
 			} else {
@@ -1921,7 +2136,7 @@ function import_display_results($import_debug_info, $filestatus, $web = false, $
 					$dep_text   = '';
 					$dep_errors = false;
 
-					if ((isset($vals['dep'])) && (sizeof($vals['dep']) > 0)) {
+					if ((isset($vals['dep'])) && (cacti_sizeof($vals['dep']) > 0)) {
 						foreach ($vals['dep'] as $dep_hash => $dep_status) {
 							if ($dep_status == 'met') {
 								$dep_status_text = "<span class='foundDependency'>" . __('Found Dependency:') . "</span>";
@@ -1952,7 +2167,7 @@ function import_display_results($import_debug_info, $filestatus, $web = false, $
 	} else {
 		$output = ob_get_clean();
 		$output = explode("\n", $output);
-		if (sizeof($output)) {
+		if (cacti_sizeof($output)) {
 			foreach($output as $line) {
 				$line = trim(str_replace('&nbsp;', '', strip_tags($line)));
 
