@@ -1,7 +1,7 @@
 <?php
 /*
  +-------------------------------------------------------------------------+
- | Copyright (C) 2004-2018 The Cacti Group                                 |
+ | Copyright (C) 2004-2020 The Cacti Group                                 |
  |                                                                         |
  | This program is free software; you can redistribute it and/or           |
  | modify it under the terms of the GNU General Public License             |
@@ -152,6 +152,70 @@ function cacti_ldap_search_dn($username, $dn = '', $host = '', $port = '', $port
 	return $ldap->Search();
 }
 
+/* cacti_ldap_search_cn
+  @arg $username - username to search for in the LDAP directory
+  @arg $cn - array of CN to search on LDAP
+  @arg $dn - configured LDAP DN for binding, '<username>' will be replaced with $username
+  @arg $host - Hostname or IP of LDAP server, Default = Configured settings value
+  @arg $port - Port of the LDAP server uses, Default = Configured settings value
+  @arg $port_ssl - Port of the LDAP server uses for SSL, Default = Configured settings value
+  @arg $version - '2' or '3', LDAP protocol version, Default = Configured settings value
+  @arg $encryption - '0' None, '1' SSL, '2' TLS, Default = Configured settings value
+  @arg $referrals - '0' Referrals from server are ignored, '1' Referrals from server are processed, Default = Configured setting value
+  @arg $mode - '0' No Searching, '1' Anonymous Searching, '2' Specfic Searching, Default = Configured settings value
+  @arg $search_base - Search base DN, Default = Configured settings value
+  @arg $search_filter - Filter to find the user, Default = Configured settings value
+  @arg $specific_dn - DN for binding to perform user search, Default = Configured settings value
+  @arg $specific_password - Password for binding to perform user search, Default - Configured settings value
+  @return - array of values
+    'cn' = array of values
+    'error_num' = error number returned
+    'error_text' = error text
+    'dn' = found dn of user
+Error codes:
+#       Text
+==============================================================
+0       User found
+1       No username defined
+2       Unable to create LDAP connection object
+3       Unable to find users DN
+4       Protocol error, unable to set version
+5       Protocol error, unable to start TLS communications
+6       Protocol error
+7       Invalid credential
+8       Insufficient access
+9       Unable to connect to server
+10      Timeout
+11      General bind error
+12      Unable to set referrals option
+13      More than one matching user found
+14      Specific DN and Password required
+15      CN unknown on LDAP
+99      PHP LDAP not enabled
+*/
+function cacti_ldap_search_cn($username, $cn = array(), $dn = '', $host = '', $port = '', $port_ssl = '', $version = '', $encryption = '',
+	$referrals = '', $mode = '', $search_base = '', $search_filter = '', $specific_dn = '', $specific_password = '') {
+
+	$ldap = new Ldap;
+
+	if (!empty($username))          $ldap->username          = $username;
+	if (!empty($cn))                $ldap->cn                = $cn;
+	if (!empty($dn))                $ldap->dn                = $dn;
+	if (!empty($host))              $ldap->host              = $host;
+	if (!empty($port))              $ldap->port              = $port;
+	if (!empty($port_ssl))          $ldap->port_ssl          = $port_ssl;
+	if (!empty($version))           $ldap->version           = $version;
+	if (!empty($encryption))        $ldap->encryption        = $encryption;
+	if (!empty($referrals))         $ldap->referrals         = $referrals;
+	if (!empty($mode))              $ldap->mode              = $mode;
+	if (!empty($search_base))       $ldap->search_base       = $search_base;
+	if (!empty($search_filter))     $ldap->search_filter     = $search_filter;
+	if (!empty($specific_dn))       $ldap->specific_dn       = $specific_dn;
+	if (!empty($specific_password)) $ldap->specific_password = $specific_password;
+
+	return $ldap->Getcn();
+}
+
 abstract class LdapError {
 	const None                  = 0;
 	const Success               = 0;
@@ -258,7 +322,7 @@ abstract class LdapError {
 				break;
 		}
 
-		return array('error_num' => $error_num, 'error_text' => $error_text, 'error_ldap' => $ldapError, 'dn' => '');
+		return array('error_num' => $error_num, 'error_text' => $error_text, 'error_ldap' => $ldapError, 'dn' => '', 'stack' => cacti_debug_backtrace('', false, false));
 	}
 }
 
@@ -307,6 +371,8 @@ class Ldap {
 
 		/* set an error handler for ldap */
 		set_error_handler(array($this, 'ErrorHandler'));
+
+		session_write_close();
 	}
 
 	function RestoreCactiHandler() {
@@ -315,6 +381,14 @@ class Ldap {
 
 		/* set an error handler for Cacti */
 		set_error_handler('CactiErrorHandler');
+
+		session_start();
+	}
+
+	function RecordError($output, $section = 'LDAP') {
+		$logDN = empty($output['dn']) ? '' : (', DN: ' . $output['dn']);
+		cacti_log($section . ': ' . $output['error_text'] . $logDN, false, 'AUTH');
+		cacti_log($section . ': ' . $output['stack'], false, 'AUTH', POLLER_VERBOSITY_HIGH);
 	}
 
 	function Authenticate() {
@@ -336,23 +410,25 @@ class Ldap {
 		$this->username = html_entity_decode($this->username, $this->GetMask(), 'UTF-8');
 		$this->password = html_entity_decode($this->password, $this->GetMask(), 'UTF-8');
 
+		/* Determine connection method and create LDAP Object */
 		$this->SetLdapHandler();
 
-		/* Determine connection method and create LDAP Object */
 		if ($this->encryption == '1') {
 			/* This only works with OpenLDAP, I'm pretty sure this will not work with Solaris, Tony */
+			cacti_log('LDAP: Connect using ldaps://' . $this->host . ':' . $this->port_ssl, false, 'AUTH', POLLER_VERBOSITY_HIGH);
 			$ldap_conn = ldap_connect('ldaps://' . $this->host . ':' . $this->port_ssl);
 		} else {
+			cacti_log('LDAP: Connect using ldap://'. $this->host . ':' . $this->port, false, 'AUTH', POLLER_VERBOSITY_HIGH);
 			$ldap_conn = ldap_connect($this->host, $this->port);
 		}
 
 		if ($ldap_conn) {
 			/* Set protocol version */
-			cacti_log('LDAP: Setting protocol version to ' . $this->version, false, 'AUTH');
+			cacti_log('LDAP: Setting protocol version to ' . $this->version, false, 'AUTH', POLLER_VERBOSITY_MEDIUM);
 
 			if (!ldap_set_option($ldap_conn, LDAP_OPT_PROTOCOL_VERSION, $this->version)) {
 				$output = LdapError::GetErrorDetails(LdapError::ProtocolErrorVersion);
-				cacti_log('LDAP: ' . $output['error_text'], false, 'AUTH');
+				Ldap::RecordError($output);
 				ldap_close($ldap_conn);
 				$this->RestoreCactiHandler();
 				return $output;
@@ -362,7 +438,7 @@ class Ldap {
 			if ($this->referrals == '0') {
 				if (!ldap_set_option($ldap_conn, LDAP_OPT_REFERRALS, 0)) {
 					$output = LdapError::GetErrorDetails(LdapError::ProtocolErrorReferral);
-					cacti_log('LDAP: ' . $output['error_text'], false, 'AUTH');
+					Ldap::RecordError($output);
 					ldap_close($ldap_conn);
 					$this->RestoreCactiHandler();
 					return $output;
@@ -373,7 +449,7 @@ class Ldap {
 			if ($this->encryption == '2') {
 				if (!ldap_start_tls($ldap_conn)) {
 					$output = LdapError::GetErrorDetails(LdapError::ProtocolErrorTls);
-					cacti_log('LDAP: ' . $output['error_text'], false, 'AUTH');
+					Ldap::RecordError($output);
 					ldap_close($ldap_conn);
 					$this->RestoreCactiHandler();
 					return $output;
@@ -381,6 +457,7 @@ class Ldap {
 			}
 
 			/* Bind to the LDAP directory */
+			cacti_log('LDAP: Binding with "' . $this->dn . '"', false, 'AUTH', POLLER_VERBOSITY_HIGH);
 			$ldap_response = ldap_bind($ldap_conn, $this->dn, $this->password);
 			if ($ldap_response) {
 				if ($this->group_require == 1) {
@@ -401,7 +478,7 @@ class Ldap {
 
 						/* we will test in two ways */
 						if ($first_entry !== false) {
-							$true_dn             = ldap_get_dn($ldap_conn, $first_entry);
+							$true_dn     = ldap_get_dn($ldap_conn, $first_entry);
 							$ldap_group_response = ldap_compare($ldap_conn, $this->group_dn, $this->group_attrib, $true_dn);
 						} else {
 							$ldap_group_response = ldap_compare($ldap_conn, $this->group_dn, $this->group_attrib, $this->username);
@@ -413,13 +490,13 @@ class Ldap {
 						$output = LdapError::GetErrorDetails(LdapError::Success);
 					} else if ($ldap_group_response === false) {
 						$output = LdapError::GetErrorDetails(LdapError::InsufficientAccess);
-						cacti_log('LDAP: ' . $output['error_text'], false, 'AUTH');
+						Ldap::RecordError($output);
 						ldap_close($ldap_conn);
 						$this->RestoreCactiHandler();
 						return $output;
 					} else {
 						$output = LdapError::GetErrorDetails(LdapError::SearchFoundNoGroup);
-						cacti_log('LDAP: ' . $output['error_text'], false, 'AUTH');
+						Ldap::RecordError($output);
 						ldap_close($ldap_conn);
 						$this->RestoreCactiHandler();
 						return $output;
@@ -433,7 +510,7 @@ class Ldap {
 				$ldap_error = ldap_errno($ldap_conn);
 				if ($ldap_error == 0x03) {
 					/* protocol error */
-					$ouptut = LdapError::GetErrorDetails(LdapError::ProtocolErrorGeneral, null, $ldap_error);
+					$output = LdapError::GetErrorDetails(LdapError::ProtocolErrorGeneral, null, $ldap_error);
 				} elseif ($ldap_error == 0x31) {
 					/* invalid credentials */
 					$output = LdapError::GetErrorDetails(LdapError::Failure);
@@ -460,7 +537,7 @@ class Ldap {
 		ldap_close($ldap_conn);
 
 		if ($output['error_num'] > 0) {
-			cacti_log('LDAP: ' . $output['error_text'], false, 'AUTH');
+			Ldap::RecordError($output);
 		}
 
 		$this->RestoreCactiHandler();
@@ -486,8 +563,167 @@ class Ldap {
 
 		/* validation */
 		if (empty($this->username)) {
-			cacti_log('LDAP_SEARCH: No username defined', false, 'AUTH');
-			return LdapError::GetErrorDetails(LdapError::UndefinedUsername);
+			$output = LdapError::GetErrorDetails(LdapError::UndefinedUsername);
+			Ldap::RecordError($output);
+			return $output;
+		}
+
+		/* Encode username */
+		$this->username = html_entity_decode($this->username, $this->GetMask(), 'UTF-8');
+
+		/* strip bad chars from username - prevent altering filter from username */
+		$this->username = str_replace(array('&', '|', '(', ')', '*', '>', '<', '!', '='), '', $this->username);
+		$this->dn = str_replace('<username>', $this->username, $this->dn);
+
+		if ($this->mode == '0') {
+			/* Just bind mode, make dn and return */
+			$output = LdapError::GetErrorDetails(LdapError::Success);
+			$output['dn'] = $this->dn;
+			return $output;
+		} elseif ($this->mode == '2') {
+			/* Specific */
+			if (empty($this->specific_dn) || empty($this->specific_password)) {
+				$output = LdapError::GetErrorDetails(LdapError::UndefinedDnOrPassword);
+				$output['dn'] = $this->dn;
+				Ldap::RecordError($output, 'LDAP_SEARCH');
+				return $output;
+			}
+		} elseif ($this->mode == '1'){
+			/* assume anonymous */
+			$this->specific_dn       = '';
+			$this->specific_password = '';
+		}
+
+		$this->search_filter = str_replace('<username>', $this->username, $this->search_filter);
+
+		/* Fix encoding on ldap specific search DN and password */
+		$this->specific_password = html_entity_decode($this->specific_password, $this->GetMask(), 'UTF-8');
+		$this->specific_dn       = html_entity_decode($this->specific_dn, $this->GetMask(), 'UTF-8');
+
+		/* Searching mode */
+		$this->SetLdapHandler();
+
+		if ($this->encryption == '1') {
+			/* This only works with OpenLDAP, I'm pretty sure this will not work with Solaris, Tony */
+			cacti_log('LDAP: Search using ldaps://' . $this->host . ':' . $this->port_ssl, false, 'AUTH', POLLER_VERBOSITY_HIGH);
+			$ldap_conn = ldap_connect('ldaps://' . $this->host . ':' . $this->port_ssl);
+		} else {
+			cacti_log('LDAP: Search using ldap://' . $this->host . ':' . $this->port, false, 'AUTH', POLLER_VERBOSITY_HIGH);
+			$ldap_conn = ldap_connect($this->host, $this->port);
+		}
+
+		if ($ldap_conn) {
+			/* Set protocol version */
+			if (!ldap_set_option($ldap_conn, LDAP_OPT_PROTOCOL_VERSION, $this->version)) {
+				/* protocol error */
+				$output = LdapError::GetErrorDetails(LdapError::ProtocolErrorVersion);
+				Ldap::RecordError($output, 'LDAP_SEARCH');
+				ldap_close($ldap_conn);
+
+				$this->RestoreCactiHandler();
+				return $output;
+			}
+
+			/* set referrals */
+			if ($this->referrals == '0') {
+				if (!ldap_set_option($ldap_conn, LDAP_OPT_REFERRALS, 0)) {
+					/* referrals set error */
+					$output = LdapError::GetErrorDetails(LdapError::ProtocolErrorReferral);
+					Ldap::RecordError($output, 'LDAP_SEARCH');
+					ldap_close($ldap_conn);
+
+					$this->RestoreCactiHandler();
+					return $output;
+				}
+			}
+
+			/* start TLS if requested */
+			if ($this->encryption == '2') {
+				if (!ldap_start_tls($ldap_conn)) {
+					/* TLS startup error */
+					$output = LdapError::GetErrorDetails(LdapError::ProtocolErrorTls);
+					Ldap::RecordError($output, 'LDAP_SEARCH');
+					ldap_close($ldap_conn);
+
+					$this->RestoreCactiHandler();
+					return $output;
+				}
+			}
+
+			/* bind to the directory */
+			if (ldap_bind($ldap_conn, $this->specific_dn, $this->specific_password)) {
+				/* Search */
+				$ldap_results = ldap_search($ldap_conn, $this->search_base, $this->search_filter, array('dn'));
+				if ($ldap_results) {
+					$ldap_entries = ldap_get_entries($ldap_conn, $ldap_results);
+
+					if ($ldap_entries['count'] == '1') {
+						/* single response return user dn */
+						$output = LdapError::GetErrorDetails(LdapError::Success);
+						$output['dn'] = $ldap_entries['0']['dn'];
+						Ldap::RecordError($output, 'LDAP_SEARCH');
+					} elseif ($ldap_entries['count'] > 1) {
+						/* more than 1 result */
+						$output = LdapError::GetErrorDetails(LdapError::SearchFoundMultiUser);
+					} else {
+						/* no search results */
+						$output = LdapError::GetErrorDetails(LdapError::SearchFoundNoUserDN);
+					}
+				} else {
+					/* no search results, user not found*/
+					$output = LdapError::GetErrorDetails(LdapError::SearchFoundNoUser);
+				}
+			} else {
+				/* unable to bind */
+				$ldap_error = ldap_errno($ldap_conn);
+				if ($ldap_error == 0x03) {
+					/* protocol error */
+					$output = LdapError::GetErrorDetails(LdapError::ProtocolErrorGeneral,null,$ldap_error);
+				} elseif ($ldap_error == 0x31) {
+					/* invalid credentials */
+					$output = LdapError::GetErrorDetails(LdapError::Failure);
+				} elseif ($ldap_error == 0x32) {
+					/* insuffient access */
+					$output = LdapError::GetErrorDetails(LdapError::InsufficientAccess);
+				} elseif ($ldap_error == 0x51) {
+					/* unable to connect to server */
+					$output = LdapError::GetErrorDetails(LdapError::ConnectionUnavailable);
+				} elseif ($ldap_error == 0x55) {
+					/* timeout */
+					$output = LdapError::GetErrorDetails(LdapError::ConnectionTimeout);
+				} else {
+					/* general bind error */
+					$output = LdapError::GetErrorDetails(LdapError::ProtocolErrorBind, null, $ldap_error);
+				}
+			}
+		} else {
+			/* unable to setup connection */
+			$output = LdapError::GetErrorDetails(LdapError::MissingLdapObject);
+		}
+
+		ldap_close($ldap_conn);
+
+		if ($output['error_num'] > 0) {
+			Ldap::RecordError($output, 'LDAP_SEARCH');
+		}
+
+		$this->RestoreCactiHandler();
+		return $output;
+	}
+
+	function Getcn() {
+		$output = array();
+
+		/* function check */
+		if (!function_exists('ldap_connect')) {
+			return LdapError::GetErrorDetails(LdapError::Disabled);
+		}
+
+		/* validation */
+		if (empty($this->username)) {
+			$output = LdapError::GetErrorDetails(LdapError::Disabled);
+			Ldap::ReportError($output, 'LDAP_SEARCH');
+			return $output;
 		}
 
 		/* Encode username */
@@ -522,41 +758,51 @@ class Ldap {
 		$this->specific_dn       = html_entity_decode($this->specific_dn, $this->GetMask(), 'UTF-8');
 
 		/* Searching mode */
+		$this->SetLdapHandler();
+
 		if ($this->encryption == '1') {
 			/* This only works with OpenLDAP, I'm pretty sure this will not work with Solaris, Tony */
-			$ldap_conn = @ldap_connect('ldaps://' . $this->host . ':' . $this->port_ssl);
+			cacti_log('LDAP: GetCN using ldaps://' . $this->host . ':' . $this->port_ssl, false, 'AUTH', POLLER_VERBOSITY_HIGH);
+			$ldap_conn = ldap_connect('ldaps://' . $this->host . ':' . $this->port_ssl);
 		} else {
-			$ldap_conn = @ldap_connect($this->host, $this->port);
+			cacti_log('LDAP: GetCN using ldap://' . $this->host . ':' . $this->port_ssl, false, 'AUTH', POLLER_VERBOSITY_HIGH);
+			$ldap_conn = ldap_connect($this->host, $this->port);
 		}
 
 		if ($ldap_conn) {
 			/* Set protocol version */
-			if (!@ldap_set_option($ldap_conn, LDAP_OPT_PROTOCOL_VERSION, $this->version)) {
+			if (!ldap_set_option($ldap_conn, LDAP_OPT_PROTOCOL_VERSION, $this->version)) {
 				/* protocol error */
 				$output = LdapError::GetErrorDetails(LdapError::ProtocolErrorVersion);
-				cacti_log('LDAP_SEARCH: ' . $output['error_text'], false, 'AUTH');
-				@ldap_close($ldap_conn);
+				Ldap::RecordError($output, 'LDAP_SEARCH');
+				ldap_close($ldap_conn);
+
+				$this->RestoreCactiHandler();
 				return $output;
 			}
 
 			/* set referrals */
 			if ($this->referrals == '0') {
-				if (!@ldap_set_option($ldap_conn, LDAP_OPT_REFERRALS, 0)) {
+				if (!ldap_set_option($ldap_conn, LDAP_OPT_REFERRALS, 0)) {
 					/* referrals set error */
 					$output = LdapError::GetErrorDetails(LdapError::ProtocolErrorReferral);
-					cacti_log('LDAP_SEARCH: ' . $output['error_text'], false, 'AUTH');
-					@ldap_close($ldap_conn);
+					Ldap::RecordError($output, 'LDAP_SEARCH');
+					ldap_close($ldap_conn);
+
+					$this->RestoreCactiHandler();
 					return $output;
 				}
 			}
 
 			/* start TLS if requested */
 			if ($this->encryption == '2') {
-				if (!@ldap_start_tls($ldap_conn)) {
+				if (!ldap_start_tls($ldap_conn)) {
 					/* TLS startup error */
 					$output = LdapError::GetErrorDetails(LdapError::ProtocolErrorTls);
-					cacti_log('LDAP_SEARCH: ' . $output['error_text'], false, 'AUTH');
-					@ldap_close($ldap_conn);
+					Ldap::RecordError($output, 'LDAP_SEARCH');
+					ldap_close($ldap_conn);
+
+					$this->RestoreCactiHandler();
 					return $output;
 				}
 			}
@@ -564,32 +810,39 @@ class Ldap {
 			/* bind to the directory */
 			if (ldap_bind($ldap_conn, $this->specific_dn, $this->specific_password)) {
 				/* Search */
-				$ldap_results = ldap_search($ldap_conn, $this->search_base, $this->search_filter, array('dn'));
+				$ldap_results = ldap_search($ldap_conn, $this->search_base, $this->search_filter, $this->cn);
+
 				if ($ldap_results) {
 					$ldap_entries =  ldap_get_entries($ldap_conn, $ldap_results);
-
-					if ($ldap_entries['count'] == '1') {
-						/* single response return user dn */
+					/* We find 1 entries */
+					if ($ldap_entries['count'] == 1) {
 						$output = LdapError::GetErrorDetails(LdapError::Success);
-						$output['dn'] = $ldap_entries['0']['dn'];
-						cacti_log('LDAP_SEARCH: ' . $output['error_text'] . ', DN \'' . $output['dn'] . '\'', false, 'AUTH');
-					} elseif ($ldap_entries['count'] > 1) {
-						/* more than 1 result */
-						$output = LdapError::GetErrorDetails(LdapError::SearchFoundMultiUser);
+						// check if we got an full username entry
+						if (array_key_exists($this->cn[0], $ldap_entries[0])) {
+							$output['cn'][$this->cn[0]] = $ldap_entries[0][$this->cn[0]][0];
+						} else {
+							$output['cn'][$this->cn[0]] = '';
+						}
+
+						// check if we got an email entry
+						if(array_key_exists($this->cn[1], $ldap_entries[0])) {
+							$output['cn'][$this->cn[1]] = $ldap_entries[0][$this->cn[1]][0];
+						} else {
+							$output['cn'][$this->cn[1]] = '';
+						}
 					} else {
-						/* no search results */
-						$output = LdapError::GetErrorDetails(LdapError::SearchFoundNoUserDN);
+						$output = LdapError::GetErrorDetails(LdapError::SearchFoundMultiUser);
 					}
 				} else {
 					/* no search results, user not found*/
-					$output = LdapError::GetErrorDetails(LdapError::SearchFoundNoUser);
+					$output = LdapError::GetErrorDetails(LdapError::SearchFoundNoUserDN);
 				}
 			} else {
 				/* unable to bind */
 				$ldap_error = ldap_errno($ldap_conn);
 				if ($ldap_error == 0x03) {
 					/* protocol error */
-					$output = LdapError::GetErrorDetails(LdapError::ProtocolErrorGeneral,null,$ldapError);
+					$output = LdapError::GetErrorDetails(LdapError::ProtocolErrorGeneral, null, $ldap_error);
 				} elseif ($ldap_error == 0x31) {
 					/* invalid credentials */
 					$output = LdapError::GetErrorDetails(LdapError::Failure);
@@ -608,16 +861,17 @@ class Ldap {
 				}
 			}
 		} else {
-			/* unable to setup connection */
+			/* Error intializing LDAP */
 			$output = LdapError::GetErrorDetails(LdapError::MissingLdapObject);
 		}
 
-		@ldap_close($ldap_conn);
+		ldap_close($ldap_conn);
 
 		if ($output['error_num'] > 0) {
-			cacti_log('LDAP_SEARCH: ' . $output['error_text'], false, 'AUTH');
+			Ldap::RecordError($output, 'LDAP_SEARCH_CN');
 		}
 
+		$this->RestoreCactiHandler();
 		return $output;
 	}
 }
