@@ -1,7 +1,7 @@
 <?php
 /*
  +-------------------------------------------------------------------------+
- | Copyright (C) 2004-2018 The Cacti Group                                 |
+ | Copyright (C) 2004-2020 The Cacti Group                                 |
  |                                                                         |
  | This program is free software; you can redistribute it and/or           |
  | modify it under the terms of the GNU General Public License             |
@@ -22,18 +22,18 @@
  +-------------------------------------------------------------------------+
 */
 
-function do_hook ($name) {
+function do_hook($name) {
 	$data = func_get_args();
-	$data = api_plugin_hook ($name, $data);
+	$data = api_plugin_hook($name, $data);
 	return $data;
 }
 
 function do_hook_function($name,$parm=NULL) {
-	return api_plugin_hook_function ($name, $parm);
+	return api_plugin_hook_function($name, $parm);
 }
 
-function api_user_realm_auth ($filename = '') {
-	return api_plugin_user_realm_auth ($filename);
+function api_user_realm_auth($filename = '') {
+	return api_plugin_user_realm_auth($filename);
 }
 
 /**
@@ -41,8 +41,9 @@ function api_user_realm_auth ($filename = '') {
  * @param string $name Name of hook to fire
  * @return mixed $data
  */
-function api_plugin_hook ($name) {
+function api_plugin_hook($name) {
 	global $config, $plugin_hooks, $plugins_integrated;
+
 	$args = func_get_args();
 	$ret = '';
 
@@ -81,7 +82,7 @@ function api_plugin_hook ($name) {
 	return $args;
 }
 
-function api_plugin_hook_function ($name, $parm = NULL) {
+function api_plugin_hook_function($name, $parm = NULL) {
 	global $config, $plugin_hooks, $plugins_integrated;
 
 	$ret = $parm;
@@ -130,8 +131,14 @@ function api_plugin_run_plugin_hook($hook, $plugin, $function, $args) {
 
 		$required_capabilities = array(
 			// Poller related
+			'poller_top'               => array('remote_collect'), // Poller Top
 			'poller_bottom'            => array('remote_collect'), // Poller execution, api_plugin_hook
 			'update_host_status'       => array('remote_collect'), // Processing poller output, api_plugin_hook
+			'poller_output'            => array('remote_collect'), // Poller output activities
+			'poller_command_args'      => array('remote_collect'), // Command line arguments
+			'cacti_stats_update'       => array('remote_collect'), // Updating statistics
+			'poller_finishing'         => array('remote_collect'), // Poller post processing
+			'poller_exiting'           => array('remote_collect'), // Poller exception handling
 
 			// GUI Related
 			'page_head'                => array('online_view', 'offline_view'), // Navigation, api_plugin_hook
@@ -234,13 +241,21 @@ function api_plugin_get_dependencies($plugin) {
 
 	$file = $config['base_path'] . '/plugins/' . $plugin . '/INFO';
 
+	$returndeps = array();
+
 	if (file_exists($file)) {
 		$info = parse_ini_file($file, true);
 
-		if (isset($info['info']['requires'])) {
-			$components = explode(',', $info['info']['requires']);
-			foreach($components as $c) {
-				$returndeps[trim($c)] = trim($c);
+		if (isset($info['info']['requires']) && trim($info['info']['requires']) != '') {
+			$parts = explode(' ', trim($info['info']['requires']));
+
+			foreach ($parts as $p) {
+				$vparts = explode(':', $p);
+				if (isset($vparts[1])) {
+					$returndeps[$vparts[0]] = $vparts[1];
+				} else {
+					$returndeps[$p] = true;
+				}
 			}
 
 			return $returndeps;
@@ -250,10 +265,28 @@ function api_plugin_get_dependencies($plugin) {
 	return false;
 }
 
-function api_plugin_installed($plugin) {
-	$plugin_data = db_fetch_row_prepared('SELECT directory, status FROM plugin_config WHERE directory = ?', array($plugin));
+function api_plugin_minimum_version($plugin, $version) {
+	if (strlen($version)) {
+		$plugin_version = db_fetch_cell_prepared('SELECT version
+			FROM plugin_config
+			WHERE directory = ?',
+			array($plugin));
 
-	if (sizeof($plugin_data)) {
+		$result = cacti_version_compare($version, $plugin_version, '<=');
+	} else {
+		$plugin_version = '<not read>';
+		$result = true;
+	}
+	return $result;
+}
+
+function api_plugin_installed($plugin) {
+	$plugin_data = db_fetch_row_prepared('SELECT directory, status
+		FROM plugin_config
+		WHERE directory = ?',
+		array($plugin));
+
+	if (cacti_sizeof($plugin_data)) {
 		if ($plugin_data['status'] >= 1) {
 			return true;
 		}
@@ -275,7 +308,7 @@ function api_plugin_remote_capabilities($plugin) {
 		if (file_exists($file)) {
 			$info = parse_ini_file($file, true);
 
-			if (sizeof($info)) {
+			if (cacti_sizeof($info)) {
 				$info_data[$plugin] = $info['info'];
 			}
 		}
@@ -338,7 +371,7 @@ function api_plugin_status_run($hook, $required_capabilities, $plugin_capabiliti
 	return false;
 }
 
-function api_plugin_db_table_create ($plugin, $table, $data) {
+function api_plugin_db_table_create($plugin, $table, $data) {
 	global $config;
 
 	include_once($config['library_path'] . '/database.php');
@@ -356,21 +389,40 @@ function api_plugin_db_table_create ($plugin, $table, $data) {
 		$sql = 'CREATE TABLE `' . $table . "` (\n";
 		foreach ($data['columns'] as $column) {
 			if (isset($column['name'])) {
-				if ($c > 0)
+				if ($c > 0) {
 					$sql .= ",\n";
+				}
+
 				$sql .= '`' . $column['name'] . '`';
-				if (isset($column['type']))
+
+				if (isset($column['type'])) {
 					$sql .= ' ' . $column['type'];
-				if (isset($column['unsigned']))
+				}
+
+				if (isset($column['unsigned'])) {
 					$sql .= ' unsigned';
-				if (isset($column['NULL']) && $column['NULL'] == false)
+				}
+
+				if (isset($column['NULL']) && $column['NULL'] == false) {
 					$sql .= ' NOT NULL';
-				if (isset($column['NULL']) && $column['NULL'] == true && !isset($column['default']))
+				}
+
+				if (isset($column['NULL']) && $column['NULL'] == true && !isset($column['default'])) {
 					$sql .= ' default NULL';
-				if (isset($column['default']))
-					$sql .= ' default ' . (is_numeric($column['default']) ? $column['default'] : "'" . $column['default'] . "'");
-				if (isset($column['auto_increment']))
+				}
+
+				if (isset($column['default'])) {
+					if (strtolower($column['type']) == 'timestamp' && $column['default'] === 'CURRENT_TIMESTAMP') {
+						$sql .= ' default CURRENT_TIMESTAMP';
+					} else {
+						$sql .= ' default ' . (is_numeric($column['default']) ? $column['default'] : "'" . $column['default'] . "'");
+					}
+				}
+
+				if (isset($column['auto_increment'])) {
 					$sql .= ' auto_increment';
+				}
+
 				$c++;
 			}
 		}
@@ -379,10 +431,10 @@ function api_plugin_db_table_create ($plugin, $table, $data) {
 			$sql .= ",\n PRIMARY KEY (`" . $data['primary'] . '`)';
 		}
 
-		if (isset($data['keys']) && sizeof($data['keys'])) {
+		if (isset($data['keys']) && cacti_sizeof($data['keys'])) {
 			foreach ($data['keys'] as $key) {
 				if (isset($key['name'])) {
-					$sql .= ",\n INDEX `" . $key['name'] . '` (`' . $key['columns'] . '`)';
+					$sql .= ",\n INDEX `" . $key['name'] . '` (' . db_format_index_create($key['columns']) . ')';
 				}
 			}
 		}
@@ -390,7 +442,7 @@ function api_plugin_db_table_create ($plugin, $table, $data) {
 		if (isset($data['unique_keys'])) {
 			foreach ($data['unique_keys'] as $key) {
 				if (isset($key['name'])) {
-					$sql .= ",\n UNIQUE INDEX `" . $key['name'] . '` (`' . $key['columns'] . '`)';
+					$sql .= ",\n UNIQUE INDEX `" . $key['name'] . '` (' . db_format_index_create($key['columns']) . ')';
 				}
 			}
 		}
@@ -401,6 +453,14 @@ function api_plugin_db_table_create ($plugin, $table, $data) {
 			$sql .= " COMMENT = '" . $data['comment'] . "'";
 		}
 
+		if (isset($data['charset'])) {
+			$sql .= ' DEFAULT CHARSET=' . $data['charset'];
+		}
+
+		if (isset($data['row_format']) && db_get_global_variable('innodb_file_format') == 'Barracuda') {
+			$sql .= ' ROW_FORMAT=' . $data['row_format'];
+		}
+
 		if (db_execute($sql)) {
 			db_execute_prepared("REPLACE INTO plugin_db_changes
 				(plugin, `table`, `column`, `method`)
@@ -410,14 +470,14 @@ function api_plugin_db_table_create ($plugin, $table, $data) {
 	}
 }
 
-function api_plugin_db_changes_remove ($plugin) {
+function api_plugin_db_changes_remove($plugin) {
 	$tables = db_fetch_assoc_prepared("SELECT `table`
 		FROM plugin_db_changes
 		WHERE plugin = ?
 		AND method ='create'",
 		array($plugin), false);
 
-	if (count($tables)) {
+	if (cacti_count($tables)) {
 		foreach ($tables as $table) {
 			db_execute('DROP TABLE IF EXISTS `' . $table['table'] . '`;');
 		}
@@ -430,7 +490,7 @@ function api_plugin_db_changes_remove ($plugin) {
 		AND method ='addcolumn'",
 		array($plugin), false);
 
-	if (count($columns)) {
+	if (cacti_count($columns)) {
 		foreach ($columns as $column) {
 			db_execute('ALTER TABLE `' . $column['table'] . '` DROP `' . $column['column'] . '`');
 		}
@@ -453,20 +513,38 @@ function api_plugin_db_add_column ($plugin, $table, $column) {
 	}
 	if (isset($column['name']) && !in_array($column['name'], $columns)) {
 		$sql = 'ALTER TABLE `' . $table . '` ADD `' . $column['name'] . '`';
-		if (isset($column['type']))
+
+		if (isset($column['type'])) {
 			$sql .= ' ' . $column['type'];
-		if (isset($column['unsigned']))
+		}
+
+		if (isset($column['unsigned'])) {
 			$sql .= ' unsigned';
-		if (isset($column['NULL']) && $column['NULL'] == false)
+		}
+
+		if (isset($column['NULL']) && $column['NULL'] == false) {
 			$sql .= ' NOT NULL';
-		if (isset($column['NULL']) && $column['NULL'] == true && !isset($column['default']))
+		}
+
+		if (isset($column['NULL']) && $column['NULL'] == true && !isset($column['default'])) {
 			$sql .= ' default NULL';
-		if (isset($column['default']))
-			$sql .= ' default ' . (is_numeric($column['default']) ? $column['default'] : "'" . $column['default'] . "'");
-		if (isset($column['auto_increment']))
+		}
+
+		if (isset($column['default'])) {
+			if (strtolower($column['type']) == 'timestamp' && $column['default'] === 'CURRENT_TIMESTAMP') {
+				$sql .= ' default CURRENT_TIMESTAMP';
+			} else {
+				$sql .= ' default ' . (is_numeric($column['default']) ? $column['default'] : "'" . $column['default'] . "'");
+			}
+		}
+
+		if (isset($column['auto_increment'])) {
 			$sql .= ' auto_increment';
-		if (isset($column['after']))
+		}
+
+		if (isset($column['after'])) {
 			$sql .= ' AFTER ' . $column['after'];
+		}
 
 		if (db_execute($sql)) {
 			db_execute_prepared("INSERT INTO plugin_db_changes (plugin, `table`, `column`, `method`) VALUES (?, ?, ?, 'addcolumn')", array($plugin, $table, $column['name']));
@@ -474,37 +552,51 @@ function api_plugin_db_add_column ($plugin, $table, $column) {
 	}
 }
 
-function api_plugin_install ($plugin) {
-	global $config;
-	include_once($config['base_path'] . "/plugins/$plugin/setup.php");
-
+function api_plugin_can_install($plugin, &$message) {
 	$dependencies = api_plugin_get_dependencies($plugin);
+	$message = '';
+	$proceed = true;
+	if (is_array($dependencies) && cacti_sizeof($dependencies)) {
+		foreach($dependencies as $dependency => $version) {
+			if (!api_plugin_minimum_version($dependency, $version)) {
+				$message .= __('%s Version %s or above is required for %s. ', ucwords($dependency), $version, ucwords($plugin));
 
-	if (is_array($dependencies) && sizeof($dependencies)) {
-		$message = '';
-		$proceed = true;
-		foreach($dependencies as $dependency) {
-			if (!api_plugin_installed($dependency)) {
-				$message .= ($message != '' ? '<br>':'') . __('Plugin %s is required for %s, and it is not installed.', $dependency, $plugin);
+				$proceed = false;
+			} else if (!api_plugin_installed($dependency)) {
+				$message .= __('%s is required for %s, and it is not installed. ', ucwords($dependency), ucwords($plugin));
 
 				$proceed = false;
 			}
 		}
+	}
+	return $proceed;
+}
 
-		if (!$proceed) {
-			$message .= '<br>' . __('Plugin cannot be installed.');
-			$_SESSION['reports_message'] = $message;
+function api_plugin_install($plugin) {
+	global $config;
 
-			raise_message('reports_message');
+	$dependencies = api_plugin_get_dependencies($plugin);
 
-			header('Location: plugins.php?header=false');
-			exit;
-		}
+	$proceed = api_plugin_can_install($plugin, $message);
+	if (!$proceed) {
+		$message .= '<br><br>' . __('Plugin cannot be installed.');
+		raise_message($message, MESSAGE_LEVEL_ERROR);
+
+		header('Location: plugins.php?header=false');
+		exit;
 	}
 
-	$exists = db_fetch_assoc_prepared('SELECT id FROM plugin_config WHERE directory = ?', array($plugin), false);
-	if (sizeof($exists)) {
-		db_execute_prepared('DELETE FROM plugin_config WHERE directory = ?', array($plugin));
+	include_once($config['base_path'] . "/plugins/$plugin/setup.php");
+
+	$exists = db_fetch_assoc_prepared('SELECT id
+		FROM plugin_config
+		WHERE directory = ?',
+		array($plugin), false);
+
+	if (cacti_sizeof($exists)) {
+		db_execute_prepared('DELETE FROM plugin_config
+			WHERE directory = ?',
+			array($plugin));
 	}
 
 	$name = $author = $webpage = $version = '';
@@ -523,7 +615,10 @@ function api_plugin_install ($plugin) {
 		$version = $info['version'];
 	}
 
-	db_execute_prepared('INSERT INTO plugin_config (directory, name, author, webpage, version) VALUES (?, ?, ?, ?, ?)', array($plugin, $name, $author, $webpage, $version));
+	db_execute_prepared('INSERT INTO plugin_config
+		(directory, name, author, webpage, version)
+		VALUES (?, ?, ?, ?, ?)',
+		array($plugin, $name, $author, $webpage, $version));
 
 	$function = 'plugin_' . $plugin . '_install';
 	if (function_exists($function)){
@@ -539,7 +634,7 @@ function api_plugin_install ($plugin) {
 	}
 }
 
-function api_plugin_uninstall ($plugin) {
+function api_plugin_uninstall($plugin) {
 	global $config;
 	include_once($config['base_path'] . "/plugins/$plugin/setup.php");
 	// Run the Plugin's Uninstall Function first
@@ -555,36 +650,41 @@ function api_plugin_uninstall ($plugin) {
 
 function api_plugin_check_config ($plugin) {
 	global $config;
-	include_once($config['base_path'] . "/plugins/$plugin/setup.php");
-	$function = 'plugin_' . $plugin . '_check_config';
-	if (function_exists($function)) {
-		return $function();
+	clearstatcache();
+	if (file_exists($config['base_path'] . "/plugins/$plugin/setup.php")) {
+		include_once($config['base_path'] . "/plugins/$plugin/setup.php");
+		$function = 'plugin_' . $plugin . '_check_config';
+
+		if (function_exists($function)) {
+			return $function();
+		}
+		return true;
 	}
-	return true;
+	return false;
 }
 
-function api_plugin_enable ($plugin) {
-	$ready = api_plugin_check_config ($plugin);
+function api_plugin_enable($plugin) {
+	$ready = api_plugin_check_config($plugin);
 	if ($ready) {
-		api_plugin_enable_hooks ($plugin);
+		api_plugin_enable_hooks($plugin);
 		db_execute_prepared('UPDATE plugin_config SET status = 1 WHERE directory = ?', array($plugin));
 	}
 }
 
-function api_plugin_is_enabled ($plugin) {
+function api_plugin_is_enabled($plugin) {
 	$status = db_fetch_cell_prepared('SELECT status FROM plugin_config WHERE directory = ?', array($plugin), false);
 	if ($status == '1')
 		return true;
 	return false;
 }
 
-function api_plugin_disable ($plugin) {
-	api_plugin_disable_hooks ($plugin);
+function api_plugin_disable($plugin) {
+	api_plugin_disable_hooks($plugin);
 	db_execute_prepared('UPDATE plugin_config SET status = 4 WHERE directory = ?', array($plugin));
 }
 
-function api_plugin_disable_all ($plugin) {
-	api_plugin_disable_hooks_all ($plugin);
+function api_plugin_disable_all($plugin) {
+	api_plugin_disable_hooks_all($plugin);
 	db_execute_prepared('UPDATE plugin_config SET status = 4 WHERE directory = ?', array($plugin));
 }
 
@@ -612,43 +712,69 @@ function api_plugin_movedown($plugin) {
 	db_execute_prepared('UPDATE plugin_config SET id = ? WHERE id = ?', array($id, $temp_id));
 }
 
-function api_plugin_register_hook ($plugin, $hook, $function, $file) {
-	$exists = db_fetch_assoc_prepared('SELECT id
+function api_plugin_register_hook($plugin, $hook, $function, $file, $enable = false) {
+	$status = 0;
+	$exists = db_fetch_cell_prepared('SELECT COUNT(*)
 		FROM plugin_hooks
 		WHERE name = ?
 		AND hook = ?',
 		array($plugin, $hook), false);
 
-	if (!count($exists)) {
+	if (!$exists) {
+		// enable the hooks if they are system level hooks to enable configuration
 		$settings = array('config_settings', 'config_arrays', 'config_form');
-		$status = (!in_array($hook, $settings) ? 0 : 1);
+		$status   = (!in_array($hook, $settings) ? 0 : 1);
+
+		if ($enable) {
+			$status = 1;
+		}
+
 		db_execute_prepared('INSERT INTO plugin_hooks
-			(name, hook, function, file, status)
+			(name, hook, `function`, file, status)
 			VALUES (?, ?, ?, ?, ?)',
 			array($plugin, $hook, $function, $file, $status));
 	} else {
+		if ($enable == true) {
+			$status = 1;
+		}
+
+		// enable the hook automatically if other hooks are already enabled
+		// for this plugin.
+		if (!$status) {
+			$exists = db_fetch_cell_prepared('SELECT COUNT(*)
+				FROM plugin_hooks
+				WHERE name = ?
+				AND status = 1',
+				array($plugin));
+
+			if ($exists > 0) {
+				$status = 1;
+			}
+		}
+
 		db_execute_prepared("UPDATE plugin_hooks
-			SET function = ?,
-			file = ?
-			WHERE name = ? AND hook = ?",
-			array($function, $file, $plugin, $hook));
+			SET `function` = ?, `status` = ?,
+			`file` = ?
+			WHERE `name` = ?
+			AND `hook` = ?",
+			array($function, $status, $file, $plugin, $hook));
 	}
 }
 
-function api_plugin_remove_hooks ($plugin) {
+function api_plugin_remove_hooks($plugin) {
 	db_execute_prepared('DELETE FROM plugin_hooks
 		WHERE name = ?',
 		array($plugin));
 }
 
-function api_plugin_enable_hooks ($plugin) {
+function api_plugin_enable_hooks($plugin) {
 	db_execute_prepared('UPDATE plugin_hooks
 		SET status = 1
 		WHERE name = ?',
 		array($plugin));
 }
 
-function api_plugin_disable_hooks ($plugin) {
+function api_plugin_disable_hooks($plugin) {
 	db_execute_prepared("UPDATE plugin_hooks
 		SET status = 0
 		WHERE name = ?
@@ -658,21 +784,77 @@ function api_plugin_disable_hooks ($plugin) {
 		array($plugin));
 }
 
-function api_plugin_disable_hooks_all ($plugin) {
+function api_plugin_disable_hooks_all($plugin) {
 	db_execute_prepared("UPDATE plugin_hooks
 		SET status = 0
 		WHERE name = ?",
 		array($plugin));
 }
 
-function api_plugin_register_realm ($plugin, $file, $display, $admin = true) {
-	$exists = db_fetch_cell_prepared('SELECT id
+function api_plugin_register_realm($plugin, $file, $display, $admin = true) {
+	$files = explode(':', $file);
+
+	$i = 0;
+	$sql_where = '(';
+	foreach($files as $tfile) {
+		$sql_where .= ($sql_where != '(' ? ' OR ':'') .
+			' (file = "' . $tfile . '" OR file LIKE "' . $tfile . ',%" OR file LIKE "%,' . $tfile . ',%" OR file LIKE "%,' . $tfile . '")';
+	}
+	$sql_where .= ')';
+
+	$realm_ids = db_fetch_assoc_prepared("SELECT id
 		FROM plugin_realms
 		WHERE plugin = ?
-		AND file = ?',
-		array($plugin, $file));
+		AND $sql_where",
+		array($plugin));
 
-	if ($exists === false) {
+	if (cacti_sizeof($realm_ids) == 1) {
+		$realm_id = $realm_ids[0]['id'];
+	} elseif (cacti_sizeof($realm_ids) > 1) {
+		$realm_id = $realm_ids[0]['id'];
+		cacti_log('WARNING: Registering Realm for Plugin ' . $plugin . ' and Filenames ' . $file . ' is ambiguous.  Using first matching Realm.  Contact the plugin owner to resolve this issue.');
+
+		unset($realm_ids[0]);
+
+		foreach ($realm_ids as $id) {
+			$realm_info = db_fetch_row_prepared('SELECT *
+				FROM plugin_realms
+				WHERE id = ?',
+				array($id['id']));
+
+			if ($file == $realm_info['file']) {
+				db_execute_prepared('UPDATE IGNORE user_auth_realm
+					SET realm_id = ?
+					WHERE realm_id = ?',
+					array($realm_id+100, $realm_info['id']+100));
+
+				db_execute_prepared('UPDATE IGNORE user_auth_group_realm
+					SET realm_id = ?
+					WHERE realm_id = ?',
+					array($realm_id+100, $realm_info['id']+100));
+
+				db_execute_prepared('DELETE FROM plugin_realms
+					WHERE id = ?',
+					array($realm_info['id']));
+			} elseif (strpos($realm_info['file'], $file)) {
+				if (substr($realm_info['file'], 0, strlen($file)) == $file) {
+					$file = substr($file, strlen($file)-1);
+				} else {
+					$file = str_replace(',' . $file, '', $realm_info['file']);
+					$file = str_replace(',,', ',', $file);
+				}
+
+				db_execute_prepared('UPDATE plugin_realms
+					SET file = ?
+					WHERE id = ?',
+					array($file, $realm_info['id']));
+			}
+		}
+	} else {
+		$realm_id = false;
+	}
+
+	if ($realm_id === false) {
 		db_execute_prepared('REPLACE INTO plugin_realms
 			(plugin, file, display)
 			VALUES (?, ?, ?)',
@@ -687,15 +869,12 @@ function api_plugin_register_realm ($plugin, $file, $display, $admin = true) {
 
 			$realm_id = $realm_id + 100;
 
-			$user_ids[] = db_fetch_cell("SELECT id
-				FROM user_auth
-				WHERE username = 'admin'", false);
-
+			$user_ids[] = read_config_option('admin_user');
 			if (isset($_SESSION['sess_user_id'])) {
 				$user_ids[] = $_SESSION['sess_user_id'];
 			}
 
-			if (sizeof($user_ids)) {
+			if (cacti_sizeof($user_ids)) {
 				foreach($user_ids as $user_id) {
 					db_execute_prepared('REPLACE INTO user_auth_realm
 						(user_id, realm_id)
@@ -706,14 +885,14 @@ function api_plugin_register_realm ($plugin, $file, $display, $admin = true) {
 		}
 	} else {
 		db_execute_prepared('UPDATE plugin_realms
-			SET display = ?
-			WHERE plugin = ?
-			AND file = ?',
-			array($display, $plugin, $file));
+			SET display = ?,
+			file = ?
+			WHERE id = ?',
+			array($display, $file, $realm_id));
 	}
 }
 
-function api_plugin_remove_realms ($plugin) {
+function api_plugin_remove_realms($plugin) {
 	$realms = db_fetch_assoc_prepared('SELECT id
 		FROM plugin_realms
 		WHERE plugin = ?',
@@ -735,21 +914,27 @@ function api_plugin_remove_realms ($plugin) {
 			array($plugin));
 }
 
-function api_plugin_load_realms () {
+function api_plugin_load_realms() {
 	global $user_auth_realms, $user_auth_realm_filenames;
-	$plugin_realms = db_fetch_assoc('SELECT * FROM plugin_realms ORDER BY plugin, display', false);
-	if (count($plugin_realms)) {
+
+	$plugin_realms = db_fetch_assoc('SELECT *
+		FROM plugin_realms
+		ORDER BY plugin, display');
+
+	if (cacti_sizeof($plugin_realms)) {
 		foreach ($plugin_realms as $plugin_realm) {
 			$plugin_files = explode(',', $plugin_realm['file']);
+
 			foreach($plugin_files as $plugin_file) {
 				$user_auth_realm_filenames[$plugin_file] = $plugin_realm['id'] + 100;
 			}
+
 			$user_auth_realms[$plugin_realm['id'] + 100] = $plugin_realm['display'];
 		}
 	}
 }
 
-function api_plugin_user_realm_auth ($filename = '') {
+function api_plugin_user_realm_auth($filename = '') {
 	global $user_auth_realm_filenames;
 	/* list all realms that this user has access to */
 
@@ -762,18 +947,18 @@ function api_plugin_user_realm_auth ($filename = '') {
 	return false;
 }
 
-function plugin_config_arrays () {
+function plugin_config_arrays() {
 	global $config, $menu;
 
 	if ($config['poller_id'] == 1 || $config['connection'] == 'online') {
-		$menu[__('Configuration')]['plugins.php'] = __('Plugin Management');
+		$menu[__('Configuration')]['plugins.php'] = __('Plugins');
 	}
 
-	api_plugin_load_realms ();
+	api_plugin_load_realms();
 }
 
-function plugin_draw_navigation_text ($nav) {
-	$nav['plugins.php:'] = array('title' => __('Plugin Management'), 'mapping' => 'index.php:', 'url' => 'plugins.php', 'level' => '1');
+function plugin_draw_navigation_text($nav) {
+	$nav['plugins.php:'] = array('title' => __('Plugins'), 'mapping' => 'index.php:', 'url' => 'plugins.php', 'level' => '1');
 	return $nav;
 }
 
@@ -793,12 +978,55 @@ function plugin_is_compatible($plugin) {
 	return array('compat' => true, 'requires' => __('Requires: Cacti >= %s', $info['compat']));
 }
 
+function plugin_load_info_defaults($file, $info, $defaults = array()) {
+	$result = $info;
+	$dir    = @basename(@dirname($file));
+
+	if (!is_array($defaults)) {
+		$defaults = array();
+	}
+
+	if (!is_array($result)) {
+		$result = array();
+	}
+
+	$info_fields = array(
+		'name'         => $dir,
+		'requires'     => '',
+		'longname'     => ucfirst($dir),
+		'status'       => file_exists($file) ? 0 : -4,
+		'version'      => __('Unknown'),
+		'author'       => __('Unknown'),
+		'homepage'     => isset($info['webpage']) ? $info['webpage'] : __('Not Stated'),
+		'capabilities' => '',
+		'directory'    => $dir,
+	);
+
+	$info_fields = $info_fields + $defaults;
+	foreach ($info_fields as $name => $value) {
+		if (!array_key_exists($name, $result)) {
+			$result[$name] = $value;
+		}
+	}
+
+	if (strstr($dir, ' ') !== false) {
+		$result['status'] = -3;
+	} elseif ($dir != $result['name']) {
+		$result['status'] = -2;
+	} elseif (!isset($result['compat']) || cacti_version_compare(CACTI_VERSION, $result['compat'], '<')) {
+		$result['status'] = -1;
+	}
+
+	return $result;
+}
+
 function plugin_load_info_file($file) {
+	$info = false;
 	if (file_exists($file)) {
 		if (is_readable($file)) {
 			$info = parse_ini_file($file, true);
-			if (sizeof($info)) {
-				return $info['info'];
+			if (cacti_sizeof($info) && array_key_exists('info', $info)) {
+				$info = plugin_load_info_defaults($file, $info['info']);
 			} else {
 				cacti_log('WARNING: Loading plugin INFO file failed.  Parsing INI file failed.', false, 'WEBUI');
 			}
@@ -809,5 +1037,5 @@ function plugin_load_info_file($file) {
 		cacti_log('WARNING: Loading plugin INFO file failed.  INFO file does not exist.', false, 'WEBUI');
 	}
 
-	return false;
+	return $info;
 }
