@@ -1,7 +1,7 @@
 <?php
 /*
  +-------------------------------------------------------------------------+
- | Copyright (C) 2004-2018 The Cacti Group                                 |
+ | Copyright (C) 2004-2020 The Cacti Group                                 |
  |                                                                         |
  | This program is free software; you can redistribute it and/or           |
  | modify it under the terms of the GNU General Public License             |
@@ -74,7 +74,7 @@ if (isset_request_var('mode') && in_array(get_nfilter_request_var('mode'), $mode
 				api_plugin_install($id);
 			}
 
-			if ($_SESSION['sess_plugins_state'] != '-3') {
+			if ($_SESSION['sess_plugins_state'] >= 0) {
 				header('Location: plugins.php?state=5' . ($option != '' ? '&' . $option:''));
 			} else {
 				header('Location: plugins.php' . ($option != '' ? '?' . $option:''));
@@ -137,7 +137,7 @@ update_show_current();
 bottom_footer();
 
 function plugins_temp_table_exists($table) {
-	return sizeof(db_fetch_row("SHOW TABLES LIKE '$table'"));
+	return cacti_sizeof(db_fetch_row("SHOW TABLES LIKE '$table'"));
 }
 
 function plugins_load_temp_table() {
@@ -168,42 +168,18 @@ function plugins_load_temp_table() {
 			ADD COLUMN infoname varchar(20) DEFAULT NULL");
 	}
 
-	$path = $config['base_path'] . '/plugins/';
-	$dh   = opendir($path);
+	$path  = $config['base_path'] . '/plugins/';
+	$dh    = opendir($path);
+	$cinfo = array();
 	if ($dh !== false) {
 		while (($file = readdir($dh)) !== false) {
 			if (is_dir("$path$file") && file_exists("$path$file/setup.php") && !in_array($file, $plugins_integrated)) {
-				if (file_exists("$path$file/INFO")) {
-					$cinfo[$file] = plugin_load_info_file("$path/$file/INFO");
-
-					if (!isset($cinfo[$file]['author']))   $cinfo[$file]['author']   = __('Unknown');
-					if (!isset($cinfo[$file]['homepage'])) $cinfo[$file]['homepage'] = __('Not Stated');
-					if (isset($cinfo[$file]['webpage']))   $cinfo[$file]['homepage'] = $cinfo[$file]['webpage'];
-					if (!isset($cinfo[$file]['longname'])) $cinfo[$file]['longname'] = ucfirst($file);
-					if (!isset($cinfo[$file]['version']))  $cinfo[$file]['version']  = __('Unknown');
-					if (!isset($cinfo[$file]['requires'])) $cinfo[$file]['requires'] = '';
-
-					$cinfo[$file]['status'] = 0;
+				$info_file = "$path$file/INFO";
+				if (file_exists($info_file)) {
+					$cinfo[$file] = plugin_load_info_file($info_file);
 					$pluginslist[] = $file;
-
-					if (strstr($file, ' ') !== false) {
-						$cinfo[$file]['status'] = -3;
-					} elseif ($file != $cinfo[$file]['name']) {
-						$cinfo[$file]['status'] = -2;
-					} elseif (!isset($cinfo[$file]['compat']) || cacti_version_compare(CACTI_VERSION, $cinfo[$file]['compat'], '<')) {
-						$cinfo[$file]['status'] = -1;
-					}
 				} else {
-					$cinfo[$file] = array(
-						'name'     => ucfirst($file),
-						'longname' => ucfirst($file),
-						'author'   => '',
-						'homepage' => '',
-						'version'  => __('Unknown'),
-						'compat'   => '0.8',
-						'requires' => '',
-						'status'   => -4
-					);
+					$cinfo[$file] = plugin_load_info_defaults($info_file, false);
 				}
 
 				$exists = db_fetch_cell_prepared("SELECT COUNT(*)
@@ -238,6 +214,44 @@ function plugins_load_temp_table() {
 		closedir($dh);
 	}
 
+	$found_plugins = array_keys($cinfo);
+	$plugins = db_fetch_assoc('SELECT id, directory, status FROM plugin_config');
+
+	if ($plugins !== false && sizeof($plugins)) {
+		foreach ($plugins as $plugin) {
+			if (!in_array($plugin['directory'], $found_plugins)) {
+				$plugin['status'] = '-5';
+
+				$exists = db_fetch_cell_prepared("SELECT COUNT(*)
+					FROM $table
+					WHERE directory = ?",
+					array($plugin['directory']));
+
+				if (!$exists) {
+					db_execute_prepared("INSERT INTO $table
+						(directory, name, status, author, webpage, version, requires, infoname)
+						VALUES (?, ?, ?, ?, ?, ?, ?, ?)",
+						array(
+							$plugin['directory'],
+							$plugin['longname'],
+							$plugin['status'],
+							$plugin['author'],
+							$plugin['homepage'],
+							$plugin['version'],
+							$plugin['requires'],
+							$plugin['name']
+						)
+					);
+				} else {
+					db_execute_prepared("UPDATE $table
+						SET status = ?
+						WHERE directory = ?",
+						array($plugin['status'], $plugin['directory']));
+				}
+			}
+		}
+	}
+
 	return $table;
 }
 
@@ -256,10 +270,9 @@ function update_show_current () {
 			'default' => '1'
 			),
 		'filter' => array(
-			'filter' => FILTER_CALLBACK,
+			'filter' => FILTER_DEFAULT,
 			'pageset' => true,
-			'default' => '',
-			'options' => array('options' => 'sanitize_search_string')
+			'default' => ''
 			),
 		'sort_column' => array(
 			'filter' => FILTER_CALLBACK,
@@ -274,7 +287,7 @@ function update_show_current () {
 		'state' => array(
 			'filter' => FILTER_VALIDATE_INT,
 			'pageset' => true,
-			'default' => '-3'
+			'default' => '-99'
 			)
 	);
 
@@ -327,14 +340,15 @@ function update_show_current () {
 						<?php print __('Search');?>
 					</td>
 					<td>
-						<input id='filter' type='text' name='filter' size='25' value='<?php print get_request_var('filter');?>'>
+						<input type='text' class='ui-state-default ui-corner-all' id='filter' size='25' value='<?php print html_escape_request_var('filter');?>'>
 					</td>
 					<td>
 						<?php print __('Status');?>
 					</td>
 					<td>
 						<select id='state' name='state' onChange='applyFilter()'>
-							<option value='-3'<?php if (get_request_var('state') == '-3') {?> selected<?php }?>><?php print __('All');?></option>
+							<option value='-99'<?php if (get_request_var('state') == '-99') {?> selected<?php }?>><?php print __('All');?></option>
+							<option value='-98'<?php if (get_request_var('state') == '-98') {?> selected<?php }?>><?php print __('Plugin Error');?></option>
 							<option value='1'<?php if (get_request_var('state') == '1') {?> selected<?php }?>><?php print __('Active');?></option>
 							<option value='4'<?php if (get_request_var('state') == '4') {?> selected<?php }?>><?php print __('Installed');?></option>
 							<option value='5'<?php if (get_request_var('state') == '5') {?> selected<?php }?>><?php print __('Active/Installed');?></option>
@@ -349,7 +363,7 @@ function update_show_current () {
 						<select id='rows' name='rows' onChange='applyFilter()'>
 							<option value='-1'<?php print (get_request_var('rows') == '-1' ? ' selected>':'>') . __('Default');?></option>
 							<?php
-							if (sizeof($item_rows) > 0) {
+							if (cacti_sizeof($item_rows) > 0) {
 								foreach ($item_rows as $key => $value) {
 									print "<option value='" . $key . "'"; if (get_request_var('rows') == $key) { print ' selected'; } print '>' . html_escape($value) . "</option>\n";
 								}
@@ -359,8 +373,8 @@ function update_show_current () {
 					</td>
 					<td>
 						<span>
-							<input type='button' id='refresh' value='<?php print __esc('Go');?>' title='<?php print __esc('Set/Refresh Filters');?>'>
-							<input type='button' id='clear' value='<?php print __esc('Clear');?>' title='<?php print __esc('Clear Filters');?>'>
+							<input type='button' class='ui-button ui-corner-all ui-widget' id='refresh' value='<?php print __esc('Go');?>' title='<?php print __esc('Set/Refresh Filters');?>'>
+							<input type='button' class='ui-button ui-corner-all ui-widget' id='clear' value='<?php print __esc('Clear');?>' title='<?php print __esc('Clear Filters');?>'>
 						</span>
 					</td>
 				</tr>
@@ -376,15 +390,25 @@ function update_show_current () {
 
 	/* form the 'where' clause for our main sql query */
 	if (get_request_var('filter') != '') {
-		$sql_where = "WHERE ($table.name LIKE '%" . get_request_var('filter') . "%')";
+		$sql_where = "WHERE ($table.name LIKE " . db_qstr('%' . get_request_var('filter') . '%') . ')';
 	}
 
-	if (get_request_var('state') > -3) {
-		if (get_request_var('state') == 5) {
+	if (!isset_request_var('state')) {
+		set_request_var('status', -99);
+	}
+
+	switch (get_request_var('state')) {
+		case 5:
 			$sql_where .= ($sql_where != '' ? ' AND ':'WHERE ') . ' status IN(1,4)';
-		} else {
+			break;
+		case -98:
+			$sql_where .= ($sql_where != '' ? ' AND ':'WHERE ') . ' status < 0';
+			break;
+		case -99:
+			break;
+		default:
 			$sql_where .= ($sql_where != '' ? ' AND ':'WHERE ') . ' status=' . get_request_var('state');
-		}
+			break;
 	}
 
 	if (get_request_var('rows') == '-1') {
@@ -401,14 +425,17 @@ function update_show_current () {
 	$sql_order = get_order_string();
 	$sql_limit = ' LIMIT ' . ($rows*(get_request_var('page')-1)) . ',' . $rows;
 
+	$sql_order = str_replace('`version` ', 'INET_ATON(`version`) ', $sql_order);
 	$sql_order = str_replace('version ', 'version+0 ', $sql_order);
 	$sql_order = str_replace('id DESC', 'id ASC', $sql_order);
 
-	$plugins = db_fetch_assoc("SELECT *
+	$sql = "SELECT *
 		FROM $table
 		$sql_where
 		$sql_order
-		$sql_limit");
+		$sql_limit";
+
+	$plugins = db_fetch_assoc($sql);
 
 	$nav = html_nav_bar('plugins.php?filter=' . get_request_var('filter'), MAX_DISPLAY_PAGES, get_request_var('page'), $rows, $total_rows, 8, __('Plugins'), 'page', 'main');
 
@@ -432,7 +459,7 @@ function update_show_current () {
 	html_header_sort($display_text, get_request_var('sort_column'), get_request_var('sort_direction'), 1);
 
 	$i = 0;
-	if (sizeof($plugins)) {
+	if (cacti_sizeof($plugins)) {
 		$j = 0;
 		foreach ($plugins as $plugin) {
 			if ((isset($plugins[$j+1]) && $plugins[$j+1]['status'] < 0) || (!isset($plugins[$j+1]))) {
@@ -455,18 +482,18 @@ function update_show_current () {
 			$j++;
 		}
 	} else {
-		print '<tr><td colspan="' . sizeof($display_text) . '"><em>' . __('No Plugins Found') . '</em></td></tr>';
+		print '<tr><td colspan="' . cacti_sizeof($display_text) . '"><em>' . __('No Plugins Found') . '</em></td></tr>';
 	}
 
 	html_end_box(false);
 
-	if (sizeof($plugins)) {
+	if (cacti_sizeof($plugins)) {
 		print $nav;
 	}
 
 	form_end();
 
-	$uninstall_msg = __('Uninstalling this Plugin will remove all Plugin Data and Settings.  If you really want to Uninstall the Pluign, click \'Uninstall\' below.  Otherwise click \'Cancel\'');
+	$uninstall_msg = __('Uninstalling this Plugin will remove all Plugin Data and Settings.  If you really want to Uninstall the Plugin, click \'Uninstall\' below.  Otherwise click \'Cancel\'');
 	$uninstall_title = __('Are you sure you want to Uninstall?');
 
 	?>
@@ -491,7 +518,7 @@ function update_show_current () {
 					id: 'btnUninstall',
 					click: function() {
 						$('#uninstalldialog').dialog('close');
-						loadPageNoHeader(url + '&header=false');
+						document.location = url;
 					}
 				}
 			};
@@ -518,7 +545,7 @@ function format_plugin_row($plugin, $last_plugin, $include_ordering, $table) {
 
 	$row = plugin_actions($plugin, $table);
 
-	$row .= "<td><a href='" . html_escape($plugin['webpage']) . "' target='_blank'>" . (get_request_var('filter') != '' ? preg_replace('/(' . preg_quote(get_request_var('filter')) . ')/i', "<span class='filteredValue'>\\1</span>", ucfirst($plugin['directory'])) : ucfirst($plugin['directory'])) . '</a>' . (is_dir($config['base_path'] . '/plugins/' . $plugin['directory']) ? '':' (<span class="txtErrorText">' . __('ERROR: Directory Missing') . '</span>)') . '</td>';
+	$row .= "<td><a href='" . html_escape($plugin['webpage']) . "' target='_blank'>" . (get_request_var('filter') != '' ? preg_replace('/(' . preg_quote(get_request_var('filter')) . ')/i', "<span class='filteredValue'>\\1</span>", ucfirst($plugin['directory'])) : ucfirst($plugin['directory'])) . '</a></td>';
 
 	$row .= "<td class='nowrap'>" . filter_value($plugin['name'], get_request_var('filter')) . "</td>\n";
 
@@ -531,8 +558,19 @@ function format_plugin_row($plugin, $last_plugin, $include_ordering, $table) {
 		$row .= "<td class='nowrap'>" . $status_names[$plugin['status']] . "</td>\n";
 	}
 
+	if ($plugin['requires'] != '') {
+		$requires = explode(' ', $plugin['requires']);
+		foreach($requires as $r) {
+			$nr[] = ucfirst($r);
+		}
+
+		$requires = implode(', ', $nr);
+	} else {
+		$requires = $plugin['requires'];
+	}
+
 	$row .= "<td class='nowrap'>" . $plugin['author'] . "</td>\n";
-	$row .= "<td class='nowrap'>" . ucfirst($plugin['requires']) . "</td>\n";
+	$row .= "<td class='nowrap'>" . $requires . "</td>\n";
 	$row .= "<td class='right'>"  . $plugin['version'] . "</td>\n";
 
 	if ($include_ordering) {
@@ -568,7 +606,12 @@ function plugin_required_for_others($plugin, $table) {
 		AND status IN (1,4)");
 
 	if ($required_for_others) {
-		return $required_for_others;
+		$parts = explode(',', $required_for_others);
+		foreach($parts as $p) {
+			$np[] = ucfirst($p);
+		}
+
+		return implode(', ', $np);
 	} else {
 		return false;
 	}
@@ -576,26 +619,7 @@ function plugin_required_for_others($plugin, $table) {
 
 function plugin_required_installed($plugin, $table) {
 	$not_installed = '';
-
-	$requires = db_fetch_cell_prepared("SELECT requires
-		FROM $table
-		WHERE directory = ?",
-		array($plugin['directory']));
-
-	if ($requires != '') {
-		$requires = explode(' ', $requires);
-		foreach($requires as $r) {
-			$installed = db_fetch_cell_prepared("SELECT status
-				FROM $table
-				WHERE directory = ?",
-				array($r));
-
-			if (!$installed) {
-				$not_installed .= ($not_installed != '' ? ', ':'') . $r;
-			}
-		}
-	}
-
+	api_plugin_can_install($plugin['infoname'], $not_installed);
 	return $not_installed;
 }
 
@@ -606,7 +630,7 @@ function plugin_actions($plugin, $table) {
 	switch ($plugin['status']) {
 		case '0': // Not Installed
 			$not_installed = plugin_required_installed($plugin, $table);
-			if ($not_installed != '') {
+		 	if ($not_installed != '') {
 				$link .= "<a href='#' title='" . __esc('Unable to Install Plugin.  The following Plugins must be installed first: %s', ucfirst($not_installed)) . "' class='linkEditMain'><img src='" . $config['url_path'] . "images/cog_error.png'></a>";
 			} else {
 				$link .= "<a href='" . html_escape($config['url_path'] . 'plugins.php?mode=install&id=' . $plugin['directory']) . "' title='" . __esc('Install Plugin') . "' class='linkEditMain'><img src='" . $config['url_path'] . "images/cog_add.png'></a>";
@@ -634,8 +658,11 @@ function plugin_actions($plugin, $table) {
 			}
 			$link .= "<a href='" . html_escape($config['url_path'] . 'plugins.php?mode=enable&id=' . $plugin['directory']) . "' title='" . __esc('Enable Plugin') . "'><img src='" . $config['url_path'] . "images/accept.png'></a>";
 			break;
-		case '-4': // Plugins can have spaces in their names
-			$link .= "<a href='#' title='" . __esc('Plugin does not include an INFO file') . "' class='linkEditMain'><img src='images/cog_error.png'></a>";
+		case '-5': // Plugin directory missing
+			$link .= "<a href='#' title='" . __esc('Plugin directory is missing!') . "' class='linkEditMain'><img src='images/cog_error.png'></a>";
+			break;
+		case '-4': // Plugins should have INFO file since 1.0.0
+			$link .= "<a href='#' title='" . __esc('Plugin is not compatible (Pre-1.x)') . "' class='linkEditMain'><img src='images/cog_error.png'></a>";
 			break;
 		case '-3': // Plugins can have spaces in their names
 			$link .= "<a href='#' title='" . __esc('Plugin directories can not include spaces') . "' class='linkEditMain'><img src='images/cog_error.png'></a>";
