@@ -2,7 +2,7 @@
 <?php
 /*
  +-------------------------------------------------------------------------+
- | Copyright (C) 2004-2018 The Cacti Group                                 |
+ | Copyright (C) 2004-2020 The Cacti Group                                 |
  |                                                                         |
  | This program is free software; you can redistribute it and/or           |
  | modify it under the terms of the GNU General Public License             |
@@ -26,11 +26,31 @@
 /* tick use required as of PHP 4.3.0 to accomodate signal handling */
 declare(ticks = 1);
 
-/* we are not talking to the browser */
-$no_http_headers = true;
-
 ini_set('output_buffering', 'Off');
 
+require(__DIR__ . '/include/cli_check.php');
+require_once($config['base_path'] . '/lib/api_device.php');
+require_once($config['base_path'] . '/lib/api_data_source.php');
+require_once($config['base_path'] . '/lib/api_graph.php');
+require_once($config['base_path'] . '/lib/api_tree.php');
+require_once($config['base_path'] . '/lib/data_query.php');
+require_once($config['base_path'] . '/lib/html_form_template.php');
+require_once($config['base_path'] . '/lib/ping.php');
+require_once($config['base_path'] . '/lib/poller.php');
+require_once($config['base_path'] . '/lib/snmp.php');
+require_once($config['base_path'] . '/lib/sort.php');
+require_once($config['base_path'] . '/lib/template.php');
+require_once($config['base_path'] . '/lib/utility.php');
+
+if ($config['poller_id'] > 1) {
+	if ($config['connection'] == 'online') {
+		db_force_remote_cnn();
+	} else {
+		cacti_log('WARNING: Main Cacti database offline.  Can not run automation', false, 'AUTOM8');
+		exit(1);
+	}
+}
+
 /** sig_handler - provides a generic means to catch exceptions to the Cacti log.
  * @arg $signo  - (int) the signal that was thrown by the interface.
  * @return      - null */
@@ -47,9 +67,10 @@ function sig_handler($signo) {
 				$pids = array_rekey(db_fetch_assoc_prepared("SELECT pid
 					FROM automation_processes
 					WHERE network_id = ?
-					AND task!='tmaster'", array($network_id)), 'pid', 'pid');
+					AND task!='tmaster'",
+					array($network_id)), 'pid', 'pid');
 
-				if (sizeof($pids)) {
+				if (cacti_sizeof($pids)) {
 					foreach($pids as $pid) {
 						posix_kill($pid, SIGTERM);
 					}
@@ -57,14 +78,20 @@ function sig_handler($signo) {
 
 				clearTask($network_id, getmypid());
 
-				db_execute_prepared('DELETE FROM automation_ips WHERE network_id = ?', array($network_id));
+				sleep(5);
+
+				db_execute_prepared('DELETE
+					FROM automation_ips
+					WHERE network_id = ?',
+					array($network_id));
 			} else {
 				$pids = array_rekey(db_fetch_assoc_prepared("SELECT pid
 					FROM automation_processes
 					WHERE poller_id = ?
-					AND task='tmaster'", array($poller_id)), 'pid', 'pid');
+					AND task='tmaster'",
+					array($poller_id)), 'pid', 'pid');
 
-				if (sizeof($pids)) {
+				if (cacti_sizeof($pids)) {
 					foreach($pids as $pid) {
 						posix_kill($pid, SIGTERM);
 					}
@@ -81,11 +108,6 @@ function sig_handler($signo) {
     }
 }
 
-/* let PHP run just as long as it has to */
-if (!isset($_SERVER['argv'][0]) || isset($_SERVER['REQUEST_METHOD'])  || isset($_SERVER['REMOTE_ADDR'])) {
-	die('<br><strong>This script is only meant to run at the command line.</strong>');
-}
-
 /* take time and log performance data */
 $start = microtime(true);
 
@@ -98,24 +120,6 @@ ini_set('max_execution_time', '0');
 $dir = dirname(__FILE__);
 chdir($dir);
 
-include('./include/global.php');
-include_once($config['base_path'] . '/lib/snmp.php');
-include_once($config['base_path'] . '/lib/ping.php');
-include_once($config['base_path'] . '/lib/poller.php');
-include_once($config['base_path'] . '/lib/utility.php');
-include_once($config['base_path'] . '/lib/api_data_source.php');
-include_once($config['base_path'] . '/lib/api_graph.php');
-include_once($config['base_path'] . '/lib/snmp.php');
-include_once($config['base_path'] . '/lib/data_query.php');
-include_once($config['base_path'] . '/lib/api_device.php');
-
-include_once($config['base_path'] . '/lib/sort.php');
-include_once($config['base_path'] . '/lib/html_form_template.php');
-include_once($config['base_path'] . '/lib/template.php');
-
-include_once($config['base_path'] . '/lib/api_tree.php');
-include_once($config['base_path'] . '/lib/api_automation.php');
-
 /* process calling arguments */
 $parms = $_SERVER['argv'];
 array_shift($parms);
@@ -129,7 +133,7 @@ $master     = false;
 
 global $debug, $poller_id, $network_id, $thread, $master;
 
-if (sizeof($parms)) {
+if (cacti_sizeof($parms)) {
 	foreach($parms as $parameter) {
 		if (strpos($parameter, '=')) {
 			list($arg, $value) = explode('=', $parameter);
@@ -197,7 +201,7 @@ if (!$master && $thread == 0) {
 		array($network_id, $poller_id));
 
 	if ($status != 'on' && !$force) {
-		cacti_log("ERROR: The Network ID: $network_id is disabled.  You must use the 'force' option to force it's execution.", true, 'AUTOM8');
+		cacti_log(automation_get_pid() . " WARNING: The Network ID: $network_id is disabled.  You must use the 'force' option to force it's execution.", true, 'AUTOM8');
 		exit;
 	}
 }
@@ -209,7 +213,7 @@ if ($master) {
 		array($poller_id));
 
 	$launched = 0;
-	if (sizeof($networks)) {
+	if (cacti_sizeof($networks)) {
 		foreach($networks as $network) {
 			if (api_automation_is_time_to_start($network['id']) || $force) {
 				automation_debug("Launching Network Master for '" . $network['name'] . "'\n");
@@ -238,7 +242,7 @@ if (!$master && $thread == 0) {
 	);
 
 	automation_debug("Killing any prior running threads\n");
-	if (sizeof($pids)) {
+	if (cacti_sizeof($pids)) {
 		foreach($pids as $pid) {
 			if (isProcessRunning($pid)) {
 				killProcess($pid);
@@ -261,7 +265,9 @@ if (!$master && $thread == 0) {
 
 	registerTask($network_id, getmypid(), $poller_id, 'tmaster');
 
-	cacti_log("Network Discover is now running for Subnet Range '$network_id'", true, 'AUTOM8');
+	cacti_log(automation_get_pid() . " Network Discover is now running for Subnet Range '$network_id'", true, 'AUTOM8');
+
+	$preexisting_devices = getNetworkDevices($network_id);
 
 	automation_primeIPAddressTable($network_id);
 
@@ -312,7 +318,9 @@ if (!$master && $thread == 0) {
 
 		automation_debug("Found $running Threads\n");
 
-		if ($running == 0 && $failcount > 3) {
+		// Are there no more running tasks? Wait up to 15 seconds to
+		// allow processes to start before checking for failures
+		if (($running == 0 && $failcount > 3) || $command == 'cancel') {
 			db_execute_prepared('DELETE FROM automation_ips
 				WHERE network_id = ?',
 				array($network_id));
@@ -332,6 +340,7 @@ if (!$master && $thread == 0) {
 				array($totals['up'], $totals['snmp'], date('Y-m-d H:i:s', $startTime), ($end - $start), $network_id));
 
 			clearAllTasks($network_id);
+			reportNetworkStatus($network_id, $preexisting_devices);
 
 			exit;
 		} else {
@@ -371,12 +380,25 @@ function discoverDevices($network_id, $thread) {
 	$count            = 0;
 
 	while(true) {
+		// Check for cancel
+		$command = db_fetch_cell_prepared('SELECT command
+			FROM automation_processes
+			WHERE network_id = ?
+			AND task = "tmaster"',
+			array($network_id));
+
+		if ($command == 'cancel' || empty($command)) {
+			removeMyProcess(getmypid(), $network_id);
+			killProcess(getmypid());
+			exit;
+		}
+
 		// set and ip to be scanned
 		db_execute_prepared('UPDATE automation_ips
 			SET pid = ?, thread = ?
 			WHERE network_id = ?
-			AND status=0
-			AND pid=0
+			AND status = 0
+			AND pid = 0
 			LIMIT 1',
 			array(getmypid(), $thread, $network_id));
 
@@ -387,8 +409,11 @@ function discoverDevices($network_id, $thread) {
 			AND status=0',
 			array(getmypid(), $thread));
 
-		if (sizeof($device) && isset($device['ip_address'])) {
+		if (cacti_sizeof($device) && isset($device['ip_address'])) {
 			$count++;
+
+			cacti_log(automation_get_pid() . ' NOTE: Found device IP address \'' . $device['ip_address'] .'\' to check',false,'AUTOM8',POLLER_VERBOSITY_MEDIUM);
+
 			if ($dns != '') {
 				$dnsname = automation_get_dns_from_ip($device['ip_address'], $dns, 300);
 				if ($dnsname != $device['ip_address'] && $dnsname != 'timed_out') {
@@ -467,12 +492,12 @@ function discoverDevices($network_id, $thread) {
 				}
 			}
 
-			$exists = db_fetch_row_prepared('SELECT id, snmp_version, status
+			$exists = db_fetch_row_prepared('SELECT id, snmp_version, status, deleted
 				FROM host
 				WHERE hostname IN (?,?)',
 				array($device['ip_address'], $device['hostname']));
 
-			if (!sizeof($exists)) {
+			if (!cacti_sizeof($exists)) {
 				automation_debug(", Status: Not in Cacti");
 
 				if (substr($device['ip_address'], -3) < 255) {
@@ -517,18 +542,27 @@ function discoverDevices($network_id, $thread) {
 					$ping->port    = $network['ping_port'];;
 
 					/* perform the appropriate ping check of the host */
-					$result = $ping->ping(AVAIL_PING, $network['ping_method'], $network['ping_timeout'], 1);
-
-					if (!$result) {
-						automation_debug(" No response");
-						updateDownDevice($network_id, $device['ip_address']);
-					} else {
-						automation_debug(" Responded");
-						$stats['ping']++;
-						addUpDevice($network_id, getmypid());
+					$bypass_ping = false;
+					$result      = false;
+					if ($network['ping_method'] == PING_SNMP) {
+						$bypass_ping = true;
 					}
 
-					if ($result && automation_valid_snmp_device($device)) {
+					if ($bypass_ping == false) {
+						$result = $ping->ping(AVAIL_PING, $network['ping_method'], $network['ping_timeout'], 1);
+
+						if (!$result) {
+							automation_debug(" No response");
+							updateDownDevice($network_id, $device['ip_address']);
+						} else {
+							automation_debug(" Responded");
+							$stats['ping']++;
+							addUpDevice($network_id, getmypid());
+						}
+					}
+
+
+					if (($result || $bypass_ping) && automation_valid_snmp_device($device)) {
 						$snmp_sysName       = trim($device['snmp_sysName']);
 						$snmp_sysName_short = '';
 						if (!is_ipaddress($snmp_sysName)) {
@@ -547,34 +581,46 @@ function discoverDevices($network_id, $thread) {
 							$snmp_sysName_short = $snmp_sysName;
 						}
 
-						$exists = db_fetch_row_prepared('SELECT id, status, snmp_version
+						$exists = db_fetch_row_prepared('SELECT id, status, snmp_version, deleted
 							FROM host
 							WHERE hostname IN (?,?)',
 							array($snmp_sysName_short, $snmp_sysName));
 
-						if (sizeof($exists)) {
-							if ($exists['status'] == 3 || $exists['status'] == 2) {
-								addUpDevice($network_id, getmypid());
+						if (cacti_sizeof($exists)) {
+							if ($exists['deleted'] != 'on') {
+								if ($exists['status'] == 3 || $exists['status'] == 2) {
+									addUpDevice($network_id, getmypid());
 
-								if ($exists['snmp_version'] > 0) {
-									addSNMPDevice($network_id, getmypid());
+									if ($exists['snmp_version'] > 0) {
+										addSNMPDevice($network_id, getmypid());
+									}
+
+									// Rerun data queries if specified
+									rerunDataQueries($exists['id'], $network);
 								}
 
-								// Rerun data queries if specified
-								rerunDataQueries($exists['id'], $network);
+								automation_debug(' Device is in Cacti!');
+							} else {
+								automation_debug(' Device is in Cacti but marked as deleted!');
 							}
 
-							automation_debug(' Device is in Cacti!');
-
 							markIPDone($device['ip_address'], $network_id);
 						} else {
 							$host_id = 0;
 
 							if ($snmp_sysName != '') {
+								$hostname = gethostbyaddr($device['ip_address']);
+								if ($hostname != $device['ip_address']) {
+									if (strpos($hostname, '.')) {
+										$hostname = substr($hostname, 0, strpos($hostname, '.') - 1);
+									}
+								}
+
 								$isCactiSysName = db_fetch_cell_prepared('SELECT COUNT(*)
 									FROM host
-									WHERE snmp_sysName = ?',
-									array($snmp_sysName));
+									WHERE snmp_sysName = ?
+									AND (hostname = ? OR hostname LIKE "' . $hostname . '%")',
+									array($snmp_sysName, $device['ip_address']));
 
 								if ($isCactiSysName) {
 									automation_debug(", Skipping sysName '" . $snmp_sysName . "' already in Cacti!\n");
@@ -582,18 +628,26 @@ function discoverDevices($network_id, $thread) {
 									continue;
 								}
 
-								$isDuplicateSysName = db_fetch_cell_prepared('SELECT COUNT(*)
-									FROM automation_devices
-									WHERE network_id = ?
-									AND sysName != ""
-									AND ip != ?
-									AND sysName = ?',
-									array($device['ip_address'], $network_id, $snmp_sysName));
+								if ($network['same_sysname'] == '') {
+									$isDuplicateSysNameDiscovery = db_fetch_cell_prepared('SELECT COUNT(*)
+										FROM automation_devices
+										WHERE network_id = ?
+										AND sysName != ""
+										AND ip != ?
+										AND sysName = ?',
+										array($network_id, $device['ip_address'], $snmp_sysName));
 
-								if ($isDuplicateSysName) {
-									automation_debug(", Skipping sysName '" . $snmp_sysName . "' already Discovered!\n");
-									markIPDone($device['ip_address'], $network_id);
-									continue;
+									$isDuplicateSysNameCacti = db_fetch_cell_prepared('SELECT COUNT(*)
+										FROM host
+										WHERE snmp_sysName = ?
+										AND hostname != ?',
+										array($snmp_sysName, $device['ip_address']));
+
+									if ($isDuplicateSysNameDiscovery || $isDuplicateSysNameCacti) {
+										automation_debug(", Skipping sysName '" . $snmp_sysName . "' already Discovered!\n");
+										markIPDone($device['ip_address'], $network_id);
+										continue;
+									}
 								}
 
 								$stats['snmp']++;
@@ -604,7 +658,7 @@ function discoverDevices($network_id, $thread) {
 								$fos = automation_find_os($device['snmp_sysDescr'], $device['snmp_sysObjectID'], $device['snmp_sysName']);
 
 								if ($fos != false && $network['add_to_cacti'] == 'on') {
-									automation_debug(", Template: " . $fos['name']);
+									automation_debug(', Template: ' . $fos['name'] . "\n");
 									$device['os']                   = $fos['name'];
 									$device['host_template']        = $fos['host_template'];
 									$device['availability_method']  = $fos['availability_method'];
@@ -659,11 +713,11 @@ function discoverDevices($network_id, $thread) {
 
 									$stats['added']++;
 								} elseif ($fos == false) {
-									automation_debug(", Template: Not found, Not adding to Cacti");
+									automation_debug(", Template: Not found, Not adding to Cacti\n");
 								} else {
 									automation_debug(", Template: " . $fos['name']);
 									$device['os'] = $fos['name'];
-									automation_debug(", Skipped: Add to Cacti disabled");
+									automation_debug(", Skipped: Add to Cacti disabled\n");
 								}
 							}
 
@@ -694,7 +748,7 @@ function discoverDevices($network_id, $thread) {
 
 							markIPDone($device['ip_address'], $network_id);
 						}
-					}else if ($result) {
+					} elseif ($result) {
 						db_execute('REPLACE INTO automation_devices
 							(network_id, hostname, ip, snmp_community, snmp_version, snmp_port, snmp_username, snmp_password, snmp_auth_protocol, snmp_priv_passphrase, snmp_priv_protocol, snmp_context, sysName, sysLocation, sysContact, sysDescr, sysUptime, os, snmp, up, time) VALUES ('
 							. $network_id                              . ', '
@@ -729,18 +783,23 @@ function discoverDevices($network_id, $thread) {
 					markIPDone($device['ip_address'], $network_id);
 				}
 			} else {
-				if ($exists['status'] == 3 || $exists['status'] == 2) {
-					addUpDevice($network_id, getmypid());
+				if ($exists['deleted'] != 'on') {
+					if ($exists['status'] == 3 || $exists['status'] == 2) {
+						addUpDevice($network_id, getmypid());
 
-					if ($exists['snmp_version'] > 0) {
-						addSNMPDevice($network_id, getmypid());
+						if ($exists['snmp_version'] > 0) {
+							addSNMPDevice($network_id, getmypid());
+						}
+
+						// Rerun data queries if specified
+						rerunDataQueries($exists['id'], $network);
 					}
 
-					// Rerun data queries if specified
-					rerunDataQueries($exists['id'], $network);
+					automation_debug(", Status: Already in Cacti\n");
+				} else {
+					automation_debug(", Status: Already in Cacti but marked as deleted\n");
 				}
 
-				automation_debug(", Status: Already in Cacti\n");
 				markIPDone($device['ip_address'], $network_id);
 			}
 		} else {
@@ -749,7 +808,7 @@ function discoverDevices($network_id, $thread) {
 		}
 	}
 
-	cacti_log('Network ' . $network['name'] . " Thread $thread Finished, " . $stats['scanned'] . ' IPs Scanned, ' . $stats['ping'] . ' IPs Responded to Ping, ' . $stats['snmp'] . ' Responded to SNMP, ' . $stats['added'] . ' Device Added, ' . $count_graph .  ' Graphs Added to Cacti', true, 'AUTOM8');
+	cacti_log(automation_get_pid() . ' Network ' . $network['name'] . " Thread $thread Finished, " . $stats['scanned'] . ' IPs Scanned, ' . $stats['ping'] . ' IPs Responded to Ping, ' . $stats['snmp'] . ' Responded to SNMP, ' . $stats['added'] . ' Device Added, ' . $count_graph .  ' Graphs Added to Cacti', true, 'AUTOM8');
 
 	return true;
 }
@@ -789,6 +848,18 @@ function killProcess($pid) {
 	return posix_kill($pid, SIGTERM);
 }
 
+function removeMyProcess($pid, $network_id) {
+	db_execute_prepared('DELETE FROM automation_processes
+		WHERE pid = ?
+		AND network_id = ?',
+		array($pid, $network_id));
+
+	db_execute_prepared('DELETE FROM automation_ips
+		WHERE pid = ?
+		AND network_id = ?',
+		array($pid, $network_id));
+}
+
 function rerunDataQueries($host_id, &$network) {
 	if ($network['rerun_data_queries'] == 'on') {
 		$snmp_queries = db_fetch_assoc_prepared('SELECT snmp_query_id
@@ -796,7 +867,7 @@ function rerunDataQueries($host_id, &$network) {
 			WHERE host_id = ?',
 			array($host_id));
 
-		if (sizeof($snmp_queries)) {
+		if (cacti_sizeof($snmp_queries)) {
 			foreach($snmp_queries as $query) {
 				run_data_query($host_id, $query['snmp_query_id']);
 			}
@@ -806,8 +877,8 @@ function rerunDataQueries($host_id, &$network) {
 
 function registerTask($network_id, $pid, $poller_id, $task = 'collector') {
 	db_execute_prepared("REPLACE INTO automation_processes
-		(pid, poller_id, network_id, task, status, heartbeat)
-		VALUES (?, ?, ?, ?, 'running', NOW())",
+		(pid, poller_id, network_id, task, status, heartbeat, command)
+		VALUES (?, ?, ?, ?, 'running', NOW(), 'start')",
 		array($pid, $poller_id, $network_id, $task));
 }
 
@@ -835,15 +906,207 @@ function addSNMPDevice($network_id, $pid) {
 		array($pid, $network_id));
 }
 
+function reportNetworkStatus($network_id, $old_devices) {
+	$details = db_fetch_row_prepared('SELECT notification_enabled, notification_email,
+		notification_fromname, notification_fromemail
+		FROM automation_networks
+		WHERE id = ?',
+		array($network_id));
+
+	if (cacti_sizeof($details)) {
+		if ($details['notification_enabled'] == 'on') {
+			if ($details['notification_fromname'] == '') {
+				$fromname = read_config_option('automation_fromname');
+				if ($fromname == '') {
+					$fromname = read_config_option('settings_from_name');
+
+					if ($fromname == '') {
+						$fromname = __('Cacti Primary Admin');
+					}
+				}
+			} else {
+				$fromname = $details['notification_fromname'];
+			}
+
+			if ($details['notification_fromemail'] == '') {
+				$fromemail = read_config_option('automation_fromemail');
+				if ($fromemail == '') {
+					$fromemail = read_config_option('settings_from_email');
+
+					if ($fromemail == '') {
+						$fromemail = 'root@cacti.net';
+					}
+				}
+			} else {
+				$fromemail = $details['notification_fromemail'];
+			}
+
+			$from = $fromname . ' <' . $fromemail . '>';
+
+			if ($details['notification_email'] != '') {
+				$email = $details['notification_email'];
+			} else {
+				$email = read_config_option('automation_email');
+
+				if ($email == '') {
+					$admin_user = read_config_option('admin_user');
+
+					if ($admin_user == '') {
+						cacti_log('WARNING: Unable to send Automation Notification Email.  No Primary Admin User Account specified.', false, 'POLLER');
+
+						return false;
+					}
+
+					$details = db_fetch_cell_prepared('SELECT email_address AS notification_email, full_name
+						FROM user_auth
+						WHERE id = ?',
+						array($admin_user));
+
+					if (!cacti_sizeof($details)) {
+						cacti_log('WARNING: Unable to send Automation Notification Email.  The Primary Admin User Account does not exist.', false, 'POLLER');
+						return false;
+					}
+
+					if ($details['notification_email'] == '') {
+						cacti_log('WARNING: Unable to send Automation Notification Email.  The Primary Admin User Account does not have an Email Address.', false, 'POLLER');
+						return false;
+					}
+
+					$email = ($details['full_name'] != '' ? $details['full_name']:__('Cacti Primary Admin')) . ' <' . $details['notification_email'] . '>';
+				}
+			}
+
+			$new_devices = getNetworkDevices($network_id);
+
+			$ids = array();
+			populateDeviceIndex($ids, 0, $old_devices);
+			populateDeviceIndex($ids, 1, $new_devices);
+
+			$table_head_style = 'style="border-bottom: 1px solid black"';
+			$table_head = '<tr>' .
+				"<td $table_head_style><i>Hostname</i></td>" .
+				"<td $table_head_style><i>IP Address</i></td>" .
+				"<td $table_head_style><i>SNMP Name</i></td>" .
+				"<td $table_head_style><i>Has SNMP?</i></td>" .
+				"<td $table_head_style><i>Responding?</i></td>" .
+				'</tr>';
+
+			$table_exist = '';
+			$table_new = '';
+			$count_exist = 0;
+			$count_new = 0;
+
+			$font_up = '<font color="green">up</font>';
+			$font_down = '<font color="red">down</font>';
+
+			foreach ($new_devices as $device) {
+				$id = $device['ip'];
+				$html_line = '<tr><td>' . $device['hostname'] .
+					'</td><td>' . $device['ip'] .
+					'</td><td>' . (empty($device['sysName']) ? '<i><u>None</u></i>' : $device['sysName']) .
+					'</td><td>' . ($device['snmp'] ? $font_up : $font_down) .
+					'</td><td>' . ($device['up'] ? $font_up : $font_down) .
+					'</td></tr>';
+
+				if ($ids[$id]['old'] != '') {
+					$table_exist .= $html_line;
+					$count_exist++;
+				} else {
+					$table_new .= $html_line;
+					$count_new++;
+				}
+			}
+
+			if (strlen($table_exist) > 0) {
+				$table_exist .= '<tr><td colspan="5"</td>&nbsp;</td></tr>';
+			}
+
+			if (strlen($table_new) > 0) {
+				$table_new .= '<tr><td colspan="5"</td>&nbsp;</td></tr>';
+			}
+
+			$v = get_cacti_version();
+			$headers['User-Agent'] = 'Cacti-Automation-v' . $v;
+
+			$status = ($count_new + $count_exist) . ' devices discovered';
+			if ($count_new > 0) {
+				$status .= ', ' . $count_new . ' new!';
+			}
+
+			$network = db_fetch_row_prepared('SELECT id, name, subnet_range, last_started, last_runtime
+				FROM automation_networks
+				WHERE id = ?',
+				array($network_id));
+
+			$subject = 'Discovery of ' . $network['name'] . ' (' . $network['subnet_range'] . ') - ' . $status;
+			$output = '<h1>Discovery of ' . $network['name'] . '</h1><hr><br>' .
+				'<h2>Summary</h2><table>' .
+				'<tr><td>Network:</td><td>' . $network['subnet_range'] . '</td></tr>'.
+				'<tr><td>Started:</td><td>' . $network['last_started'] . '</td></tr>' .
+				'<tr><td>Duration:</td><td>' . intval($network['last_runtime']) . '</td></tr>' .
+				'<tr><td>Existing:</td><td>' . $count_exist . ' devices</td></tr>' .
+				'<tr><td>New:</td><td>' . $count_new . ' devices</td></tr>' .
+				'</table><br><br>';
+
+			if ($count_new > 0 || $count_exist > 0) {
+				$output .= '<table cellspacing="5" cellpadding="5">';
+				if ($count_new > 0) {
+					$output .= '<tr><td colspan="5"><h3>New Devices</h3></td></tr>' . $table_head . $table_new;
+				}
+
+				if ($count_exist > 0) {
+					$output .= '<tr><td colspan="5"><h3>Existing Devices</h3></td></tr>' . $table_head . $table_exist;
+				}
+				$output .= '</table>';
+			}
+
+			$error = mailer(
+				$from,
+				$email,
+				'',
+				'',
+				'',
+				$subject,
+				$output,
+				__('Cacti Automation Report requires an html based Email client'),
+				'',
+				$headers
+			);
+
+			if (strlen($error)) {
+				cacti_log("WARNING: Automation had problems sending to '$email' for $status.  The error was '$error'", false, 'AUTOM8');
+			} else {
+				cacti_log("NOTICE: Email Notification Sent to '$email' for $status.", false, 'AUTOM8');
+			}
+		}
+	}
+}
+
+function populateDeviceIndex(&$ids, $is_new, $devices) {
+	$field = ($is_new ? 'new' : 'old');
+
+	foreach ($devices as $index => $device) {
+		$id = $device['ip'];
+
+		if (!isset($ids[$id])) {
+			$ids[$id] = array('old' => '', 'new' => '');
+		}
+
+		$ids[$id][$field] = $id;
+	}
+}
+
 function clearTask($network_id, $pid) {
-	db_execute_prepared('DELETE FROM automation_processes
+	db_execute_prepared('DELETE
+		FROM automation_processes
 		WHERE pid = ?
 		AND network_id = ?',
 		array($pid, $network_id));
 
-	db_execute_prepared('DELETE FROM automation_ips
+	db_execute_prepared('DELETE
+		FROM automation_ips
 		WHERE network_id = ?',
-		array($pid, $network_id));
+		array($network_id));
 }
 
 function clearAllTasks($network_id) {
@@ -868,6 +1131,14 @@ function markIPDone($ip_address, $network_id) {
 		array($ip_address, $network_id));
 }
 
+function getNetworkDevices($network_id) {
+	return db_fetch_assoc_prepared('SELECT id, hostname, ip, sysName, snmp, up, time
+		FROM automation_devices
+		WHERE network_id = ?
+		ORDER BY hostname',
+		array($network_id));
+}
+
 function updateDownDevice($network_id, $ip) {
 	$exists = db_fetch_cell_prepared('SELECT COUNT(*)
 		FROM automation_devices
