@@ -1,7 +1,7 @@
 <?php
 /*
  +-------------------------------------------------------------------------+
- | Copyright (C) 2004-2018 The Cacti Group                                 |
+ | Copyright (C) 2004-2020 The Cacti Group                                 |
  |                                                                         |
  | This program is free software; you can redistribute it and/or           |
  | modify it under the terms of the GNU General Public License             |
@@ -23,24 +23,32 @@
 */
 
 include('./include/auth.php');
+include_once('./lib/poller.php');
 
 /* set default action */
 set_default_action();
 
 get_filter_request_var('tab', FILTER_CALLBACK, array('options' => 'sanitize_search_string'));
 
+global $disable_log_rotation, $local_db_cnn_id;
+
 switch (get_request_var('action')) {
 case 'save':
-	foreach ($settings{get_request_var('tab')} as $field_name => $field_array) {
+	$errors = array();
+	$inserts = array();
+
+	foreach ($settings[get_request_var('tab')] as $field_name => $field_array) {
 		if (($field_array['method'] == 'header') || ($field_array['method'] == 'spacer' )){
 			/* do nothing */
 		} elseif ($field_array['method'] == 'checkbox') {
 			if (isset_request_var($field_name)) {
+				$inserts[] = '(' . db_qstr($field_name) . ', "on")';
 				db_execute_prepared("REPLACE INTO settings
 					(name, value)
 					VALUES (?, 'on')",
 					array($field_name));
 			} else {
+				$inserts[] = '(' . db_qstr($field_name) . ', "")';
 				db_execute_prepared("REPLACE INTO settings
 					(name, value)
 					VALUES (?, '')",
@@ -49,11 +57,13 @@ case 'save':
 		} elseif ($field_array['method'] == 'checkbox_group') {
 			foreach ($field_array['items'] as $sub_field_name => $sub_field_array) {
 				if (isset_request_var($sub_field_name)) {
+					$inserts[] = '(' . db_qstr($field_name) . ', "on")';
 					db_execute_prepared("REPLACE INTO settings
 					(name, value)
 					VALUES (?, 'on')",
 					array($sub_field_name));
 				} else {
+					$inserts[] = '(' . db_qstr($field_name) . ', "on")';
 					db_execute_prepared("REPLACE INTO settings
 					(name, value)
 					VALUES (?, '')",
@@ -62,16 +72,31 @@ case 'save':
 			}
 		} elseif ($field_array['method'] == 'dirpath') {
 			if (get_nfilter_request_var($field_name) != '' && !is_dir(get_nfilter_request_var($field_name))) {
-				raise_message(8);
+				$_SESSION['sess_error_fields'][$field_name] = $field_name;
+				$_SESSION['sess_field_values'][$field_name] = get_nfilter_request_var($field_name);
+				$errors[8] = 8;
 			} else {
-				db_execute_prepared('REPLACE INTO settings
-					(name, value)
-					VALUES (?, ?)',
-					array($field_name, get_nfilter_request_var($field_name)));
+				if (get_request_var('tab') == 'path' && is_remote_path_setting($field_name)) {
+					db_execute_prepared('REPLACE INTO settings
+						(name, value)
+						VALUES (?, ?)',
+						array($field_name, get_nfilter_request_var($field_name)), true, $local_db_cnn_id);
+				} else {
+					$inserts[] = '(' . db_qstr($field_name) . ', ' . db_qstr(get_nfilter_request_var($field_name)) . ')';
+					db_execute_prepared('REPLACE INTO settings
+						(name, value)
+						VALUES (?, ?)',
+						array($field_name, get_nfilter_request_var($field_name)));
+				}
 			}
 		} elseif ($field_array['method'] == 'filepath') {
-			if (get_nfilter_request_var($field_name) != '' && !is_file(get_nfilter_request_var($field_name))) {
-				raise_message(8);
+			if (isset($field_array['file_type']) &&
+				$field_array['file_type'] == 'binary' &&
+				get_nfilter_request_var($field_name) != '' &&
+				file_exists(get_nfilter_request_var($field_name)) === false) {
+				$_SESSION['sess_error_fields'][$field_name] = $field_name;
+				$_SESSION['sess_field_values'][$field_name] = get_nfilter_request_var($field_name);
+				$errors[36] = 36;
 			} else {
 				$continue = true;
 
@@ -79,23 +104,40 @@ case 'save':
 					$extension = pathinfo(get_nfilter_request_var($field_name), PATHINFO_EXTENSION);
 
 					if ($extension != 'log') {
-						raise_message(9);
+						$_SESSION['sess_error_fields'][$field_name] = $field_name;
+						$_SESSION['sess_field_values'][$field_name] = get_nfilter_request_var($field_name);
+						$errors[9] = 9;
 						$continue = false;
 					}
+				} elseif (get_nfilter_request_var($field_name) != '' && !is_valid_pathname(get_nfilter_request_var($field_name))) {
+					$_SESSION['sess_error_fields'][$field_name] = $field_name;
+					$_SESSION['sess_field_values'][$field_name] = get_nfilter_request_var($field_name);
+					$errors[36] = 36;
 				}
 
 				if ($continue) {
-					db_execute_prepared('REPLACE INTO settings
-						(name, value)
-						VALUES (?, ?)',
-						array($field_name, get_nfilter_request_var($field_name)));
+					if (get_request_var('tab') == 'path' && is_remote_path_setting($field_name)) {
+						db_execute_prepared('REPLACE INTO settings
+							(name, value)
+							VALUES (?, ?)',
+							array($field_name, get_nfilter_request_var($field_name)), true, $local_db_cnn_id);
+					} else {
+						$inserts[] = '(' . db_qstr($field_name) . ', ' . db_qstr(get_nfilter_request_var($field_name)) . ')';
+						db_execute_prepared('REPLACE INTO settings
+							(name, value)
+							VALUES (?, ?)',
+							array($field_name, get_nfilter_request_var($field_name)));
+					}
 				}
 			}
 		} elseif ($field_array['method'] == 'textbox_password') {
-			if (get_nfilter_request_var($field_name) != get_nfilter_request_var($field_name.'_confirm')) {
-				raise_message(4);
+			if (get_nfilter_request_var($field_name) != get_nfilter_request_var($field_name . '_confirm')) {
+				$_SESSION['sess_error_fields'][$field_name] = $field_name;
+				$_SESSION['sess_field_values'][$field_name] = get_nfilter_request_var($field_name);
+				$errors[4] = 4;
 				break;
 			} elseif (!isempty_request_var($field_name)) {
+				$inserts[] = '(' . db_qstr($field_name) . ', ' . db_qstr(get_nfilter_request_var($field_name)) . ')';
 				db_execute_prepared('REPLACE INTO settings
 					(name, value)
 					VALUES (?, ?)',
@@ -104,6 +146,7 @@ case 'save':
 		} elseif ((isset($field_array['items'])) && (is_array($field_array['items']))) {
 			foreach ($field_array['items'] as $sub_field_name => $sub_field_array) {
 				if (isset_request_var($sub_field_name)) {
+					$inserts[] = '(' . db_qstr($field_name) . ', ' . db_qstr(get_nfilter_request_var($sub_field_name)) . ')';
 					db_execute_prepared('REPLACE INTO settings
 					(name, value)
 					VALUES (?, ?)',
@@ -113,17 +156,20 @@ case 'save':
 		} elseif ($field_array['method'] == 'drop_multi') {
 			if (isset_request_var($field_name)) {
 				if (is_array(get_nfilter_request_var($field_name))) {
+					$inserts[] = '(' . db_qstr($field_name) . ', ' . db_qstr(implode(',', get_nfilter_request_var($field_name))) . ')';
 					db_execute_prepared('REPLACE INTO settings
 					(name, value)
 					VALUES (?, ?)',
 					array($field_name, implode(',', get_nfilter_request_var($field_name))));
 				} else {
+					$inserts[] = '(' . db_qstr($field_name) . ', ' . db_qstr(get_nfilter_request_var($field_name)) . ')';
 					db_execute_prepared('REPLACE INTO settings
 					(name, value)
 					VALUES (?, ?)',
 					array($field_name, get_nfilter_request_var($field_name)));
 				}
 			} else {
+				$inserts[] = '(' . db_qstr($field_name) . ', "")';
 				db_execute_prepared('REPLACE INTO settings
 					(name, value)
 					VALUES (?, "")',
@@ -131,11 +177,13 @@ case 'save':
 			}
 		} elseif (isset_request_var($field_name)) {
 			if (is_array(get_nfilter_request_var($field_name))) {
+				$inserts[] = '(' . db_qstr($field_name) . ', ' . db_qstr(implode(',', get_nfilter_request_var($field_name))) . ')';
 				db_execute_prepared('REPLACE INTO settings
 					(name, value)
 					VALUES (?, ?)',
 					array($field_name, implode(',', get_nfilter_request_var($field_name))));
 			} else {
+				$inserts[] = '(' . db_qstr($field_name) . ', ' . db_qstr(get_nfilter_request_var($field_name)) . ')';
 				db_execute_prepared('REPLACE INTO settings
 					(name, value)
 					VALUES (?, ?)',
@@ -146,23 +194,77 @@ case 'save':
 
 	if (isset_request_var('log_verbosity')) {
 		if (!isset_request_var('selective_debug')) {
+			$inserts[] = '("selective_debug", "")';
 			db_execute('REPLACE INTO settings
 				(name, value)
 				VALUES ("selective_debug", "")');
 		}
 
 		if (!isset_request_var('selective_plugin_debug')) {
+			$inserts[] = '("selective_debug_plugin", "")';
 			db_execute('REPLACE INTO settings
 				(name, value)
 				VALUES ("selective_plugin_debug", "")');
 		}
 	}
 
-	/* update snmpcache */
+	// Disable template user from being able to login
+	if (isset_request_var('user_template')) {
+		db_execute_prepared('UPDATE user_auth
+			SET enabled=""
+			WHERE id = ?',
+			array(get_nfilter_request_var('user_template')));
+	}
+
+	// Update snmpcache
 	snmpagent_global_settings_update();
 
 	api_plugin_hook_function('global_settings_update');
-	raise_message(1);
+
+	$gone_time = read_config_option('poller_interval') * 2;
+
+	$pollers = array_rekey(
+		db_fetch_assoc_prepared('SELECT id
+			FROM poller
+			WHERE id > 1
+			AND disabled=""
+			AND UNIX_TIMESTAMP() - UNIX_TIMESTAMP(last_status) <= ?',
+			array($gone_time)),
+		'id', 'id'
+	);
+
+	if (get_request_var('tab') == 'path' && $config['poller_id'] > 1) {
+		raise_message('poller_paths');
+	}
+
+	if (cacti_sizeof($errors) == 0) {
+		if (cacti_sizeof($pollers) && $config['poller_id'] == 1) {
+			$sql = 'INSERT INTO settings
+				(name, value)
+				VALUES ' . implode(', ', $inserts) . '
+				ON DUPLICATE KEY UPDATE value=VALUES(value)';
+
+			foreach($pollers as $p) {
+				$rcnn_id = poller_connect_to_remote($p);
+
+				if ($rcnn_id) {
+					if (db_execute($sql, false, $rcnn_id) === false) {
+						raise_message('poller_' . $p, __('Settings save to Data Collector %d Failed.', $p), MESSAGE_LEVEL_ERROR);
+					}
+				}
+			}
+
+			raise_message(42);
+		} else {
+			raise_message(1);
+		}
+	} else {
+		raise_message(35);
+
+		foreach($errors as $error) {
+			raise_message($error);
+		}
+	}
 
 	/* reset local settings cache so the user sees the new settings */
 	kill_session_var('sess_config_array');
@@ -192,17 +294,44 @@ default:
 	} else {
 		$current_tab = get_request_var('tab');
 	}
+
+	// If the tab no longer exists, use the first
+	if (!isset($tabs[$current_tab])) {
+		$current_tab = array_keys($tabs);
+		$current_tab = $current_tab[0];
+	}
+
 	$_SESSION['sess_settings_tab'] = $current_tab;
 
+	$data_collectors = db_fetch_cell('SELECT COUNT(*) FROM poller WHERE disabled=""');
+
+	if ($data_collectors > 1) {
+		set_config_option('boost_rrd_update_enable', 'on');
+		set_config_option('boost_redirect', 'on');
+	}
+
+	$system_tabs = array(
+		'general',
+		'path',
+		'snmp',
+		'poller',
+		'data',
+		'visual',
+		'authentication',
+		'boost',
+		'spikes',
+		'mail'
+	);
+
 	/* draw the categories tabs on the top of the page */
 	print "<div>\n";
 	print "<div class='tabs' style='float:left;'><nav><ul role='tablist'>\n";
 
-	if (sizeof($tabs) > 0) {
+	if (cacti_sizeof($tabs) > 0) {
 		$i = 0;
 
 		foreach (array_keys($tabs) as $tab_short_name) {
-			print "<li class='subTab'><a " . (($tab_short_name == $current_tab) ? "class='selected'" : "class=''") . " href='" . html_escape("settings.php?tab=$tab_short_name") . "'>" . $tabs[$tab_short_name] . "</a></li>\n";
+			print "<li class='subTab" . (!in_array($tab_short_name, $system_tabs) ? ' pluginTab':'') . "'><a " . (($tab_short_name == $current_tab) ? "class='selected'" : "class=''") . " href='" . html_escape("settings.php?tab=$tab_short_name") . "'>" . $tabs[$tab_short_name] . "</a></li>\n";
 
 			$i++;
 		}
@@ -211,12 +340,30 @@ default:
 	print "</ul></nav></div>\n";
 	print "</div>\n";
 
-	form_start('settings.php', 'chk');
+	form_start('settings.php', 'form_settings');
 
-	html_start_box( __('Cacti Settings (%s)', $tabs[$current_tab]), '100%', true, '3', 'center', '');
+	if ($config['poller_id'] > 1 && $current_tab == 'path') {
+		$suffix = ' [<span class="deviceDown">' . __('NOTE: Path Settings on this Tab are only saved locally!') . '</span>]';
+	} else {
+		$suffix = '';
+	}
+
+	html_start_box( __('Cacti Settings (%s)%s', $tabs[$current_tab], $suffix), '100%', true, '3', 'center', '');
 
 	$form_array = array();
 
+	// Remove log rotation is disabled by package maintainer
+	if (isset($disable_log_rotation) && $disable_log_rotation == true) {
+		unset($settings['path']['logrotate_enabled']);
+		unset($settings['path']['logrotate_frequency']);
+		unset($settings['path']['logrotate_retain']);
+	}
+
+	// RRDtool is not required for remote data collectors
+	if ($config['poller_id'] > 1) {
+		$settings['path']['path_rrdtool']['method'] = 'other';
+	}
+
 	if (isset($settings[$current_tab])) {
 		foreach ($settings[$current_tab] as $field_name => $field_array) {
 			$form_array += array($field_name => $field_array);
@@ -227,20 +374,34 @@ default:
 						$form_array[$field_name]['items'][$sub_field_name]['form_id'] = 1;
 					}
 
-					$form_array[$field_name]['items'][$sub_field_name]['value'] = db_fetch_cell_prepared('SELECT value
-						FROM settings
-						WHERE name = ?',
-						array($sub_field_name));
+					if ($current_tab == 'path' && is_remote_path_setting($field_name)) {
+						$form_array[$field_name]['items'][$sub_field_name]['value'] = db_fetch_cell_prepared('SELECT value
+							FROM settings
+							WHERE name = ?',
+							array($sub_field_name), '', true, $local_db_cnn_id);
+					} else {
+						$form_array[$field_name]['items'][$sub_field_name]['value'] = db_fetch_cell_prepared('SELECT value
+							FROM settings
+							WHERE name = ?',
+							array($sub_field_name));
+					}
 				}
 			} else {
 				if (config_value_exists($field_name)) {
 					$form_array[$field_name]['form_id'] = 1;
 				}
 
-				$form_array[$field_name]['value'] = db_fetch_cell_prepared('SELECT value
-					FROM settings
-					WHERE name = ?',
-					array($field_name));
+				if ($current_tab == 'path' && is_remote_path_setting($field_name)) {
+					$form_array[$field_name]['value'] = db_fetch_cell_prepared('SELECT value
+						FROM settings
+						WHERE name = ?',
+						array($field_name), '', true, $local_db_cnn_id);
+				} else {
+					$form_array[$field_name]['value'] = db_fetch_cell_prepared('SELECT value
+						FROM settings
+						WHERE name = ?',
+						array($field_name));
+				}
 			}
 		}
 	}
@@ -263,101 +424,17 @@ default:
 
 	var themeChanged = false;
 	var currentTheme = '';
+	var rrdArchivePath = '';
+	var smtpPath = '';
+	var currentTab = '<?php print $current_tab;?>';
+	var dataCollectors = '<?php print $data_collectors;?>';
 
 	$(function() {
-		$('#selective_plugin_debug').multiselect({
-			height: 300,
-			noneSelectedText: '<?php print __('Select Plugin(s)');?>',
-			selectedText: function(numChecked, numTotal, checkedItems) {
-				myReturn = numChecked + ' <?php print __('Plugins Selected');?>';
-				return myReturn;
-			},
-			checkAllText: '<?php print __('All');?>',
-			uncheckAllText: '<?php print __('None');?>',
-			uncheckall: function() {
-				$(this).multiselect('widget').find(':checkbox:first').each(function() {
-					$(this).prop('checked', true);
-				});
-			}
-		}).multiselectfilter( {
-			label: '<?php print __('Search');?>',
-			placeholder: '<?php print __('Enter keyword');?>',
-			width: '150'
-		});
-
-		$('#selective_debug').multiselect({
-			noneSelectedText: '<?php print __('Select File(s)');?>',
-			selectedText: function(numChecked, numTotal, checkedItems) {
-				myReturn = numChecked + ' <?php print __('Files Selected');?>';
-				return myReturn;
-			},
-			checkAllText: '<?php print __('All');?>',
-			uncheckAllText: '<?php print __('None');?>',
-			uncheckall: function() {
-				$(this).multiselect('widget').find(':checkbox:first').each(function() {
-					$(this).prop('checked', true);
-				});
-			}
-		}).multiselectfilter( {
-			label: '<?php print __('Search');?>',
-			placeholder: '<?php print __('Enter keyword');?>',
-			width: '150'
-		});
-
-		$('#spikekill_templates').multiselect({
-			height: 300,
-			noneSelectedText: '<?php print __('Select Template(s)');?>',
-			selectedText: function(numChecked, numTotal, checkedItems) {
-				myReturn = numChecked + ' <?php print __('Templates Selected');?>';
-				$.each(checkedItems, function(index, value) {
-					if (value.value == '0') {
-						myReturn='<?php print __('All Templates Selected');?>';
-						return false;
-					}
-				});
-				return myReturn;
-			},
-			checkAllText: '<?php print __('All');?>',
-			uncheckAllText: '<?php print __('None');?>',
-			uncheckall: function() {
-				$(this).multiselect('widget').find(':checkbox:first').each(function() {
-					$(this).prop('checked', true);
-				});
-			},
-			click: function(event, ui) {
-				checked=$(this).multiselect('widget').find('input:checked').length;
-
-				if (ui.value == '0') {
-					if (ui.checked == true) {
-						$('#host').multiselect('uncheckAll');
-						$(this).multiselect('widget').find(':checkbox:first').each(function() {
-							$(this).prop('checked', true);
-						});
-					}
-				}else if (checked == 0) {
-					$(this).multiselect('widget').find(':checkbox:first').each(function() {
-						$(this).click();
-					});
-				}else if ($(this).multiselect('widget').find('input:checked:first').val() == '0') {
-					if (checked > 0) {
-						$(this).multiselect('widget').find(':checkbox:first').each(function() {
-							$(this).click();
-							$(this).prop('disable', true);
-						});
-					}
-				}
-			}
-		}).multiselectfilter( {
-			label: '<?php print __('Search');?>',
-			placeholder: '<?php print __('Enter keyword');?>',
-			width: '150'
-		});
-
 		$('.subTab').find('a').click(function(event) {
 			event.preventDefault();
 			strURL = $(this).attr('href');
 			strURL += (strURL.indexOf('?') > 0 ? '&':'?') + 'header=false';
-			loadPageNoHeader(strURL);
+			loadPageNoHeader(strURL, true, false);
 		});
 
 		$('input[value="<?php print __esc('Save');?>"]').click(function(event) {
@@ -382,39 +459,156 @@ default:
 			}
 		});
 
-		if ($('#row_settings_email_header')) {
-			$('#emailtest').click(function() {
-				var $div = $('<div />').appendTo('body');
-				$div.attr('id', 'testmail');
-				$('#testmail').prop('title', '<?php print __('Test Email Results');?>');
-				$('#testmail').dialog({
-					autoOpen: false,
-					modal: true,
-					minHeight: 300,
-					maxHeight: 600,
-					height: 450,
-					width: 500,
-					show: {
-						effect: 'appear',
-						duration: 100
-					},
-					hide: {
-						effect: 'appear',
-						duratin: 100
+		if (currentTab == 'general') {
+			$('#selective_plugin_debug').multiselect({
+				height: 300,
+				noneSelectedText: '<?php print __('Select Plugin(s)');?>',
+				selectedText: function(numChecked, numTotal, checkedItems) {
+					myReturn = numChecked + ' <?php print __('Plugins Selected');?>';
+					return myReturn;
+				},
+				checkAllText: '<?php print __('All');?>',
+				uncheckAllText: '<?php print __('None');?>',
+				uncheckall: function() {
+					$(this).multiselect('widget').find(':checkbox:first').each(function() {
+						$(this).prop('checked', true);
+					});
+				}
+			}).multiselectfilter( {
+				label: '<?php print __('Search');?>',
+				placeholder: '<?php print __('Enter keyword');?>',
+				width: '150'
+			});
+
+			$('#selective_debug').multiselect({
+				noneSelectedText: '<?php print __('Select File(s)');?>',
+				selectedText: function(numChecked, numTotal, checkedItems) {
+					myReturn = numChecked + ' <?php print __('Files Selected');?>';
+					return myReturn;
+				},
+				checkAllText: '<?php print __('All');?>',
+				uncheckAllText: '<?php print __('None');?>',
+				uncheckall: function() {
+					$(this).multiselect('widget').find(':checkbox:first').each(function() {
+						$(this).prop('checked', true);
+					});
+				}
+			}).multiselectfilter( {
+				label: '<?php print __('Search');?>',
+				placeholder: '<?php print __('Enter keyword');?>',
+				width: '150'
+			});
+		} else if (currentTab == 'spikes') {
+			$('#spikekill_templates').multiselect({
+				height: 300,
+				noneSelectedText: '<?php print __('Select Template(s)');?>',
+				selectedText: function(numChecked, numTotal, checkedItems) {
+					myReturn = numChecked + ' <?php print __('Templates Selected');?>';
+					$.each(checkedItems, function(index, value) {
+						if (value.value == '0') {
+							myReturn='<?php print __('All Templates Selected');?>';
+							return false;
+						}
+					});
+					return myReturn;
+				},
+				checkAllText: '<?php print __('All');?>',
+				uncheckAllText: '<?php print __('None');?>',
+				uncheckall: function() {
+					$(this).multiselect('widget').find(':checkbox:first').each(function() {
+						$(this).prop('checked', true);
+					});
+				},
+				click: function(event, ui) {
+					checked=$(this).multiselect('widget').find('input:checked').length;
+
+					if (ui.value == '0') {
+						if (ui.checked == true) {
+							$('#host').multiselect('uncheckAll');
+							$(this).multiselect('widget').find(':checkbox:first').each(function() {
+								$(this).prop('checked', true);
+							});
+						}
+					}else if (checked == 0) {
+						$(this).multiselect('widget').find(':checkbox:first').each(function() {
+							$(this).click();
+						});
+					}else if ($(this).multiselect('widget').find('input:checked:first').val() == '0') {
+						if (checked > 0) {
+							$(this).multiselect('widget').find(':checkbox:first').each(function() {
+								$(this).click();
+								$(this).prop('disable', true);
+							});
+						}
 					}
-				});
+				}
+			}).multiselectfilter( {
+				label: '<?php print __('Search');?>',
+				placeholder: '<?php print __('Enter keyword');?>',
+				width: '150'
+			});
+		} else if (currentTab == 'data') {
+			$('#storage_location').change(function() {
+				if ($(this).val() == '0') {
+					$('#row_rrdp_header').hide();
+					$('#row_rrdp_server').hide();
+					$('#row_rrdp_port').hide();
+					$('#row_rrdp_fingerprint').hide();
+					$('#row_rrdp_header2').hide();
+					$('#row_rrdp_load_balancing').hide();
+					$('#row_rrdp_server_backup').hide();
+					$('#row_rrdp_port_backup').hide();
+					$('#row_rrdp_fingerprint_backup').hide();
+				} else {
+					$('#row_rrdp_header').show();
+					$('#row_rrdp_server').show();
+					$('#row_rrdp_port').show();
+					$('#row_rrdp_fingerprint').show();
+					$('#row_rrdp_header2').show();
+					$('#row_rrdp_load_balancing').show();
+					$('#row_rrdp_server_backup').show();
+					$('#row_rrdp_port_backup').show();
+					$('#row_rrdp_fingerprint_backup').show();
+				}
+			}).trigger('change');
+		} else if (currentTab == 'mail') {
+			$('#row_settings_email_header div.formHeaderText').append('<div id="emailtest" class="emailtest"><?php print __('Send a Test Email');?></div>');
+
+			initMail();
+
+			$('#settings_how').change(function() {
+				initMail();
+			});
+
+			$('#emailtest').click(function() {
 				$.get('settings.php?action=send_test')
 					.done(function(data) {
+						$('body').append('<div id="testmail" title="<?php print __esc('Test Email Results');?>"></div>');
 						$('#testmail').html(data);
-						$('#testmail').dialog('open');
+
+						$('#testmail').dialog({
+							autoOpen: false,
+							modal: true,
+							minHeight: 300,
+							maxHeight: 600,
+							height: 450,
+							width: 500,
+							autoOpen: true,
+							show: {
+								effect: 'appear',
+								duration: 100
+							},
+							hide: {
+								effect: 'appear',
+								duratin: 100
+							}
+						});
 					})
 					.fail(function(data) {
 						getPresentHTTPError(data);
 					});
 			});
-		}
-
-		if ($('#row_font_method')) {
+		} else if (currentTab == 'visual') {
 			currentTheme = $('#selected_theme').val();
 
 			initFonts();
@@ -431,48 +625,21 @@ default:
 			$('#realtime_enabled').change(function() {
 				initRealtime();
 			});
-		}
-
-		if ($('#row_snmp_version')) {
+		} else if (currentTab == 'snmp') {
 			// Need to set this for global snmpv3 functions to remain sane between edits
 			snmp_security_initialized = false;
 
 			setSNMP();
+
 			$('#snmp_version, #snmp_auth_protocol, #snmp_priv_protocol, #snmp_security_level').change(function() {
 				setSNMP();
 			});
-		}
 
-		if ($('#row_availability_method')) {
 			initAvail();
 			$('#availability_method').change(function() {
 				initAvail();
 			});
-		}
-
-		if ($('#row_export_type')) {
-			initFTPExport();
-			initPresentation();
-			initTiming();
-
-			$('#export_type').change(function() {
-				initFTPExport();
-			});
-
-			$('#export_presentation').change(function() {
-				initPresentation();
-			});
-
-			$('#export_timing').change(function() {
-				initTiming();
-			});
-
-			$('#export_type').change(function() {
-				initFTPExport();
-			});
-		}
-
-		if ($('#row_auth_method')) {
+		} else if (currentTab == 'authentication') {
 			initAuth();
 			initSearch();
 			initGroupMember();
@@ -488,21 +655,28 @@ default:
 			$('#ldap_group_require').change(function() {
 				initGroupMember();
 			});
-		}
-
-		if ($('#rrd_autoclean')) {
+		} else if (currentTab == 'path') {
 			initRRDClean();
 
 			$('#rrd_autoclean').change(function() {
 				initRRDClean();
 			});
 
+			if (cactiServerOS == 'win32') {
+				$('#row_path_stderrlog').hide();
+			}
+
 			$('#rrd_autoclean_method').change(function() {
 				initRRDClean();
 			});
-		}
+		} else if (currentTab == 'boost') {
+			if (dataCollectors > 1) {
+				$('#boost_rrd_update_enable').prop('checked', true);
+				$('#boost_rrd_update_enable').prop('disabled', true);
+				$('#boost_redirect').prop('checked', true);
+				$('#boost_redirect').prop('disabled', true);
+			}
 
-		if ($('#boost_rrd_update_enable')) {
 			initBoostOD();
 			initBoostCache();
 
@@ -515,21 +689,18 @@ default:
 			});
 		}
 
-		if ($('#settings_test_email')) {
-			initMail();
-
-			$('#settings_how').change(function() {
-				initMail();
-			});
-		}
-
 		function initMail() {
 			/* clear passwords */
+			if ($('#settings_sendmail_path').val() != '') {
+				smtpPath = $('#settings_sendmail_path').val();
+			}
+
 			$('#settings_smtp_password').val('');
 			$('#settings_smtp_password_confirm').val('');
 
 			switch($('#settings_how').val()) {
 			case '0':
+				$('#settings_sendmail_path').val('');
 				$('#row_settings_sendmail_header').hide();
 				$('#row_settings_sendmail_path').hide();
 				$('#row_settings_smtp_header').hide();
@@ -541,6 +712,10 @@ default:
 				$('#row_settings_smtp_timeout').hide();
 				break;
 			case '1':
+				if (smtpPath != '') {
+					$('#settings_sendmail_path').val(smtpPath);
+				}
+
 				$('#row_settings_sendmail_header').show();
 				$('#row_settings_sendmail_path').show();
 				$('#row_settings_smtp_header').hide();
@@ -552,6 +727,7 @@ default:
 				$('#row_settings_smtp_timeout').hide();
 				break;
 			case '2':
+				$('#settings_sendmail_path').val('');
 				$('#row_settings_sendmail_header').hide();
 				$('#row_settings_sendmail_path').hide();
 				$('#row_settings_smtp_header').show();
@@ -577,6 +753,8 @@ default:
 	function initBoostOD() {
 		if ($('#boost_rrd_update_enable').is(':checked')){
 			$('#row_boost_rrd_update_interval').show();
+			$('#row_boost_parallel').show();
+			$('#row_path_boost_log').show();
 			$('#row_boost_rrd_update_max_records').show();
 			$('#row_boost_rrd_update_max_records_per_select').show();
 			$('#row_boost_rrd_update_string_length').show();
@@ -585,6 +763,8 @@ default:
 			$('#row_boost_redirect').show();
 		} else {
 			$('#row_boost_rrd_update_interval').hide();
+			$('#row_boost_parallel').hide();
+			$('#row_path_boost_log').hide();
 			$('#row_boost_rrd_update_max_records').hide();
 			$('#row_boost_rrd_update_max_records_per_select').hide();
 			$('#row_boost_rrd_update_string_length').hide();
@@ -642,11 +822,23 @@ default:
 		if ($('#rrd_autoclean').is(':checked')) {
 			$('#row_rrd_autoclean_method').show();
 			if ($('#rrd_autoclean_method').val() == '3') {
+				if (rrdArchivePath != '') {
+					$('#rrd_archive').val(rrdArchivePath);
+				}
 				$('#row_rrd_archive').show();
 			} else {
+				if ($('#rrd_archive').val() != '') {
+					rrdArchivePath = $('#rrd_archive').val();
+				}
 				$('#row_rrd_archive').hide();
+				$('#rrd_archive').val('');
 			}
 		} else {
+			if ($('#rrd_archive').val() != '') {
+				rrdArchivePath = $('#rrd_archive').val();
+			}
+			$('#rrd_archive').val('');
+
 			$('#row_rrd_autoclean_method').hide();
 			$('#row_rrd_archive').hide();
 		}
@@ -735,6 +927,9 @@ default:
 			$('#row_ldap_search_filter').hide();
 			$('#row_ldap_specific_dn').hide();
 			$('#row_ldap_specific_password').hide();
+			$('#row_cn_header').hide();
+			$('#row_cn_full_name').hide();
+			$('#row_cn_email').hide();
 			$('#row_secpass_header').hide();
 			$('#row_secpass_minlen').hide();
 			$('#row_secpass_reqmixcase').hide();
@@ -774,6 +969,9 @@ default:
 			$('#row_ldap_search_filter').hide();
 			$('#row_ldap_specific_dn').hide();
 			$('#row_ldap_specific_password').hide();
+			$('#row_cn_header').hide();
+			$('#row_cn_full_name').hide();
+			$('#row_cn_email').hide();
 			$('#row_secpass_header').show();
 			$('#row_secpass_minlen').show();
 			$('#row_secpass_reqmixcase').show();
@@ -813,6 +1011,9 @@ default:
 			$('#row_ldap_search_filter').hide();
 			$('#row_ldap_specific_dn').hide();
 			$('#row_ldap_specific_password').hide();
+			$('#row_cn_header').hide();
+			$('#row_cn_full_name').hide();
+			$('#row_cn_email').hide();
 			$('#row_secpass_header').hide();
 			$('#row_secpass_minlen').hide();
 			$('#row_secpass_reqmixcase').hide();
@@ -852,6 +1053,9 @@ default:
 			$('#row_ldap_search_filter').hide();
 			$('#row_ldap_specific_dn').hide();
 			$('#row_ldap_specific_password').hide();
+			$('#row_cn_header').hide();
+			$('#row_cn_full_name').hide();
+			$('#row_cn_email').hide();
 			$('#row_secpass_header').show();
 			$('#row_secpass_minlen').show();
 			$('#row_secpass_reqmixcase').show();
@@ -891,6 +1095,9 @@ default:
 			$('#row_ldap_search_filter').show();
 			$('#row_ldap_specific_dn').show();
 			$('#row_ldap_specific_password').show();
+			$('#row_cn_header').show();
+			$('#row_cn_full_name').show();
+			$('#row_cn_email').show();
 			$('#row_secpass_header').show();
 			$('#row_secpass_minlen').show();
 			$('#row_secpass_reqmixcase').show();
@@ -932,6 +1139,9 @@ default:
 			$('#row_ldap_search_filter').hide();
 			$('#row_ldap_specific_dn').hide();
 			$('#row_ldap_specific_password').hide();
+			$('#row_cn_header').hide();
+			$('#row_cn_full_name').hide();
+			$('#row_cn_email').hide();
 			$('#row_secpass_header').show();
 			$('#row_secpass_minlen').show();
 			$('#row_secpass_reqmixcase').show();
@@ -980,72 +1190,6 @@ default:
 		}
 	}
 
-	function initFTPExport() {
-		switch($('#export_type').val()) {
-		case 'disabled':
-		case 'local':
-			$('#row_export_hdr_ftp').hide();
-			$('#row_export_ftp_sanitize').hide();
-			$('#row_export_ftp_host').hide();
-			$('#row_export_ftp_port').hide();
-			$('#row_export_ftp_passive').hide();
-			$('#row_export_ftp_user').hide();
-			$('#row_export_ftp_password').hide();
-			break;
-		case 'ftp_php':
-		case 'ftp_ncftpput':
-		case 'sftp_php':
-			$('#row_export_hdr_ftp').show();
-			$('#row_export_ftp_sanitize').show();
-			$('#row_export_ftp_host').show();
-			$('#row_export_ftp_port').show();
-			$('#row_export_ftp_passive').show();
-			$('#row_export_ftp_user').show();
-			$('#row_export_ftp_password').show();
-			break;
-		}
-	}
-
-	function initPresentation() {
-		switch($('#export_presentation').val()) {
-		case 'classical':
-			$('#row_export_tree_options').hide();
-			$('#row_export_tree_isolation').hide();
-			$('#row_export_tree_expand_hosts').hide();
-			break;
-		case 'tree':
-			$('#row_export_tree_options').show();
-			$('#row_export_tree_isolation').show();
-			$('#row_export_tree_expand_hosts').show();
-			break;
-		}
-	}
-
-	function initTiming() {
-		switch($('#export_timing').val()) {
-		case 'disabled':
-			$('#row_path_html_export_skip').hide();
-			$('#row_export_hourly').hide();
-			$('#row_export_daily').hide();
-			break;
-		case 'classic':
-			$('#row_path_html_export_skip').show();
-			$('#row_export_hourly').hide();
-			$('#row_export_daily').hide();
-			break;
-		case 'export_hourly':
-			$('#row_path_html_export_skip').hide();
-			$('#row_export_hourly').show();
-			$('#row_export_daily').hide();
-			break;
-		case 'export_daily':
-			$('#row_path_html_export_skip').hide();
-			$('#row_export_hourly').hide();
-			$('#row_export_daily').show();
-			break;
-		}
-	}
-
 	</script>
 	<?php
 
@@ -1054,3 +1198,13 @@ default:
 	break;
 }
 
+function is_remote_path_setting($field_name) {
+	global $config;
+
+	if ($config['poller_id'] > 1 && (strpos($field_name, 'path_') !== false || strpos($field_name, '_path') !== false)) {
+		return true;
+	} else {
+		return false;
+	}
+}
+
